-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Script file to create database and tablespaces for DB2 UDB 
--
-- ------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script:
-- @COG_DB_NAME@ = the database name, eg CSS
--
--    For a database install with enhanced security:
--    Remove the line "GRANT DBADM ON DATABASE TO @COG_DB_USER@" and follow
--    the instructions in the createTables.sql script.
-- 
-- @COG_DB_USER@ = database user for the cognos schema
--
-- Run the script in the DB2 command line processor with admin privileges.
--
-- USAGE:   db2 -tf createDatabase.sql
--
-- ------------------------------------------------------------------------------------
-- create the database
CREATE DATABASE @COG_DB_NAME@ AUTOMATIC STORAGE YES USING CODESET UTF-8 TERRITORY EN_US;
CHANGE DATABASE @COG_DB_NAME@ COMMENT WITH 'IBM Cognos Content Store';

-- connect to the created database:
CONNECT TO @COG_DB_NAME@;

-- The following grant is required by COGNOS for schema creation.
-- Following successful COGNOS initialization and initial schema creation,
-- the addition privileges can be removed by the following command:
-- REVOKE CREATETAB,IMPLICIT_SCHEMA ON DATABASE  FROM USER @DB_USER@;

GRANT CREATETAB,BINDADD,CONNECT,IMPLICIT_SCHEMA ON DATABASE  TO USER @COG_DB_USER@;


UPDATE DATABASE CONFIGURATION USING LOCKTIMEOUT 240 DEFERRED;

CREATE BUFFERPOOL CCS_08KBP IMMEDIATE SIZE 1000 PAGESIZE 8K;
CREATE BUFFERPOOL CCS_32KBP IMMEDIATE SIZE 1000 PAGESIZE 32K; 

CREATE SYSTEM TEMPORARY TABLESPACE TSN_SYS_CCS IN DATABASE PARTITION GROUP IBMTEMPGROUP PAGESIZE 32K BUFFERPOOL CCS_32KBP;
CREATE USER TEMPORARY TABLESPACE TSN_USR_CCS IN DATABASE PARTITION GROUP IBMDEFAULTGROUP PAGESIZE 8K BUFFERPOOL CCS_08KBP;
CREATE REGULAR TABLESPACE TSN_REG_CCS IN DATABASE PARTITION GROUP IBMDEFAULTGROUP PAGESIZE 8K BUFFERPOOL CCS_08KBP; 
GRANT USE OF TABLESPACE TSN_USR_CCS TO @COG_DB_USER@;
GRANT USE OF TABLESPACE TSN_REG_CCS TO @COG_DB_USER@;

CONNECT RESET;
