-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to create user and tablespaces for Oracle
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- @SCHEMA@ = the schema owner, eg COGNOS
-- @COG_DB_PASSWORD@ = the schema owner password
-- @TSDIR@ = the directory or prefix for tablespace datafiles, eg DEFAULTTS
--
--    For a database install with enhanced security:
--    Remove GRANT statments at the end of this script and follow the instructions
--    in the createTables.sql and additional information in the Monitor Info Center.
--
-- Run the script in SQL*Plus.
--
-- USAGE:  sqlplus system/passw0rd@orcl @createDatabase.sql
--
-- ------------------------------------------------------------------------------------

-------------------------
-- Create tablespaces 
-------------------------
DECLARE
  ts_exists INTEGER;
BEGIN
  SELECT COUNT(1) INTO ts_exists FROM DBA_TABLESPACES WHERE TABLESPACE_NAME = 'MONCOGNOS';
  IF NOT ts_exists = 1 THEN
      EXECUTE IMMEDIATE 'CREATE TABLESPACE MONCOGNOS
  DATAFILE ''@TSDIR@_MONCOGNOS.dbf'' SIZE 500M AUTOEXTEND ON NEXT 100M MAXSIZE UNLIMITED LOGGING';
  END IF;
END;
/

------------------------------------------------------------
-- Create schema owner if the user does not already exist --
------------------------------------------------------------
DECLARE
   user_exists INTEGER;
BEGIN
   SELECT COUNT(1) INTO user_exists FROM ALL_USERS WHERE USERNAME = '@SCHEMA@';
   IF NOT user_exists = 1 THEN
      EXECUTE IMMEDIATE 'CREATE USER @SCHEMA@ IDENTIFIED BY @COG_DB_PASSWORD@ DEFAULT TABLESPACE MONCOGNOS';
   END IF;
END;
/

GRANT CONNECT TO @SCHEMA@;
ALTER USER @SCHEMA@ QUOTA UNLIMITED ON MONCOGNOS;

-- The following grants are required by COGNOS for schema creation.
-- Following successful COGNOS initialization and initial schema creation,
-- the addition privileges can be removed by the following commands:

--  REVOKE CREATE USER FROM @SCHEMA@;
--  REVOKE ALTER USER FROM @SCHEMA@;
--  REVOKE DROP USER FROM @SCHEMA@;
	
--  REVOKE CREATE ANY TABLE FROM @SCHEMA@;
--  REVOKE ALTER ANY TABLE FROM @SCHEMA@;
--  REVOKE DROP ANY TABLE FROM @SCHEMA@;
	
--  REVOKE CREATE ANY SEQUENCE FROM @SCHEMA@;
--  REVOKE ALTER ANY SEQUENCE FROM @SCHEMA@;
--  REVOKE DROP ANY SEQUENCE FROM @SCHEMA@;
	
--  REVOKE CREATE ANY PROCEDURE FROM @SCHEMA@;
--  REVOKE ALTER ANY PROCEDURE FROM @SCHEMA@;
--  REVOKE DROP ANY PROCEDURE FROM @SCHEMA@;
	
--  REVOKE CREATE ANY VIEW FROM @SCHEMA@;
--  REVOKE DROP ANY VIEW FROM @SCHEMA@;
	
--  REVOKE CREATE ANY TRIGGER FROM @SCHEMA@;
--  REVOKE ALTER ANY TRIGGER FROM @SCHEMA@;
--  REVOKE DROP ANY TRIGGER FROM @SCHEMA@;
	
--  REVOKE CREATE ANY INDEX FROM @SCHEMA@;
--  REVOKE DROP ANY INDEX FROM @SCHEMA@;
	
--  REVOKE LOCK ANY TABLE FROM @SCHEMA@;

GRANT CREATE USER TO @SCHEMA@;
GRANT ALTER USER TO @SCHEMA@;
GRANT DROP USER TO @SCHEMA@;

GRANT SELECT ANY TABLE TO @SCHEMA@;
GRANT CREATE ANY TABLE TO @SCHEMA@;
GRANT ALTER ANY TABLE TO @SCHEMA@;
GRANT DROP ANY TABLE TO @SCHEMA@;
GRANT INSERT ANY TABLE TO @SCHEMA@;
GRANT UPDATE ANY TABLE TO @SCHEMA@;
GRANT DELETE ANY TABLE TO @SCHEMA@;

GRANT SELECT ANY SEQUENCE TO @SCHEMA@;
GRANT CREATE ANY SEQUENCE TO @SCHEMA@;
GRANT ALTER ANY SEQUENCE TO @SCHEMA@;
GRANT DROP ANY SEQUENCE TO @SCHEMA@;

GRANT EXECUTE ANY PROCEDURE TO @SCHEMA@;
GRANT CREATE ANY PROCEDURE TO @SCHEMA@;
GRANT ALTER ANY PROCEDURE TO @SCHEMA@;
GRANT DROP ANY PROCEDURE TO @SCHEMA@;

GRANT CREATE ANY VIEW TO @SCHEMA@;
GRANT DROP ANY VIEW TO @SCHEMA@;

GRANT CREATE ANY TRIGGER TO @SCHEMA@;
GRANT ALTER ANY TRIGGER TO @SCHEMA@;
GRANT DROP ANY TRIGGER TO @SCHEMA@;

GRANT CREATE ANY INDEX TO @SCHEMA@;
GRANT DROP ANY INDEX TO @SCHEMA@;

GRANT LOCK ANY TABLE TO @SCHEMA@;


QUIT


