-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2007, 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Script file to create database and tablespaces for DB2 UDB 
--
-- ------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script:
-- @DB_NAME@ = the database name, eg MONITOR
-- @TSDIR@ = the directory or prefix for tablespace datafiles, eg DEFAULTTS
--
--    For a database install with enhanced security:
--    Remove the line "GRANT DBADM ON DATABASE TO @DB_USER@" and follow
--    the instructions in the createTables.sql script.
-- 
-- @DB_USER@ = database user for the monitor schema
--
-- EN_US - The territory code for US English is used for creating the database.
--         In order to change the territory, replace EN_US with the territory
--         of choice.  For example, for French, repace EN_US with FR_FR.
--         
-- Run the script in the DB2 command line processor with admin privileges.
--
-- USAGE:   db2 -tf createDatabase.sql
--
-- ------------------------------------------------------------------------------------
-- create the database
CREATE DATABASE @DB_NAME@ AUTOMATIC STORAGE YES USING CODESET UTF-8 TERRITORY EN_US PAGESIZE 32768;

-- connect to the created database:
CONNECT TO @DB_NAME@;

--The following grant is used for databases without enhanced security, and can
--be removed by following the instructions in the createTables.sql script
--For more information, review the Info Center for Enhancing Security for DB2
GRANT DBADM ON DATABASE TO USER @DB_USER@;

------------- enable query optimizer MQT routing ------------------------
------------- to support the optional dimensional MQTs ---------------
UPDATE DB CFG USING DFT_MTTB_TYPES ALL;
UPDATE DB CFG USING DFT_REFRESH_AGE ANY;

------------- bufferpool for regular tablespaces
CREATE BUFFERPOOL BP32K IMMEDIATE SIZE 1000 PAGESIZE 32 K;

------------- bufferpool for temp tablespace
CREATE BUFFERPOOL TMPBP32K IMMEDIATE SIZE 1000 PAGESIZE 32 K;

------------- tablespace for DataServices (DS) 
CREATE REGULAR TABLESPACE DSTS32K PAGESIZE 32 K  
MANAGED BY SYSTEM  USING ('@TSDIR@_DSTS32K' ) 
EXTENTSIZE 8 OVERHEAD 10.5 PREFETCHSIZE 0 TRANSFERRATE 0.14 
BUFFERPOOL BP32K DROPPED TABLE RECOVERY OFF;

------------- tablespace for Data Mart Services (DMS)
CREATE REGULAR TABLESPACE DMSTS32K PAGESIZE 32 K  
MANAGED BY SYSTEM  USING ('@TSDIR@_DMSTS32K' ) 
EXTENTSIZE 8 OVERHEAD 10.5 PREFETCHSIZE 32 TRANSFERRATE 0.14 
BUFFERPOOL BP32K DROPPED TABLE RECOVERY OFF;

------------- tablespace (temp) for sorts/joins
CREATE SYSTEM TEMPORARY TABLESPACE TMPTS32K PAGESIZE 32 K  
MANAGED BY SYSTEM USING ('@TSDIR@_TMPTS32K' ) 
EXTENTSIZE 8 OVERHEAD 10.5 PREFETCHSIZE 32 TRANSFERRATE 0.14 
BUFFERPOOL TMPBP32K;

------------- user temporary tablespace (temp)
-- User Temporary Tablespace added for MQT Support
CREATE USER TEMPORARY TABLESPACE UTMPTS32K PAGESIZE 32 K  
MANAGED BY SYSTEM USING ('@TSDIR@_UTMPTS32K' ) 
EXTENTSIZE 8 OVERHEAD 10.5 PREFETCHSIZE 32 TRANSFERRATE 0.14 
BUFFERPOOL TMPBP32K;

CONNECT RESET;
