-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2009, 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
--
-- Process this script in the db2 command line processor. 
--
-- Example:
-- db2 -tvf dim_time_entries_db2_zos.sql
--
-- 
-- modify the values '2008-01-01' and 2023 in the below script to match the years needed for dimensional analysis
-- for example, if only 2011, 2012, and 2013 is needed for dimensional analysis, then change '2008-01-01' to '2011-01-01' and 2023 to 2014.
-- ------------------------------------------------------------------------------------
-- Following variable needs to be changed for customization before running this script:
-- @SCHEMA@ = the schema name, eg MONITOR


DELETE FROM @SCHEMA@.DIM_TIME;

INSERT INTO @SCHEMA@.DIM_TIME 
  (surrogate_key, year_num, month_num, day_num, month_name) 
WITH WBITIME (skey, ldate) as
(
   select 1 as skey, DATE('2008-01-01') as ldate
   from sysibm.sysdummy1

   UNION ALL
   
   SELECT parent.skey+1, ldate + 1 DAYS
   from WBITIME parent
   where YEAR(ldate + 1 days) < 2023
)
SELECT a.ldate, year(a.ldate), month(a.ldate), day(a.ldate), 
       char(month(a.ldate))
FROM WBITIME a;

INSERT INTO @SCHEMA@.DIM_TIME 
  (surrogate_key,year_num,month_num,day_num,month_name) 
VALUES (DATE('9999-01-01'),9999,1,1,'1');

-- The following UPDATE command can be used to override the default 
-- month names that are set above ('1', '2', etc).  
-- This UPDATE command can be edited with any location preference for 
-- month names (for example, upper case month names, 3-letter 
-- abbreviated names, French month names, etc).

UPDATE @SCHEMA@.DIM_TIME
   SET MONTH_NAME =
        CASE
                   WHEN MONTH_NUM = 1  THEN 'January'
                   WHEN MONTH_NUM = 2  THEN 'February'
                   WHEN MONTH_NUM = 3  THEN 'March'
                   WHEN MONTH_NUM = 4  THEN 'April'
                   WHEN MONTH_NUM = 5  THEN 'May'
                   WHEN MONTH_NUM = 6  THEN 'June'
                   WHEN MONTH_NUM = 7  THEN 'July'
                   WHEN MONTH_NUM = 8  THEN 'August'
                   WHEN MONTH_NUM = 9  THEN 'September'
                   WHEN MONTH_NUM = 10 THEN 'October'
                   WHEN MONTH_NUM = 11 THEN 'November'
                   WHEN MONTH_NUM = 12 THEN 'December'
                   ELSE 'Error'
                 END;