-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2007, 2012.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to create schema (tables) for Oracle
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- @SCHEMA@ = the schema owner, eg MONITOR
--
--    For a database install with enhanced security:
--    Scroll to the bottom of the file and un-comment the series of GRANT statements 
--    by replacing the string "--GRANT STMT--" with ""
-- 
-- @DB_USER@ = database user for the monitor schema, only needed for GRANT statements
--    for database installation with enhanced security
--
--
-- Run the script in SQL*Plus.
--
-- USAGE:  sqlplus system/passw0rd@orcl @createTables.sql
--
-- ------------------------------------------------------------------------------------

-------------------
-- Create tables --
-------------------

CREATE TABLE @SCHEMA@.SCHEMA_VERSION
(
  SCHEMA_VERSION                     NUMBER(10,0)        NOT NULL ,
  DATA_MIGRATION                     NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS 
  LOGGING;

ALTER TABLE @SCHEMA@.SCHEMA_VERSION ADD
(
  PRIMARY KEY ( SCHEMA_VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.MODERATOR_NEXT_ID  
(
  MODEL_ID                           VARCHAR2(256)       NOT NULL,
  UNIQUE_ID_KEY                      VARCHAR2(256)       NOT NULL , 
  START_ID_OF_NEXT_SET               NUMBER(20,0)        NOT NULL  
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.MODERATOR_NEXT_ID ADD
(
  PRIMARY KEY ( MODEL_ID, UNIQUE_ID_KEY )
  USING INDEX TABLESPACE MONIDXTS
);


CREATE TABLE @SCHEMA@.MM_TO_WPS_PT  (
  MODEL_ID           		     VARCHAR2(256)       NOT NULL,
  VERSION_ID                         NUMBER(20,0)        NOT NULL, 
  WPS_PT_ID                          VARCHAR2(512)       NOT NULL,
  WPS_PT_VALID_FROM                  VARCHAR2(128)       NOT NULL,
  PRIMARY KEY (MODEL_ID, VERSION_ID, WPS_PT_ID, WPS_PT_VALID_FROM)
      USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.RECORDED_EVENTS
(
  ID                                 NUMBER(20,0)        NOT NULL ,
  PAYLOAD_BLOB                       BLOB ,
  TIME_RECORDED                      TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
  BACKUP_ID                          VARCHAR2(32),
  PRIMARY KEY ( ID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE SEQUENCE @SCHEMA@.RECORDED_EVENTS_SEQ INCREMENT BY 1 START WITH 1 NOCYCLE ORDER;

CREATE TABLE @SCHEMA@.RECORDED_EVENTS_MODELVERSIONS
(
  EVENT_ID                           NUMBER(20,0)        NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  PRIMARY KEY ( EVENT_ID, MODEL_ID, VERSION ) USING INDEX TABLESPACE MONIDXTS,
  FOREIGN KEY ( EVENT_ID ) REFERENCES @SCHEMA@.RECORDED_EVENTS( ID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  GENERAL_PROPERTIES                 BLOB                         ,
  INTEGRATION_STATUS                 VARCHAR2(32)                 ,
  INTEGRATION_TYPE                   VARCHAR2(32)                 ,
  CEICELL                            VARCHAR2(128)                ,
  CEINODE                            VARCHAR2(128)                ,
  CEISERVER                          VARCHAR2(128)                ,
  CEICLUSTER                         VARCHAR2(128)                ,
  CEIHOST                            VARCHAR2(128)                ,
  CEIPORT                            VARCHAR2(128)                ,
  CEIUSER_ID                         VARCHAR2(128)                ,
  CEIPASSWORD                        VARCHAR2(128)                ,
  CEIEVENT_GROUP_PROFILE_LIST        VARCHAR2(128)                ,
  MEGA_NAME_MAPPER                   BLOB                         ,   
  SERVER_ID                          VARCHAR2(256)                ,
  FGS_LASTUPDATE_SECFILTER           TIMESTAMP                    ,
  FGS_LASTUPDATE_OBJSEC              TIMESTAMP                    ,
  FGS_LASTUPDATE_COGNOS              TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T ADD
(
  PRIMARY KEY ( MODEL_ID )
  USING INDEX TABLESPACE MONIDXTS
);  

CREATE TABLE @SCHEMA@.CEI_SERVER_SOURCE_T
(
  SERVER_ID                          VARCHAR2(256)       NOT NULL ,
  ALIAS                              VARCHAR2(128)                ,
  DESCRIPTION                        VARCHAR2(256)                ,
  DEFAULT_SERVER                     NUMBER(5,0)                  ,
  CELL                               VARCHAR2(128)                ,
  NODE                               VARCHAR2(128)                ,
  SERVER                             VARCHAR2(128)                ,
  CLUSTER_ID                         VARCHAR2(128)                ,
  HOST                               VARCHAR2(128)                ,
  RMI_PORT                           VARCHAR2(128)                ,
  EVENT_GROUP_PROFILE_LIST           VARCHAR2(128)                ,
  AUTH_ALIAS                         VARCHAR2(128)                ,
  QUEUE_BYPASS_CONFIG                NUMBER(5,0)                  ,
  QUEUE_BASED_CONFIG                 NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.CEI_SERVER_SOURCE_T ADD
(
  PRIMARY KEY ( SERVER_ID )
  USING INDEX TABLESPACE MONIDXTS
);


CREATE TABLE @SCHEMA@.META_MODEL_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  SCHEMA_NAME                        VARCHAR2(128)                ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  CURRENT_STATE                      VARCHAR2(32)                 ,
  INITIAL_STATE                      VARCHAR2(32)                 ,
  TARGET_STATE                       VARCHAR2(32)                 ,
  APPLICATION                        VARCHAR2(64)                 ,
  MODEL                              BLOB                         ,
  CUBES                              BLOB                         ,
  DIAGRAMS                           BLOB                         ,
  SCHEMA_DDLS                        BLOB                         ,
  DMS_DDLS                           BLOB                         ,
  GENERAL_PROPERTIES                 BLOB                         ,
  DEPLOYMENT_OPTIONS                 BLOB                         ,
  NAME_MAPPER                        BLOB                         ,
  UDF_JARS                           BLOB                         ,
  ROUTING_JARS                       BLOB                         ,
  ROOT_META_INF_CONTENTS             BLOB                         ,
  ABXHOST                            VARCHAR2(128)                ,
  ABXPORT                            NUMBER(10,0)                 ,
  ABXUSER_ID                         VARCHAR2(128)                ,
  ABXPASSWORD                        VARCHAR2(128)                ,
  INTEGRATION_STATUS                 VARCHAR2(32)                 ,
  REPLACEMENT_VERSION_DATE           NUMBER(20,0)                 ,
  IS_STOP_REQUESTED                  NUMBER(5,0)                  ,
  APPLICATION_UPDATE                 NUMBER(5,0)                  ,
  KPI_CACHE_REFRESH_INTERVAL         NUMBER(5,0)                  ,
  CODE_GEN_VERSION                   VARCHAR2(8)                  ,
  BUILD_ID			     VARCHAR2(40)		  ,
  COGNOS_PACKAGE		     VARCHAR2(1000)		  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MODEL_T ADD
(
  PRIMARY KEY ( MODEL_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_MODEL_STEP_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  STEP_NAME                          VARCHAR2(32)        NOT NULL ,
  LAST_SUCCESS_RUN_TIME              TIMESTAMP                    ,
  LAST_FAIL_RUN_TIME                 TIMESTAMP                    ,
  IS_COMPLETE                        NUMBER(5,0)                  ,
  MONITOR_VERSION                    VARCHAR2(32)                 ,
  MESSAGE                            BLOB                         ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MODEL_STEP_T ADD
(
  PRIMARY KEY ( MODEL_ID, VERSION, STEP_NAME )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE INDEX @SCHEMA@.META_MODEL_STEP_I1 
  ON @SCHEMA@.META_MODEL_STEP_T (STEP_NAME ASC, VERSION_ID ASC, 
  MONITOR_VERSION ASC, IS_COMPLETE ASC, LAST_FAIL_RUN_TIME ASC,
  LAST_SUCCESS_RUN_TIME ASC, MODEL_ID ASC, VERSION ASC)
  TABLESPACE MONIDXTS;


CREATE INDEX @SCHEMA@.META_MODEL_STEP_I2 
  ON @SCHEMA@.META_MODEL_STEP_T 
  (IS_COMPLETE ASC, STEP_NAME ASC, MODEL_ID ASC)
  TABLESPACE MONIDXTS;


CREATE TABLE @SCHEMA@.META_MONITOR_CONTEXT_T
(
  MCID                               VARCHAR2(256)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCPARENT_ID                        VARCHAR2(256)                ,
  DIAGRAM                            BLOB                         ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  CUBE_NAME                          VARCHAR2(1000)               ,
  TABLE_NAME                         VARCHAR2(128)       NOT NULL ,
  VIEW_NAME                          VARCHAR2(128)       NOT NULL ,
  DMS_TABLE_NAME                     VARCHAR2(128)       NOT NULL ,
  DMS_CHG_LOG_TABLE                  VARCHAR2(128)       NOT NULL ,  
  COGNOS_NAMESPACE                   VARCHAR2(1000)               ,  
  HIDDEN                             NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T ADD
(
  PRIMARY KEY ( MCID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

ALTER TABLE  @SCHEMA@.META_MONITOR_CONTEXT_T 
MODIFY	     HIDDEN
CONSTRAINT   META_MONITOR_CONTEXT_C1  NOT NULL;


CREATE TABLE @SCHEMA@.META_MONITOR_METRIC_T
(
  METRIC_ID                          VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  METRIC_TYPE                        VARCHAR2(14)        NOT NULL ,
  IS_SORTABLE                        NUMBER(5,0)                  ,
  COLUMN_NAME                        VARCHAR2(128)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  COGNOS_NAME                        VARCHAR2(1000)               ,
  HIDDEN                             NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T ADD
(
  PRIMARY KEY ( METRIC_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

ALTER TABLE  @SCHEMA@.META_MONITOR_METRIC_T 
MODIFY	     HIDDEN
CONSTRAINT   META_MONITOR_METRIC_C1  NOT NULL;


CREATE TABLE @SCHEMA@.META_DIMENSION_T
(
  DIMENSION_ID                       VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  TABLE_NAME                         VARCHAR2(128)                ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  ABX_DISPLAY_NAME                   VARCHAR2(1000)               ,  
  COGNOS_NAME                        VARCHAR2(1000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_DIMENSION_T ADD
(
  PRIMARY KEY ( DIMENSION_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_DIM_ATTR_T
(
  DIM_ATTRIBUTE_ID                   VARCHAR2(512)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DIMENSION_ID                       VARCHAR2(384)       NOT NULL ,
  ABX_DIM_ATTR_NAME                  VARCHAR2(1000)               ,  
  DIM_LEVEL                          NUMBER(10,0)        NOT NULL ,
  IS_PART_OF_DIM_KEY                 NUMBER(10,0)        NOT NULL ,
  METRIC_ID                          VARCHAR2(384)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  COGNOS_NAME                        VARCHAR2(1000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_DIM_ATTR_T ADD
(
  PRIMARY KEY ( DIM_ATTRIBUTE_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_MEASURE_T
(
  MEASURE_ID                         VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  ABX_DISPLAY_NAME                   VARCHAR2(1000)               ,  
  AGGREGATION_TYPE                   VARCHAR2(14)        NOT NULL ,
  TRACKING_KEY                       VARCHAR2(256)                ,
  MEASURE_DATA_TYPE                  VARCHAR2(14)                 ,
  SOURCE_METRIC_ID                   VARCHAR2(384)                ,
  COGNOS_NAME                        VARCHAR2(1000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_MEASURE_T ADD
(
  PRIMARY KEY ( MEASURE_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_CONTEXT_T
(
  KPI_CONTEXT_ID                     VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  DIAGRAM                            BLOB                         ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_CONTEXT_T ADD
(
  PRIMARY KEY ( KPI_CONTEXT_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  KPI_CONTEXT_ID                     VARCHAR2(256)                ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  DESCRIPTION                        VARCHAR2(256)                ,
  KPI_ORIGIN                         NUMBER(5,0)         NOT NULL ,
  KPI_DATA_TYPE                      NUMBER(5,0)         NOT NULL ,
  TARGET                             NUMBER                       ,
  KPI_RANGE_TYPE                     NUMBER(5,0)         NOT NULL ,
  KPI_CALC_METHOD                    NUMBER(5,0)         NOT NULL ,
  AGGREGATED_MEASURE_ID              VARCHAR2(384)                ,
  AGGREGATED_METRIC_ID               VARCHAR2(384)                ,
  AGGREGATION_FUNCTION               VARCHAR2(10)                 ,
  VERSION_AGGREGATION                NUMBER(5,0)                  ,
  TIME_PERIOD_METRIC_ID              VARCHAR2(384)                ,
  TIME_PERIOD_METHOD                 NUMBER(5,0)                  ,
  REPEATING_PERIOD_DURATION          NUMBER(5,0)                  ,
  REPEATING_PERIOD_BASIS             NUMBER(5,0)                  ,
  REPEATING_PERIOD_TIMEZONE          VARCHAR2(30)                 ,
  ROLLING_PERIOD_DURATION            NUMBER(5,0)                  ,
  ROLLING_PERIOD_QUANTITY            NUMBER(10,0)                 ,
  FIXED_PERIOD_START                 TIMESTAMP                    ,
  FIXED_PERIOD_END                   TIMESTAMP                    ,
  FIXED_PERIOD_TIMEZONE              VARCHAR2(30)                 ,
  CALCULATED_KPI_EXPRESSION          CLOB                         ,
  USER_ID                            VARCHAR2(256)                ,
  VIEW_ACCESS                        NUMBER(5,0)         NOT NULL ,
  FORMAT_DECIMAL_PRECISION           NUMBER(5,0)                  ,
  FORMAT_CURRENCY                    VARCHAR2(3)                  ,
  FORMAT_PERCENTAGE                  NUMBER(5,0)         NOT NULL ,
  ENABLE_KPI_HISTORY                 NUMBER(5,0)                  ,
  ENABLE_KPI_PREDICTIONS             NUMBER(5,0)                  ,
  HISTORY_VALID_FROM                 TIMESTAMP                    ,
  KPI_CACHE_VALUE                    NUMBER                       ,
  KPI_LAST_CALCULATION_TIME          TIMESTAMP                    ,
  KPI_CACHE_OVERRIDE_INTERVAL        NUMBER(5,0)                  ,
  HIST_GRANULARITY                   NUMBER(5,0)                  ,
  HIST_REPEATING_PERIOD_BASIS        NUMBER(5,0)                  ,
  HIST_REPEATING_PERIOD_DURATION     NUMBER(5,0)                  ,
  HIST_REPEATING_PERIOD_QUANTITY     NUMBER(10,0)                 ,
  HIST_ROLLING_PERIOD_DURATION       NUMBER(5,0)                  ,
  HIST_ROLLING_PERIOD_QUANTITY       NUMBER(10,0)                 ,
  HIST_TIME_RANGE_START              TIMESTAMP                    ,
  HIST_TIME_RANGE_END                TIMESTAMP                    ,
  HIST_TIMEZONE                      VARCHAR2(30)                 ,
  HIST_INCLUDE_PREDICTIONS           NUMBER(5,0)                  ,
  HIST_ALL_VERSIONS                  NUMBER(5,0)                  ,
  HIST_TIME_RANGE_METHOD             NUMBER(5,0)                  ,
  HIST_DISPLAY_TARGET                NUMBER(5,0)                  ,
  HIST_DISPLAY_RANGES                NUMBER(5,0)                  ,
  HIST_RETRO_ACTIVE_CALC_QTY         NUMBER(10,0)                 ,
  DEFAULT_PREDICTION_MODEL_ID        VARCHAR2(384)                ,
  HIDDEN                             NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING
  LOB (CALCULATED_KPI_EXPRESSION) STORE AS
  (
    TABLESPACE MONLOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE @SCHEMA@.META_KPI_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

ALTER TABLE  @SCHEMA@.META_KPI_T 
MODIFY	     HIDDEN
CONSTRAINT   META_KPI_C1  NOT NULL;

CREATE TABLE @SCHEMA@.META_KPI_PERSONALIZED_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  USER_ID                            VARCHAR2(256)       NOT NULL ,
  TARGET                             NUMBER                       ,
  FGS_FILTER_HASH                    NUMBER(20,0)                 ,
  ENABLE_KPI_HISTORY                 NUMBER(5,0)         NOT NULL ,
  ENABLE_KPI_PREDICTIONS             NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_PERSONALIZED_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, USER_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_CACHE_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  FGS_FILTER_HASH                    NUMBER(20,0)        NOT NULL ,
  KPI_CACHE_VALUE                    NUMBER                       ,
  KPI_LAST_CALCULATION_TIME          TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_CACHE_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, FGS_FILTER_HASH )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_METRIC_FILTER_T
(
  KPI_METRIC_FILTER_ID               VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  FILTER_METRIC_ID                   VARCHAR2(512)       NOT NULL ,
  FILTER_OPERATOR                    NUMBER(5,0)         NOT NULL ,
  FILTER_OPERATOR_CASE_SENSITIVE     NUMBER(5,0)         NOT NULL ,
  FILTER_VALUE                       CLOB                         ,
  VALUE_INDEX                        NUMBER(10,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING
  LOB (FILTER_VALUE) STORE AS
  (
    TABLESPACE MONLOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE @SCHEMA@.META_KPI_METRIC_FILTER_T ADD
(
  PRIMARY KEY ( KPI_METRIC_FILTER_ID, VERSION, VALUE_INDEX )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_RANGE_T
(
  KPI_RANGE_ID                       VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  START_VALUE                        NUMBER              NOT NULL ,
  END_VALUE                          NUMBER              NOT NULL ,
  COLOR                              VARCHAR2(7)                  ,
  ICON                               VARCHAR2(256)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_RANGE_T ADD
(
  PRIMARY KEY ( KPI_RANGE_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T
(
  KPI_RANGE_ID                       VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  USER_ID                            VARCHAR2(256)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  START_VALUE                        NUMBER              NOT NULL ,
  END_VALUE                          NUMBER              NOT NULL ,
  COLOR                              VARCHAR2(7)                  ,
  ICON                               VARCHAR2(256)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T ADD
(
  PRIMARY KEY ( KPI_RANGE_ID, VERSION, USER_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_DEPENDENCY_T
(
  PARENT_KPI_ID                      VARCHAR2(384)       NOT NULL ,
  PARENT_KPI_VERSION                 NUMBER(20,0)        NOT NULL ,
  CHILD_KPI_ID                       VARCHAR2(384)       NOT NULL ,
  CHILD_KPI_VERSION                  NUMBER(20,0)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_DEPENDENCY_T ADD
(
  PRIMARY KEY ( PARENT_KPI_ID, PARENT_KPI_VERSION, CHILD_KPI_ID, CHILD_KPI_VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.KPI_HISTORY_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  KPI_DYNAMIC_FILTER_VALUE           VARCHAR2(500)       NOT NULL ,
  PERIOD_TIMESTAMP                   TIMESTAMP           NOT NULL ,
  FGS_FILTER_HASH                    NUMBER(20,0)  DEFAULT 0 NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  OBSERVED_TIME                      TIMESTAMP                    ,
  OBSERVED_VALUE                     NUMBER                       ,
  TARGET                             NUMBER                       ,
  HISTORY_UPDATE_TIME                TIMESTAMP                    ,
  HISTORY_VALUE                      NUMBER                       ,
  PREDICTION_BASIS_VALUE             NUMBER                       ,
  PREDICTION_BASIS_TIME              TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.KPI_HISTORY_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, KPI_DYNAMIC_FILTER_VALUE, PERIOD_TIMESTAMP, FGS_FILTER_HASH )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.KPI_TS_PREDICTION_T
(
  KPI_PREDICTION_ID                  VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  PREDICTION_MODEL_ID                VARCHAR2(384)       NOT NULL ,
  FGS_FILTER_HASH                    NUMBER(20,0)   DEFAULT 0 NOT NULL ,
  PREDICTED_FOR_TIME                 TIMESTAMP                    ,
  PREDICTED_VALUE                    NUMBER                       ,
  LAST_KPI_TIME                      TIMESTAMP           NOT NULL ,
  STANDARD_DEVIATION                 NUMBER                       ,
  VALUE_PLUS_STDDEV                  NUMBER                       ,
  VALUE_MINUS_STDDEV                 NUMBER                       ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T ADD
(
  PRIMARY KEY ( KPI_PREDICTION_ID, VERSION, LAST_KPI_TIME, FGS_FILTER_HASH )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.KPI_TS_PREDICTION_MODEL_T
(
  PREDICTION_MODEL_ID                VARCHAR2(384)       NOT NULL ,
  PREDICTION_MODEL_NAME              VARCHAR2(256)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  MODEL_ID                           VARCHAR2(128)                ,
  KPI_DYNAMIC_FILTER_VALUE           VARCHAR2(500)                ,
  MODEL_XML                          BLOB                         ,
  PREDICTION_HORIZON                 NUMBER(10,0)                 ,
  PREDICT_TO_PERIOD_END              NUMBER(5,0)         NOT NULL ,
  PREDICTION_INTERVAL_NUM            NUMBER(10,0)                 ,
  PREDICTION_INTERVAL_UNIT           NUMBER(5,0)                  ,
  INTERVALS_PER_CYCLE                NUMBER(10,0)                 ,
  CYCLE_LENGTH_UNIT                  NUMBER(5,0)                  ,
  KPI_BEHAVIOR                       NUMBER(5,0)                  ,
  PREDICTION_SUBMODEL_ID             VARCHAR2(384)                ,
  MODEL_OPTIMIZE_SCHEDULE            NUMBER(5,0)                  ,
  CREATION_USER_ID                   VARCHAR2(256)                ,
  UPDATE_USER_ID                     VARCHAR2(256)                ,
  NEXT_EVAL_TIME                     TIMESTAMP                    ,
  NEXT_OPTIMIZE_TIME                 TIMESTAMP                    ,
  CREATION_TIME                      TIMESTAMP                    ,
  UPDATE_TIME                        TIMESTAMP                    ,
  INITIAL_KPI_TIME                   TIMESTAMP                    ,
  LAST_KPI_TIME                      TIMESTAMP                    ,
  PREDICTION_LAST_RUN                TIMESTAMP                    ,
  TIME_LAST_OPTIMIZED                TIMESTAMP                    ,
  IS_VALID                           NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_MODEL_T ADD
(
  PRIMARY KEY ( PREDICTION_MODEL_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.DMS_METADATA_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  DMS_COPY_PROC                      VARCHAR2(128)                ,
  TARGET_TABLE                       VARCHAR2(128)                ,
  LAST_TIME_STARTED                  NUMBER(20,0)                 ,
  LAST_TIME_CMPLTD                   NUMBER(20,0)                 ,
  DMS_INTERVAL                       NUMBER(20,0)                 ,
  PROGRESS_FLAG                      NUMBER(5,0)                  ,
  IS_NEW_FLAG                        NUMBER(5,0)                  ,
  SUSPENDED_FLAG                     NUMBER(5,0)                  ,
  IS_DELETED_FLAG                    NUMBER(5,0)                  ,
  INVOKE_NOW                         NUMBER(5,0)                  ,
  ROWS_COPIED                        NUMBER(10,0)                 ,
  ROWS_PRUNED                        NUMBER(10,0)                 ,
  DMS_COPY_PROC_MESG                 VARCHAR2(4000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.DMS_METADATA_T ADD
(
  PRIMARY KEY ( MODEL_ID, MCID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.DMS_HISTORY_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  TIME_STARTED                       NUMBER(20,0)        NOT NULL ,
  TIME_COMPLETED                     NUMBER(20,0)                 ,
  DMS_INTERVAL                       NUMBER(20,0)                 ,
  ROWS_COPIED                        NUMBER(10,0)                 ,
  ROWS_PRUNED                        NUMBER(10,0)                 ,
  DMS_COPY_PROC_MESG                 VARCHAR2(4000)               ,
  IS_DELETED_FLAG                    NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.DMS_HISTORY_T ADD
(
  PRIMARY KEY ( MODEL_ID, MCID, VERSION, TIME_STARTED )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.DMS_TRACE_T
(
DMS_STORED_PROC                      VARCHAR2(128)                ,
START_TIME                           TIMESTAMP                    ,
END_TIME                             TIMESTAMP                    ,
ROWS_MERGED                          NUMBER(10,0)                 ,
ROWS_PRUNED                          NUMBER(10,0)                 ,
START_DCL                            NUMBER(10,0)                 ,
START_DCC                            NUMBER(10,0)                 ,
END_DCL                              NUMBER(10,0)                 ,
END_DCC                              NUMBER(10,0)                 ,
START_MCT                            NUMBER(10,0)                 ,
START_TGT                            NUMBER(10,0)                 ,
END_MCT                              NUMBER(10,0)                 ,
END_TGT                              NUMBER(10,0)                 ,
ROWS_INSERTED                        NUMBER(10,0)                 ,
ROWS_UPDATED                         NUMBER(10,0) 
) TABLESPACE MONDSTS
  LOGGING;


CREATE INDEX @SCHEMA@.DMS_TRACE_I1 ON @SCHEMA@.DMS_TRACE_T (DMS_STORED_PROC, START_TIME) TABLESPACE MONIDXTS;


CREATE TABLE @SCHEMA@.META_OUTBOUND_EVENT_T
(
  MODEL_ID                           VARCHAR2(128)                      NOT NULL ,
  EVENT_ID                           VARCHAR2(384)                      NOT NULL ,
  CONTEXT_ID                         VARCHAR2(256)                      NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)                              ,
  EXTENSION_NAME                     VARCHAR2(1000)                              ,
  CONTEXT_TYPE                       VARCHAR2(100)                               ,
  VERSION_ID                         NUMBER(5,0)                         NOT NULL
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_OUTBOUND_EVENT_T ADD
(
  PRIMARY KEY ( EVENT_ID, VERSION )
  USING INDEX TABLESPACE MONIDXTS
);



CREATE TABLE @SCHEMA@.ACT_MGR_ALERTS_T
(
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  CONTEXT_DEF                        VARCHAR2(300)                ,
  CONTEXT_INST_ID                    VARCHAR2(32)                 ,
  MODEL_NAME                         VARCHAR2(300)                ,
  MODEL_ID                           VARCHAR2(128)                ,
  CONTEXT_NAME                       VARCHAR2(300)                ,
  CONTEXT_ID                         VARCHAR2(256)                ,
  MODEL_VERSION                      VARCHAR2(128)                ,
  BUS_SIT_NAME                       VARCHAR2(384)       NOT NULL ,
  CBE_XML                            BLOB                         ,
  ALERT_SUBJECT                      VARCHAR2(240)       NOT NULL ,
  ALERT_BODY                         CLOB                NOT NULL ,
  ALERT_TIMESTAMP                    TIMESTAMP                    ,
  PRIORITY                           NUMBER(20,0)                 ,
  STATUS                             VARCHAR2(32)                 ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING
  LOB (ALERT_BODY) STORE AS
  (
    TABLESPACE MONLOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE @SCHEMA@.ACT_MGR_ALERTS_T ADD
(
  PRIMARY KEY ( ALERT_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ACT_ALERT_COMMENTS_T
(
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  COMMENT_OWNER                      VARCHAR2(32)        NOT NULL ,
  COMMENT_TIME                       TIMESTAMP           NOT NULL ,
  COMMENT_TEXT                       VARCHAR2(4000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_ALERT_COMMENTS_T ADD
(
  PRIMARY KEY ( ALERT_ID, COMMENT_OWNER, COMMENT_TIME )
  USING INDEX TABLESPACE MONIDXTS
);



CREATE TABLE @SCHEMA@.DASHBOARD_ALERT_T
(
  USER_ID                            VARCHAR2(32)        NOT NULL ,
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  ACKNOWLEDGED                       NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.DASHBOARD_ALERT_T ADD
(
  PRIMARY KEY ( USER_ID, ALERT_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE INDEX @SCHEMA@.DASHBOARD_ALERT_I1 ON @SCHEMA@.DASHBOARD_ALERT_T (USER_ID) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.ACT_HNDLR_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  TYPE                               VARCHAR2(64)        NOT NULL ,
  INVOCATION_TYPE                    VARCHAR2(20)                 ,
  INVOCATION_PROFILE                 VARCHAR2(1024)               ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_HNDLR_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ACTION_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  TYPE                               VARCHAR2(64)        NOT NULL ,
  PARAMETER_SET                      BLOB                         ,
  STATE                              VARCHAR2(20)                 ,
  ALIAS_NAME                         VARCHAR2(64)                 ,
  TEMPLATE_ID                        VARCHAR2(384)                ,
  HANDLER_ID                         VARCHAR2(32)                 ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACTION_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ACTION_HANDLER_CTRL_T
(
  ACTION_ID                          VARCHAR2(384)       NOT NULL ,
  HANDLER_ID                         VARCHAR2(32)        NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACTION_HANDLER_CTRL_T ADD
(
  PRIMARY KEY ( ACTION_ID, HANDLER_ID )
  USING INDEX TABLESPACE MONIDXTS
);


ALTER TABLE @SCHEMA@.ACTION_HANDLER_CTRL_T
  ADD CONSTRAINT D_ACTION_ID FOREIGN KEY (ACTION_ID)
  REFERENCES @SCHEMA@.ACTION_T(ID) ON DELETE CASCADE;
    
CREATE INDEX @SCHEMA@.ACTN_HNDLR_CTRL_I1 ON @SCHEMA@.ACTION_HANDLER_CTRL_T (ACTION_ID) TABLESPACE MONIDXTS;


CREATE TABLE @SCHEMA@.PLUG_SVC_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  NAME                               VARCHAR2(64)        NOT NULL ,
  SERVICE_TYPE                       VARCHAR2(20)        NOT NULL ,
  SUB_TYPE                           VARCHAR2(20)        NOT NULL ,
  ACTION_HANDLER_ID                  VARCHAR2(32)        NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.PLUG_SVC_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ACT_TMPLT_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  NAME                               VARCHAR2(384)       NOT NULL ,
  TYPE                               VARCHAR2(20)        NOT NULL ,
  CONTENT                            BLOB                NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_TMPLT_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE INDEX @SCHEMA@.ACT_TMPLT_I1 ON @SCHEMA@.ACT_TMPLT_T (NAME) TABLESPACE MONIDXTS;
CREATE INDEX @SCHEMA@.ACT_TMPLT_I2 ON @SCHEMA@.ACT_TMPLT_T (TYPE) TABLESPACE MONIDXTS;


CREATE TABLE @SCHEMA@.SUBSCRIBED_USER_T
(
  USER_ID                            VARCHAR2(32)        NOT NULL ,
  ETAACTRL_ID                        VARCHAR2(384)       NOT NULL ,
  DASHBOARD                          NUMBER(5,0)         NOT NULL ,
  EMAIL                              NUMBER(5,0)         NOT NULL ,
  CELL                               NUMBER(5,0)         NOT NULL ,
  PAGER                              NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.SUBSCRIBED_USER_T ADD
(
  PRIMARY KEY ( USER_ID, ETAACTRL_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE INDEX @SCHEMA@.SUBSCRIBED_USER_I1 ON @SCHEMA@.SUBSCRIBED_USER_T (USER_ID) TABLESPACE MONIDXTS;
CREATE INDEX @SCHEMA@.SUBSCRIBED_USER_I2 ON @SCHEMA@.SUBSCRIBED_USER_T (ETAACTRL_ID) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.ETAMATCH_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  DETERMINER_TYPE                    VARCHAR2(20)        NOT NULL ,
  EXTENSION_NAME                     VARCHAR2(384)       NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ETAMATCH_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);
CREATE INDEX @SCHEMA@.ETAMATCH_I1 ON @SCHEMA@.ETAMATCH_T (EXTENSION_NAME) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.ATAMATCH_T
(
  ID                                 VARCHAR2(32)        NOT NULL ,
  DETERMINER_TYPE                    VARCHAR2(20)        NOT NULL ,
  PARENT_ACTION_ID                   VARCHAR2(32)        NOT NULL ,
  CHILD_ACTION_ID                    VARCHAR2(32)        NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ATAMATCH_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ETAACTRL_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  ETAMID                             VARCHAR2(384)       NOT NULL ,
  ACTION_ID                          VARCHAR2(384)       NOT NULL ,
  NAME                               VARCHAR2(1000)      NOT NULL ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  BINDING_TIMESTAMP                  TIMESTAMP                    ,
  CATEGORY                           VARCHAR2(264)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ETAACTRL_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);
CREATE INDEX @SCHEMA@.ETAACTRL_I1 ON @SCHEMA@.ETAACTRL_T (ETAMID) TABLESPACE MONIDXTS;
CREATE INDEX @SCHEMA@.ETAACTRL_I2 ON @SCHEMA@.ETAACTRL_T (ACTION_ID) TABLESPACE MONIDXTS;
CREATE INDEX @SCHEMA@.ETAACTRL_I3 ON @SCHEMA@.ETAACTRL_T (ETAMID, ACTION_ID) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.ACTION_MANAGER_PROP_T
(
  ACTION_PROP_NAME                   VARCHAR2(100)       NOT NULL ,
  ACTION_PROP_VALUE                  VARCHAR2(240)       NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACTION_MANAGER_PROP_T ADD
(
  PRIMARY KEY ( ACTION_PROP_NAME )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.EQ_MODEL_VERSION_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MODEL                              VARCHAR2(1000)                     NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  LAST_SUB_INST                      VARCHAR2(32)                                ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;
CREATE INDEX @SCHEMA@.EQ_MODEL_VER_I1 ON @SCHEMA@.EQ_MODEL_VERSION_T (MODEL, VERSION) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.DQ_MODEL_VERSION_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MODEL                              VARCHAR2(1000)                     NOT NULL ,
  VERSION                            NUMBER(20,0)                       NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;
  
CREATE INDEX @SCHEMA@.DQ_MODEL_VER_I1 ON @SCHEMA@.DQ_MODEL_VERSION_T (MODEL, VERSION) TABLESPACE MONIDXTS;

CREATE INDEX @SCHEMA@.EQ_MODEL_VER_I2
  ON @SCHEMA@.EQ_MODEL_VERSION_T 
  (VERSION ASC, MODEL ASC, DBID ASC)
  TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.EQ_INSTANCE_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MV_DBID                            VARCHAR2(32)                       NOT NULL ,
  ROOT_INST_ID                       VARCHAR2(1000)                     NOT NULL ,
  FAIL_TIME                          NUMBER(20,0)                                ,
  LAST_SUB_TIME                      NUMBER(20,0)                                ,
  LAST_SUB_STATUS                    NUMBER(10,0)                       NOT NULL ,
  LAST_SEQUENCE_NUM                  NUMBER(20,0)                       NOT NULL ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ) USING INDEX TABLESPACE MONIDXTS,
  FOREIGN KEY ( MV_DBID ) REFERENCES @SCHEMA@.EQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

CREATE INDEX @SCHEMA@.EQ_INSTANCE_I1 
  ON @SCHEMA@.EQ_INSTANCE_T 
  (ROOT_INST_ID ASC, DBID ASC, MV_DBID ASC)
  TABLESPACE MONIDXTS;
  
  
CREATE INDEX @SCHEMA@.EQ_INSTANCE_I2 
  ON @SCHEMA@.EQ_INSTANCE_T 
  (ROOT_INST_ID ASC, LAST_SUB_STATUS ASC, LAST_SUB_TIME ASC, 
  LAST_SEQUENCE_NUM ASC, FAIL_TIME ASC, DBID ASC, MV_DBID ASC)
  TABLESPACE MONIDXTS;


CREATE TABLE @SCHEMA@.EQ_FAILED_EVENT_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  INSTANCE_DBID                      VARCHAR2(32)                       NOT NULL ,
  GU_INST_ID                         VARCHAR2(1000)                              ,
  EQ_SEQUENCE                        NUMBER(20,0)                       NOT NULL ,
  EVENT_SEQUENCE                     VARCHAR2(512)                               ,
  F_SUMMARY                          VARCHAR2(256)                      NOT NULL ,
  F_DETAILS                          BLOB                                        ,
  F_TIME                             NUMBER(20,0)                       NOT NULL ,
  CREATION_TIME                      NUMBER(20,0)                                ,
  EXT_NAME                           VARCHAR2(64)                                ,
  CONSUMPTION_TIME                   NUMBER(20,0)                       NOT NULL ,
  LAST_SUB_TIME                      NUMBER(20,0)                                ,
  LAST_SUB_STATUS                    NUMBER(10,0)                       NOT NULL ,
  HISTORY                            BLOB                                        ,
  DATA                               BLOB                                        ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ) USING INDEX TABLESPACE MONIDXTS,
  FOREIGN KEY ( INSTANCE_DBID ) REFERENCES @SCHEMA@.EQ_INSTANCE_T( DBID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;
CREATE INDEX @SCHEMA@.EQ_FAILED_EVENT_I1 ON @SCHEMA@.EQ_FAILED_EVENT_T (INSTANCE_DBID) TABLESPACE MONIDXTS;
CREATE INDEX @SCHEMA@.EQ_FAILED_EVENT_I2 ON @SCHEMA@.EQ_FAILED_EVENT_T (INSTANCE_DBID, EQ_SEQUENCE) TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.DQ_DEAD_EVENT_T
(
  DBID                               VARCHAR2(32)                       NOT NULL ,
  MV_DBID                            VARCHAR2(32)                       NOT NULL ,
  F_SUMMARY                          VARCHAR2(256)                      NOT NULL ,
  F_DETAILS                          BLOB                                        ,
  F_TIME                             NUMBER(20,0)                       NOT NULL ,
  DATA                               BLOB                                        ,
  VERSION_ID                         NUMBER(5,0)                        NOT NULL ,
  PRIMARY KEY ( DBID ) USING INDEX TABLESPACE MONIDXTS,
  FOREIGN KEY ( MV_DBID ) REFERENCES @SCHEMA@.DQ_MODEL_VERSION_T( DBID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;
CREATE INDEX @SCHEMA@.DQ_DEAD_EVENT_I1 ON @SCHEMA@.DQ_DEAD_EVENT_T (MV_DBID) TABLESPACE MONIDXTS;


CREATE SEQUENCE @SCHEMA@.ERROR_Q_SEQ INCREMENT BY 1 START WITH 1 NOCYCLE ORDER;

--
-- Dynamic Alerts tables
--

CREATE TABLE @SCHEMA@.ACT_SITUATION_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)      NOT NULL ,
  DESCRIPTION                        VARCHAR2(256)                ,
  ACTIVE                             NUMBER(20,0)        NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  REPEATING                          NUMBER(5,0)         NOT NULL ,
  INTERVAL                           NUMBER(5,0)         NOT NULL ,
  TIME_POINT                         NUMBER(20,0)        NOT NULL ,
  TIME_MULTIPLE                      NUMBER(20,0)        NOT NULL ,
  TIME_OFFSET                        NUMBER(20,0)        NOT NULL ,
  TIME_ZONE                          VARCHAR2(60)        NOT NULL ,
  KPI_ID                             VARCHAR2(384)                ,
  TIME_PROPERTIES                    CLOB                         ,
  START_TIME                         TIMESTAMP           NOT NULL ,
  LAST_EVAL                          TIMESTAMP                    ,
  NEXT_EVAL                          TIMESTAMP           NOT NULL ,
  SIT_TRIGGERED                      NUMBER(5,0)         NOT NULL ,
  USER_ID                            VARCHAR2(256)       NOT NULL ,
  VIEW_ACCESS                        NUMBER(5,0)         NOT NULL ,
  CREATE_TIME                        TIMESTAMP           NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_SITUATION_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.ACT_SIT_TRIG_T
(
  ID                                 VARCHAR2(384)       NOT NULL ,
  SITUATION_ID                       VARCHAR2(384)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  PREDICTION_MODEL_ID                VARCHAR2(384)                ,
  PREDICTION_TYPE                    VARCHAR2(20)                 ,
  TYPE                               VARCHAR2(256)       NOT NULL ,
  FIELD_TYPE                         VARCHAR2(256)       NOT NULL ,
  FIELD                              VARCHAR2(256)                ,
  COND_SETUP                         NUMBER(5,0)                  ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_SIT_TRIG_T ADD
(
  PRIMARY KEY ( ID )
  USING INDEX TABLESPACE MONIDXTS
);

--
-- dim time
--

CREATE TABLE @SCHEMA@.DIM_TIME
(
  SURROGATE_KEY                      DATE                NOT NULL ,
  DAY_NUM                            NUMBER(5,0)         NOT NULL ,
  MONTH_NUM                          NUMBER(5,0)         NOT NULL ,
  YEAR_NUM                           NUMBER(10,0)        NOT NULL ,
  MONTH_NAME                         VARCHAR2(27)        NOT NULL
) TABLESPACE MONDSTS
  LOGGING;

DECLARE
  start_date DATE;
  sk NUMBER;
  
  end_date DATE;
BEGIN
  start_date := to_date('01012008','DDMMYYYY');
  end_date := to_date('31122022','DDMMYYYY');
  DELETE FROM @SCHEMA@.DIM_TIME;
  WHILE(start_date <= end_date) LOOP
    INSERT INTO @SCHEMA@.DIM_TIME(surrogate_key,year_num,month_num,day_num,month_name) values (start_date,to_number(to_char(start_date,'YYYY')),to_number(to_char(start_date,'MM')),to_number(to_char(start_date,'DD')),to_char(start_date,'MONTH'));
    start_date := start_date + 1;
  END LOOP;
  INSERT INTO @SCHEMA@.DIM_TIME(surrogate_key,year_num,month_num,day_num,month_name) values (TO_DATE('01019999','DDMMYYYY'),9999,1,1,to_char(TO_DATE('01019999','DDMMYYYY'),'MONTH'));
END;
.
/

ALTER TABLE @SCHEMA@.DIM_TIME ADD
(
  PRIMARY KEY ( SURROGATE_KEY )
  USING INDEX TABLESPACE MONIDXTS
);

-- The following UPDATE command can be used to override the default month names that are set 
-- above with TO_CHAR(date,'MONTH'), which returns an upper case character string (for example, 
-- JANUARY), based on the locale of the database. 
-- This UPDATE command can be edited with any location preference for month names (for example,
-- mixed case month names, 3-letter abbreviated month names, French month names, etc).  Names 
-- in different locales can also be had with TO_CHAR(date,'MONTH','nls_date_language = FRENCH').
-- This UPDATE command can be uncommented here or it can be run separately in SQLPlus.
--
-- UPDATE @SCHEMA@.DIM_TIME
--   SET MONTH_NAME =
--        CASE
--                   WHEN MONTH_NUM = 1  THEN 'January'
--                   WHEN MONTH_NUM = 2  THEN 'February'
--                   WHEN MONTH_NUM = 3  THEN 'March'
--                   WHEN MONTH_NUM = 4  THEN 'April'
--                   WHEN MONTH_NUM = 5  THEN 'May'
--                   WHEN MONTH_NUM = 6  THEN 'June'
--                   WHEN MONTH_NUM = 7  THEN 'July'
--                   WHEN MONTH_NUM = 8  THEN 'August'
--                   WHEN MONTH_NUM = 9  THEN 'September'
--                   WHEN MONTH_NUM = 10 THEN 'October'
--                   WHEN MONTH_NUM = 11 THEN 'November'
--                   WHEN MONTH_NUM = 12 THEN 'December'
--                   ELSE 'Error'
--                 END; 

-- set schema version to: 7.5
INSERT INTO @SCHEMA@.SCHEMA_VERSION (SCHEMA_VERSION, DATA_MIGRATION) VALUES( 75, 0 );

-- View: ActTmplt
CREATE VIEW @SCHEMA@.ACT_TMPLT(ID, NAME, TYPE, DESCRIPTION ) AS 
 SELECT ACT_TMPLT_T.ID, ACT_TMPLT_T.NAME, ACT_TMPLT_T.TYPE, ACT_TMPLT_T.DESCRIPTION 
 FROM @SCHEMA@.ACT_TMPLT_T;
 
-- View: ETAMatch
CREATE VIEW @SCHEMA@.ETAMATCH(ID, DETERMINER_TYPE, EXTENSION_NAME, DESCRIPTION ) AS 
 SELECT ETAMATCH_T.ID, ETAMATCH_T.DETERMINER_TYPE, ETAMATCH_T.EXTENSION_NAME, ETAMATCH_T.DESCRIPTION 
 FROM @SCHEMA@.ETAMATCH_T;
 
-- View: Action
CREATE VIEW @SCHEMA@.ACTION(ID, NAME, TYPE ) AS 
 SELECT ACTION_T.ID, ACTION_T.NAME, ACTION_T.TYPE 
 FROM @SCHEMA@.ACTION_T;

-- View: ActMgrProp
CREATE VIEW @SCHEMA@.ACT_MGR_PROP(ACTION_PROP_NAME, ACTION_PROP_VALUE ) AS 
 SELECT ACTION_MANAGER_PROP_T.ACTION_PROP_NAME, ACTION_MANAGER_PROP_T.ACTION_PROP_VALUE 
 FROM @SCHEMA@.ACTION_MANAGER_PROP_T; 

-- View: PlugSvc
CREATE VIEW @SCHEMA@.PLUG_SVC(ID, NAME, SERVICE_TYPE, SUB_TYPE, ACTION_HANDLER_ID ) AS 
 SELECT PLUG_SVC_T.ID, PLUG_SVC_T.NAME, PLUG_SVC_T.SERVICE_TYPE, PLUG_SVC_T.SUB_TYPE, PLUG_SVC_T.ACTION_HANDLER_ID 
 FROM @SCHEMA@.PLUG_SVC_T; 

-- View: Alerts
CREATE VIEW @SCHEMA@.ALERTS(ALERT_ID, BUS_SIT_NAME, ALERT_SUBJECT, ALERT_TIMESTAMP ) AS 
 SELECT ACT_MGR_ALERTS_T.ALERT_ID, ACT_MGR_ALERTS_T.BUS_SIT_NAME, ACT_MGR_ALERTS_T.ALERT_SUBJECT, ACT_MGR_ALERTS_T.ALERT_TIMESTAMP 
 FROM @SCHEMA@.ACT_MGR_ALERTS_T; 

-- View: EQ
CREATE view @SCHEMA@.EQ_FE_IDS
(
  DBID,
  INSTANCE_DBID,
  EQ_SEQUENCE
)
AS SELECT DBID, INSTANCE_DBID, EQ_SEQUENCE FROM @SCHEMA@.EQ_FAILED_EVENT_T;

--
-- Populate Action Mgr Tables
--
INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Host','localhost', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Port','25', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.SMTP.Sender','default@127.0.0.1', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UID.Field','uid', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Email.Field','mail', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.CellEmail.Field','mobile', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.PagerEmail.Field','pager', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.CEI_Logging','no', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.URL','ldap://localhost:389', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.UserName','cn=root', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Ldap.Password',' ', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Hostname','localhost', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Port','1414', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.Mqwf.Channel','FMCQM.CL.TCP', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.UID.Field','uid', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Email.Field','mail', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.CellEmail.Field','mobile', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.PagerEmail.Field','pager', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Provider','$VMM_HOST$:$VMM_PORT$', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID) 
  VALUES ('ActionManager.VMM.Entity.Type','PersonAccount', 0);

INSERT INTO @SCHEMA@.ACTION_MANAGER_PROP_T(ACTION_PROP_NAME, ACTION_PROP_VALUE, VERSION_ID)    
  VALUES ('ActionManager.VMM.Group.Type','Group', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1001','EmailHandler','AAM_6.0_EMAIL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.EmailActionHandler','Email Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1002','AlertHandler','AAM_6.0_ALERT_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.AlertActionHandler','Alert Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1003','WebServiceHandler','AAM_6.0_WEBSERVICES_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.WebServicesHandler','WebService Action Service Handler', 0);

--To reactivate MQWF handlers uncomment these lines
--INSERT INTO @SCHEMA@.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1004','MQWF34Handler','AAM_6.0_MQWF34_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFXMLAgent','MQWF 3.4 Action Service Handler', 0);
--INSERT INTO @SCHEMA@.ACT_HNDLR_T
--(ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) values
--('1005','MQWF35Handler','AAM_6.0_MQWF35PLUS_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.MQWFwa0cAgent','MQWF 3.5+ Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1006','CellHandler','AAM_6.0_CELL_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.CellEmailHandler','Cell Email Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1007','PagerHandler','AAM_6.0_PAGER_HNDLR','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.PagerEmailHandler','Pager Email Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('1008','SCAHandler','AAM_6.0_SCA','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.handling.ScaHandler','SCA Action Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2001','IBMSimpleE2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleEventToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO @SCHEMA@.ACT_HNDLR_T
  (ID,NAME,TYPE,INVOCATION_TYPE,INVOCATION_PROFILE,DESCRIPTION, VERSION_ID) 
VALUES ('2002','IBMSimpleA2ADeterminerHandler','AAM_6.0_PLUGSVC','INVTYPE_LOCALJAVA','com.ibm.wbimonitor.actionmgr.determination.SimpleActionToActionDeterminer','Pluggable Service Handler', 0);

INSERT INTO @SCHEMA@.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1001','IBMSimpleE2ADeterminer','DETERMINER','E2A.IBM_SIMPLE','2001','Simple E2A Determiner', 0);

INSERT INTO @SCHEMA@.PLUG_SVC_T
  (ID,NAME,SERVICE_TYPE,SUB_TYPE,ACTION_HANDLER_ID,DESCRIPTION, VERSION_ID) 
VALUES ('1002','IBMSimpleA2ADeterminer','DETERMINER','A2A.IBM_SIMPLE','2002','Simple A2A Determiner', 0);

----------- REST DATA SERVICES SECURITY & AUTHORIZATION TABLES

CREATE TABLE @SCHEMA@.RAT_RESOURCE_GROUPS  (
  ID VARCHAR2(128) NOT NULL ,
  PARENT_ID VARCHAR2(128) ,
  DESCRIPTION VARCHAR2(256) 
) TABLESPACE MONDSTS
  LOGGING;
  
ALTER TABLE @SCHEMA@.RAT_RESOURCE_GROUPS
        ADD CONSTRAINT MON_AUTH_PK2 PRIMARY KEY (ID) USING INDEX TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.RAT_AUTH_DOMAIN  (
  RESOURCE1 VARCHAR2(256) NOT NULL ,
  VERSION  NUMBER(20) NOT NULL ,
  RESOURCE_GROUP VARCHAR2(128) NOT NULL ,
  DESCRIPTION VARCHAR2(256) 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.RAT_AUTH_DOMAIN
        ADD CONSTRAINT MON_AUTH_PK1 PRIMARY KEY (RESOURCE1, VERSION) USING INDEX TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.RAT_SUPERUSERS  (
  USERID VARCHAR2(256) NOT NULL ,
  ASSIGNED_BY VARCHAR2(256) 
) TABLESPACE MONDSTS
  LOGGING;

COMMENT ON TABLE @SCHEMA@.RAT_SUPERUSERS IS 'ANY USER/GROUP ENTERED HERE WILL HAVE ALL ACCESS.';

ALTER TABLE @SCHEMA@.RAT_SUPERUSERS
        ADD CONSTRAINT MON_AUTH_PK4 PRIMARY KEY (USERID) USING INDEX TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.RAT_PARTY_ROLES  (
  USERID VARCHAR2(256) NOT NULL ,
  ROLE VARCHAR2(128) NOT NULL ,
  RESOURCE_GROUP VARCHAR2(128) NOT NULL ,
  MEMBER_TYPE VARCHAR2(20)
) TABLESPACE MONDSTS
  LOGGING;

COMMENT ON TABLE @SCHEMA@.RAT_PARTY_ROLES IS 'member roles in a resource group';

ALTER TABLE @SCHEMA@.RAT_PARTY_ROLES
        ADD CONSTRAINT MON_AUTH_PK5 PRIMARY KEY (USERID, ROLE, RESOURCE_GROUP) USING INDEX TABLESPACE MONIDXTS;

CREATE TABLE @SCHEMA@.RAT_UPDATE_MONITOR  (
   TABLE_NAME VARCHAR2(40) NOT NULL ,
   MARKER VARCHAR2(4) NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

COMMENT ON COLUMN @SCHEMA@.RAT_UPDATE_MONITOR.MARKER IS 'OLD or NEW';

COMMENT ON COLUMN @SCHEMA@.RAT_UPDATE_MONITOR.TABLE_NAME IS 'stores table
names RAT_AUTH_ADMIN, RAT_PARTY_ROLES and RAT_RESOURCE_GROUPS';

ALTER TABLE @SCHEMA@.RAT_UPDATE_MONITOR
        ADD CONSTRAINT MON_AUTH_PK3 PRIMARY KEY (TABLE_NAME) USING INDEX TABLESPACE MONIDXTS;

INSERT INTO   @SCHEMA@.RAT_UPDATE_MONITOR VALUES ('RAT_AUTH_DOMAIN','OLD');
INSERT INTO   @SCHEMA@.RAT_UPDATE_MONITOR VALUES ('RAT_RESOURCE_GROUPS','OLD');
INSERT INTO   @SCHEMA@.RAT_UPDATE_MONITOR VALUES ('RAT_PARTY_ROLES','OLD');
INSERT INTO   @SCHEMA@.RAT_RESOURCE_GROUPS  VALUES('root','root','The default resource group');

----------- TRIGGERS
----------- REST DATA SERVICES SECURITY & AUTHORIZATION TRIGGERS
-----------


CREATE TRIGGER @SCHEMA@.MON_DELETE1
  AFTER DELETE ON @SCHEMA@.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
   
CREATE TRIGGER @SCHEMA@.MON_DELETE2
  AFTER DELETE ON @SCHEMA@.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW  
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_DELETE3
  AFTER DELETE ON @SCHEMA@.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;

CREATE TRIGGER @SCHEMA@.MON_DELETE4
  AFTER DELETE ON @SCHEMA@.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  DELETE FROM @SCHEMA@.RAT_PARTY_ROLES WHERE RESOURCE_GROUP=:OLD.ID; 
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_INSERT1
  AFTER INSERT ON @SCHEMA@.RAT_PARTY_ROLES
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_INSERT2
  AFTER INSERT ON @SCHEMA@.RAT_AUTH_DOMAIN
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN
  UPDATE RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_INSERT3
  AFTER INSERT ON @SCHEMA@.RAT_RESOURCE_GROUPS
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  BEGIN 
  UPDATE RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_SU
  AFTER INSERT ON @SCHEMA@.RAT_SUPERUSERS
  REFERENCING NEW AS NEW
  FOR EACH ROW
  BEGIN 
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_SU2
  AFTER DELETE ON @SCHEMA@.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_SU3
  AFTER UPDATE ON @SCHEMA@.RAT_SUPERUSERS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_UPDATE1
  AFTER UPDATE ON @SCHEMA@.RAT_PARTY_ROLES
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_PARTY_ROLES';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_UPDATE2
  AFTER UPDATE ON @SCHEMA@.RAT_AUTH_DOMAIN
  REFERENCING OLD AS OLD
  FOR EACH ROW
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_AUTH_DOMAIN';
  END;
.
RUN;
  
CREATE TRIGGER @SCHEMA@.MON_UPDATE3
  AFTER UPDATE ON @SCHEMA@.RAT_RESOURCE_GROUPS
  REFERENCING OLD AS OLD
  FOR EACH ROW 
  BEGIN
  UPDATE @SCHEMA@.RAT_UPDATE_MONITOR SET RAT_UPDATE_MONITOR.MARKER ='NEW' WHERE TABLE_NAME ='RAT_RESOURCE_GROUPS';
  END;
.
RUN;

CREATE TRIGGER @SCHEMA@.MODEL_DEPLOY
  AFTER INSERT ON @SCHEMA@.META_MODEL_T
  REFERENCING NEW AS NEW
  FOR EACH ROW 
  DECLARE i PLS_INTEGER;
  BEGIN
  SELECT COUNT(*) 
  INTO i 
  FROM @SCHEMA@.rat_auth_domain 
  WHERE :new.model_id = concat('/', @SCHEMA@.rat_auth_domain.resource1);
  IF i = 0 THEN
     INSERT INTO @SCHEMA@.rat_auth_domain values(substr(:new.model_id,2), :new.version,'root','');
  END IF;
  END;
.
RUN;
    
CREATE TRIGGER @SCHEMA@.MON_ON_DELETE
  AFTER DELETE ON @SCHEMA@.RAT_UPDATE_MONITOR
  REFERENCING OLD AS OLD
  FOR EACH ROW
  BEGIN
  INSERT INTO @SCHEMA@.RAT_UPDATE_MONITOR values(:OLD.table_name,'NEW');
  END;
.
RUN;

-----------DASHBOARD DATA TABLE
CREATE TABLE @SCHEMA@.DASHBOARD_DATA_T
(
  DASHBOARD_CONFIG                   BLOB                                NOT NULL ,
  DASHBOARD_ID                       VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_OWNER                    VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_NAME                     VARCHAR2(128)                       NOT NULL ,
  DASHBOARD_SHARED_USERS             VARCHAR2(512)                               ,
  PRIMARY KEY ( DASHBOARD_ID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

-----------WAS SCHEDULER
CREATE TABLE @SCHEMA@.MONSCHED_TASK
(
  TASKID NUMBER(19) NOT NULL,
  VERSION VARCHAR2(5) NOT NULL,
  ROW_VERSION NUMBER(10) NOT NULL,
  TASKTYPE NUMBER(10) NOT NULL,
  TASKSUSPENDED NUMBER(1) NOT NULL,
  CANCELLED NUMBER(1) NOT NULL,
  NEXTFIRETIME NUMBER(19) NOT NULL,
  STARTBYINTERVAL VARCHAR2(254),
  STARTBYTIME NUMBER(19),
  VALIDFROMTIME NUMBER(19),
  VALIDTOTIME NUMBER(19),
  REPEATINTERVAL VARCHAR2(254),
  MAXREPEATS NUMBER(10) NOT NULL,
  REPEATSLEFT NUMBER(10) NOT NULL,
  TASKINFO BLOB,
  NAME VARCHAR2(254),
  AUTOPURGE NUMBER(10) NOT NULL,
  FAILUREACTION NUMBER(10),
  MAXATTEMPTS NUMBER(10),
  QOS NUMBER(10),
  PARTITIONID NUMBER(10),
  OWNERTOKEN VARCHAR2(200) NOT NULL,
  CREATETIME NUMBER(19) NOT NULL,
  PRIMARY KEY (TASKID) USING INDEX TABLESPACE MONIDXTS 
) TABLESPACE MONDSTS
  LOGGING;

CREATE INDEX @SCHEMA@.MONSCHED_TASK_IDX1 ON @SCHEMA@.MONSCHED_TASK (TASKID, OWNERTOKEN) TABLESPACE MONIDXTS;

CREATE INDEX @SCHEMA@.MONSCHED_TASK_IDX2 ON @SCHEMA@.MONSCHED_TASK (NEXTFIRETIME ASC, REPEATSLEFT, PARTITIONID) 
  TABLESPACE MONIDXTS ;

CREATE TABLE @SCHEMA@.MONSCHED_TREG 
(
  REGKEY VARCHAR2(254) NOT NULL ,
  REGVALUE VARCHAR2(254) ,
  PRIMARY KEY ( REGKEY ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.MONSCHED_LMGR 
(
  LEASENAME VARCHAR2(254) NOT NULL,
  LEASEOWNER VARCHAR2(254),
  LEASE_EXPIRE_TIME NUMBER(19),
  DISABLED VARCHAR2(254),
  PRIMARY KEY ( LEASENAME ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.MONSCHED_LMPR 
(
  LEASENAME VARCHAR2(254) NOT NULL,
  NAME VARCHAR2(254) NOT NULL,
  VALUE VARCHAR2(254) NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

CREATE INDEX @SCHEMA@.MONSCHED_LMPR_IDX1 ON @SCHEMA@.MONSCHED_LMPR (LEASENAME, NAME) TABLESPACE MONIDXTS ;


---------- DATASERVICES SCHEDULER TABLES

CREATE TABLE @SCHEMA@.DSS_MODEL_CONFIG_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  PRIORITY                           NUMBER(20,0)        NOT NULL ,
  STATUS                             NUMBER(20,0)        NOT NULL ,
  LAST_STARTED                       TIMESTAMP                    ,
  LAST_COMPLETED                     TIMESTAMP                    ,
  LAST_RUN_STATUS                    NUMBER(20,0)                 ,
  TASK_ID                            VARCHAR2(128)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL ,
  PRIMARY KEY (MODEL_ID) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.DSS_MODEL_SERVICE_CONFIG_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  SERVICE_NAME                       VARCHAR2(128)       NOT NULL ,
  SERVICE_PRIORITY                   NUMBER(20,0)        NOT NULL ,
  SERVICE_INTERVAL                   NUMBER(20,0)        NOT NULL ,
  SERVICE_INT_UNIT                   NUMBER(20,0)        NOT NULL ,
  START_TIME                         TIMESTAMP           NOT NULL ,
  STATUS                             NUMBER(20,0)        NOT NULL ,
  LAST_STARTED                       TIMESTAMP                    ,
  LAST_COMPLETED                     TIMESTAMP                    ,
  LAST_RUN_STATUS                    NUMBER(20,0)                 ,
  NEXT_EVALUATION                    TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL ,
  PRIMARY KEY(MODEL_ID, SERVICE_NAME) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.DSS_MODEL_SERVICE_VER_CONFIG_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  SERVICE_NAME                       VARCHAR2(128)       NOT NULL ,
  STATUS                             NUMBER(20,0)        NOT NULL ,
  LAST_STARTED                       TIMESTAMP                    ,
  LAST_COMPLETED                     TIMESTAMP                    ,
  LAST_RUN_STATUS                    NUMBER(20,0)                 ,
  MESSAGE                            VARCHAR2(4000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL ,
  PRIMARY KEY ( MODEL_ID, VERSION, SERVICE_NAME) USING INDEX TABLESPACE MONIDXTS 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.DSS_MODEL_CUBE_STATUS_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MC_ID                              VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  SERVER                             VARCHAR2(128)       NOT NULL ,
  IS_STALE                           NUMBER(5,0)         NOT NULL ,
  STALE_TIME                         TIMESTAMP                    ,
  FRESH_TIME                         TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL ,
  PRIMARY KEY ( MODEL_ID, MC_ID, VERSION, SERVER ) USING INDEX TABLESPACE MONIDXTS 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.DSS_MODEL_ARCHIVE_VER_CONFIG_T
(
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  OPERATION_NAME                     VARCHAR2(32)        NOT NULL ,
  TIME_SPAN                          VARCHAR2(128)       NOT NULL ,
  PATH_NAME                          VARCHAR2(1000)               ,
  LAST_TIMESTAMP                     VARCHAR2(32)                 ,
  APPEND_FLAG                        NUMBER(5,0)                  ,
  MAX_FILE_SIZE                      NUMBER(20,0)                 ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL ,
  PRIMARY KEY ( MODEL_ID, VERSION ) USING INDEX TABLESPACE MONIDXTS    
) TABLESPACE MONDSTS
  LOGGING;


---------- FINE GRAINED SECURITY TABLES

CREATE TABLE @SCHEMA@.FGS_SECURITY_FILTER_T
(
  CONTEXT_KEY                        VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  DISPLAY_INDICATION                 NUMBER(5,0)                  ,
  FILTER_VALUE                       CLOB                         ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING
  LOB (FILTER_VALUE) STORE AS
  (
    TABLESPACE MONLOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE @SCHEMA@.FGS_SECURITY_FILTER_T ADD
(
  PRIMARY KEY ( CONTEXT_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_METRIC_T
(
  METRIC_KEY                         VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  METRIC_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_METRIC_T ADD
(
  PRIMARY KEY ( METRIC_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_MEASURE_T
(
  MEASURE_KEY                        VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(128)       NOT NULL ,
  CUBE_ID                            VARCHAR2(128)       NOT NULL ,
  MEASURE_ID                         VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_MEASURE_T ADD
(
  PRIMARY KEY ( MEASURE_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_DIMENSION_T
(
  DIMENSION_KEY                      VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(128)       NOT NULL ,
  CUBE_ID                            VARCHAR2(128)       NOT NULL ,
  DIMENSION_ID                       VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_DIMENSION_T ADD
(
  PRIMARY KEY ( DIMENSION_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_ALERT_TMPLT_T
(
  ALERT_KEY                          VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  TEMPLATE_ID                        VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_ALERT_TMPLT_T ADD
(
  PRIMARY KEY ( ALERT_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_KPI_T
(
  KPI_KEY                            VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  KPIID                              VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_KPI_T ADD
(
  PRIMARY KEY ( KPI_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_MONITOR_CONTEXT_T
(
  MC_KEY                             VARCHAR2(256)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_MONITOR_CONTEXT_T ADD
(
  PRIMARY KEY ( MC_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.MONPROPS_T
(
 CONTEXT	                     VARCHAR2(128)       NOT NULL ,
 PROPERTY	                     VARCHAR2(128)       NOT NULL ,
 CHARVALUE	                     VARCHAR2(512)                ,
 INTVALUE	                     NUMBER(10,0)                 ,
 DECVALUE	                     NUMBER                       , 
 TIMEVALUE	                     TIMESTAMP                    ,
 CLOBVALUE 	                     CLOB,
 BLOBVALUE 	                     BLOB,
 VERSION_ID                          NUMBER(5,0)
) TABLESPACE MONDSTS
  LOGGING;
  
ALTER TABLE @SCHEMA@.MONPROPS_T ADD
(
  PRIMARY KEY ( CONTEXT, PROPERTY )
  USING INDEX TABLESPACE MONIDXTS
);


---------- AUDIT LOG

CREATE TABLE @SCHEMA@.AUDIT_TRAIL_T
(
  AUDIT_ENTRY_ID                     NUMBER              NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER                       ,
  STEP                               VARCHAR(256)        NOT NULL ,
  AUDIT_TIMESTAMP                    TIMESTAMP           NOT NULL ,
  ARRIVAL_TIMESTAMP                  TIMESTAMP                    ,
  AUDIT_MSG_TYPE                     VARCHAR2(1)         NOT NULL ,
  HOSTNAME                           VARCHAR2(256)                ,
  PORT_NUMBER                        VARCHAR2(8)                  ,
  AUDIT_MESSAGE                      VARCHAR2(2048)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL
)  TABLESPACE MONDSTS
  LOGGING;
  
ALTER TABLE @SCHEMA@.AUDIT_TRAIL_T ADD
(
  PRIMARY KEY ( AUDIT_ENTRY_ID  )
  USING INDEX TABLESPACE MONIDXTS
);


CREATE SEQUENCE @SCHEMA@.AUDIT_TRAIL_SEQ INCREMENT BY 1 START WITH 1 NOCYCLE ORDER;


CREATE OR REPLACE TRIGGER @SCHEMA@.AUDIT_TRAIL_AUTONUM BEFORE INSERT ON @SCHEMA@.AUDIT_TRAIL_T FOR EACH ROW
BEGIN
   SELECT @SCHEMA@.AUDIT_TRAIL_SEQ.NEXTVAL
   INTO :NEW.AUDIT_ENTRY_ID
   FROM DUAL;
END;
/


---------- DIRECT TO DEPLOY TABLES

CREATE TABLE @SCHEMA@.D2D_ITEM (
  ID                                 VARCHAR2(255)       NOT NULL , 
  STATE                              VARCHAR2(20)        NOT NULL ,
  ITEM_LEVEL                         NUMBER(10,0)        NOT NULL ,
  MODULE_NAME                        VARCHAR2(255)                ,
  APPLICATION_NAME                   VARCHAR2(255)                ,
  VERSION                            VARCHAR2(30)                 , 
  NEW_APP_NAME                       VARCHAR2(255)                ,
  PRIMARY KEY ( ID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.D2D_PROGRESS (
  ITEM_ID                            VARCHAR2(255)       NOT NULL ,
  PROGRESS                           NUMBER(10,0)                 ,
  PROGRESS_MSG                       VARCHAR2(255)                ,
  PROGRESS_ERROR_CODE                VARCHAR2(255)                ,
  PRIMARY KEY ( ITEM_ID ) USING INDEX TABLESPACE MONIDXTS ,
  FOREIGN KEY ( ITEM_ID ) REFERENCES @SCHEMA@.D2D_ITEM( ID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.D2D_CONTENT (
  ITEM_ID                            VARCHAR2(255)       NOT NULL ,
  PI                                 BLOB                         ,
  PRIMARY KEY ( ITEM_ID ) USING INDEX TABLESPACE MONIDXTS         ,
  FOREIGN KEY ( ITEM_ID ) REFERENCES @SCHEMA@.D2D_ITEM( ID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.D2D_METADATA(
  ITEM_ID                            VARCHAR2(255)       NOT NULL ,
  NAME                               VARCHAR2(255)       NOT NULL ,
  VALUE                              VARCHAR2(255)                ,
  PRIMARY KEY ( ITEM_ID, NAME ) USING INDEX TABLESPACE MONIDXTS   ,
  FOREIGN KEY ( ITEM_ID ) REFERENCES @SCHEMA@.D2D_ITEM( ID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

CREATE TABLE @SCHEMA@.D2D_LOCK ( 
  LOCK_ID                            CHARACTER(2)       NOT NULL  ,
  LASTUPDATE                         TIMESTAMP                    ,
  PRIMARY KEY ( LOCK_ID ) USING INDEX TABLESPACE MONIDXTS
) TABLESPACE MONDSTS
  LOGGING;

INSERT INTO @SCHEMA@.D2D_LOCK VALUES ('N1', NULL);
INSERT INTO @SCHEMA@.D2D_LOCK VALUES ('N2', NULL);

CREATE TABLE @SCHEMA@.D2D_MESSAGE (
  ITEM_ID                            VARCHAR2(255)       NOT NULL ,
  SEQ                                NUMBER(10,0)        NOT NULL ,
  ID                                 VARCHAR2(63)                 ,
  DESCRIPTION                        VARCHAR2(1024)               ,
  MSG_RESOURCE                       VARCHAR2(255)                ,
  LOCATION                           VARCHAR2(63)                 ,
  FOLDER                             VARCHAR2(255)                ,
  MSG_STR                            VARCHAR2(63)                 ,
  TAG	                             VARCHAR(263)                 ,
  SEVERITY                           VARCHAR2(15)                 ,
  TIME                               TIMESTAMP                    ,
  PRIMARY KEY ( ITEM_ID, SEQ ) USING INDEX TABLESPACE MONIDXTS    ,
  FOREIGN KEY ( ITEM_ID ) REFERENCES @SCHEMA@.D2D_ITEM( ID ) ON DELETE CASCADE 
) TABLESPACE MONDSTS
  LOGGING;

-----------USER DEFINED FUNCTIONS
CREATE FUNCTION @SCHEMA@.DAYS (TIME2 IN TIMESTAMP) RETURN NUMBER DETERMINISTIC 
  AS
  TIME1   TIMESTAMP;
  DAYSRC  NUMBER;
  BEGIN
      SELECT TO_TIMESTAMP('01.01.0001:00:00:00','DD.MM.YYYY:HH24:MI:SS') INTO TIME1 FROM DUAL;
      SELECT TRUNC(TO_NUMBER(SUBSTR((TIME2-TIME1),1,INSTR(TIME2-TIME1,' ')))) INTO DAYSRC FROM DUAL; 
      RETURN DAYSRC;
  END;
.
RUN;

CREATE FUNCTION @SCHEMA@.MIDNIGHT_SECONDS (TIME2 IN TIMESTAMP) RETURN NUMBER DETERMINISTIC
  AS
  TIME1   TIMESTAMP;
  HH      NUMBER;
  MM      NUMBER;
  SS      NUMBER;
  BEGIN
      SELECT TO_TIMESTAMP('01.01.0001:00:00:00','DD.MM.YYYY:HH24:MI:SS') INTO TIME1 FROM DUAL;
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+7,2) INTO SS FROM DUAL;
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+4,2) INTO MM FROM DUAL; 
      SELECT SUBSTR((TIME2-TIME1), INSTR((TIME2-TIME1),' ')+1,2) INTO HH FROM DUAL;
      RETURN (HH * 3600) + (MM * 60) + SS;
  END;
.
RUN;


--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.SCHEMA_VERSION TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MODERATOR_NEXT_ID TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MM_TO_WPS_PT TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RECORDED_EVENTS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RECORDED_EVENTS_MODELVERSIONS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MODEL_UNVERSIONED_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.CEI_SERVER_SOURCE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MODEL_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MODEL_STEP_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MONITOR_CONTEXT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MONITOR_METRIC_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_DIMENSION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_DIM_ATTR_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_MEASURE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_CONTEXT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_PERSONALIZED_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_CACHE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_METRIC_FILTER_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_RANGE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_KPI_DEPENDENCY_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.KPI_HISTORY_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.KPI_TS_PREDICTION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.KPI_TS_PREDICTION_MODEL_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DMS_METADATA_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DMS_HISTORY_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DMS_TRACE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.META_OUTBOUND_EVENT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACT_MGR_ALERTS_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DASHBOARD_ALERT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACT_HNDLR_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACTION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACTION_HANDLER_CTRL_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.PLUG_SVC_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACT_TMPLT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.SUBSCRIBED_USER_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ETAMATCH_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ATAMATCH_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ETAACTRL_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACTION_MANAGER_PROP_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.EQ_MODEL_VERSION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DQ_MODEL_VERSION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.EQ_INSTANCE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.EQ_FAILED_EVENT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DQ_DEAD_EVENT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACT_SITUATION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ACT_SIT_TRIG_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DIM_TIME TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RAT_RESOURCE_GROUPS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RAT_AUTH_DOMAIN TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RAT_SUPERUSERS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RAT_PARTY_ROLES TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.RAT_UPDATE_MONITOR TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DASHBOARD_DATA_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MONSCHED_TASK TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MONSCHED_TREG TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MONSCHED_LMGR TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MONSCHED_LMPR TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DSS_MODEL_CONFIG_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DSS_MODEL_SERVICE_CONFIG_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DSS_MODEL_SERVICE_VER_CONFIG_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DSS_MODEL_CUBE_STATUS_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.DSS_MODEL_ARCHIVE_VER_CONFIG_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_SECURITY_FILTER_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_METRIC_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_MEASURE_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_DIMENSION_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_ALERT_TMPLT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_KPI_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.FGS_MONITOR_CONTEXT_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.MONPROPS_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.AUDIT_TRAIL_T TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_VERSION TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_PROPERTY_MAP TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_OBJECTS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_LOOKUP TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_TYPES TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.ABX_LOOKUP_VALUES TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_ITEM TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_PROGRESS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_CONTENT TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_METADATA TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_LOCK TO @DB_USER@;
--GRANT STMT-- GRANT SELECT, INSERT, UPDATE, DELETE ON @SCHEMA@.D2D_MESSAGE TO @DB_USER@;

--GRANT STMT-- GRANT SELECT ON @SCHEMA@.ACT_TMPLT TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.ETAMATCH TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.ACTION TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.ACT_MGR_PROP TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.PLUG_SVC TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.ALERTS TO @DB_USER@;
--GRANT STMT-- GRANT SELECT ON @SCHEMA@.EQ_FE_IDS TO @DB_USER@;

COMMIT;

QUIT


