-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2009, 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
--
-- Process this script using SQL*Plus 
--
-- Example: 
-- SQL> @dim_time_entries_oracle.sql
--
--
-- modify the values '01012008' and '31122022' in the below script to match the years needed for dimensional analysis
-- for example, if only 2011, 2012, and 2013 is needed for dimensional analysis, then change '01012008' to '01012011' and '31122022' to '31122013'
-- ------------------------------------------------------------------------------------
-- Following variable needs to be changed for customization before running this script:
-- @SCHEMA@ = the schema name, eg MONITOR


DECLARE
  start_date DATE;
  sk NUMBER;
  
  end_date DATE;
BEGIN
  start_date := to_date('01012008','DDMMYYYY');
  end_date := to_date('31122022','DDMMYYYY');
  DELETE FROM @SCHEMA@.DIM_TIME;
  WHILE(start_date <= end_date) LOOP
    INSERT INTO @SCHEMA@.DIM_TIME(surrogate_key,year_num,month_num,day_num,month_name) values (start_date,to_number(to_char(start_date,'YYYY')),to_number(to_char(start_date,'MM')),to_number(to_char(start_date,'DD')),to_char(start_date,'MONTH'));
    start_date := start_date + 1;
  END LOOP;
  -- '9999-01-01' is required since TerminationTime uses this value for un-terminated instances
  INSERT INTO @SCHEMA@.DIM_TIME(surrogate_key,year_num,month_num,day_num,month_name) values (TO_DATE('01019999','DDMMYYYY'),9999,1,1,to_char(TO_DATE('01019999','DDMMYYYY'),'MONTH'));
  COMMIT;
END;
/

-- The following UPDATE command can be used to override the default month names that are set 
-- above with TO_CHAR(date,'MONTH'), which returns an upper case character string (for example, 
-- JANUARY), based on the locale of the database. 
-- This UPDATE command can be edited with any location preference for month names (for example,
-- mixed case month names, 3-letter abbreviated month names, French month names, etc).  Names 
-- in different locales can also be had with TO_CHAR(date,'MONTH','nls_date_language = FRENCH').
-- This UPDATE command can be uncommented here or it can be run separately in SQLPlus.
--
-- UPDATE @SCHEMA@.DIM_TIME
--   SET MONTH_NAME =
--        CASE
--                   WHEN MONTH_NUM = 1  THEN 'January'
--                   WHEN MONTH_NUM = 2  THEN 'February'
--                   WHEN MONTH_NUM = 3  THEN 'March'
--                   WHEN MONTH_NUM = 4  THEN 'April'
--                   WHEN MONTH_NUM = 5  THEN 'May'
--                   WHEN MONTH_NUM = 6  THEN 'June'
--                   WHEN MONTH_NUM = 7  THEN 'July'
--                   WHEN MONTH_NUM = 8  THEN 'August'
--                   WHEN MONTH_NUM = 9  THEN 'September'
--                   WHEN MONTH_NUM = 10 THEN 'October'
--                   WHEN MONTH_NUM = 11 THEN 'November'
--                   WHEN MONTH_NUM = 12 THEN 'December'
--                   ELSE 'Error'
--                 END; 