-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011. All Rights Reserved.
-- US Government Users Restricted Rights- Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to upgrade database from v7.0 to v8.0 for Oracle
--
---------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script: 
-- @SCHEMA@ = the schema owner, eg MONITOR
--
-- Run the script in SQL*Plus.
--
-- USAGE:  sqlplus system/passw0rd@orcl @upgradeSchema750.sql
--
-- ------------------------------------------------------------------------------------

---------------------------------------------------------CHANGED:  SCHEMA_VERSION
DELETE FROM @SCHEMA@.SCHEMA_VERSION;
INSERT INTO @SCHEMA@.SCHEMA_VERSION (
        SCHEMA_VERSION, DATA_MIGRATION) 
        VALUES( 80, 0 );
        
-- Changes for Monitor 7.5

ALTER TABLE @SCHEMA@.META_MODEL_T 
  ADD(  ROUTING_JARS    BLOB);

---------- FINE GRAINED SECURITY TABLES

CREATE TABLE @SCHEMA@.FGS_SECURITY_FILTER_T
(
  CONTEXT_KEY                        VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  DISPLAY_INDICATION                 NUMBER(5,0)                  ,
  FILTER_VALUE                       CLOB                         ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING
  LOB (FILTER_VALUE) STORE AS
  (
    TABLESPACE MONLOBTS
    ENABLE STORAGE IN ROW
    NOCACHE
  );

ALTER TABLE @SCHEMA@.FGS_SECURITY_FILTER_T ADD
(
  PRIMARY KEY ( CONTEXT_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_METRIC_T
(
  METRIC_KEY                         VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  METRIC_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_METRIC_T ADD
(
  PRIMARY KEY ( METRIC_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_MEASURE_T
(
  MEASURE_KEY                        VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(128)       NOT NULL ,
  CUBE_ID                            VARCHAR2(128)       NOT NULL ,
  MEASURE_ID                         VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_MEASURE_T ADD
(
  PRIMARY KEY ( MEASURE_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_DIMENSION_T
(
  DIMENSION_KEY                      VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(128)       NOT NULL ,
  CUBE_ID                            VARCHAR2(128)       NOT NULL ,
  DIMENSION_ID                       VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_DIMENSION_T ADD
(
  PRIMARY KEY ( DIMENSION_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_ALERT_TMPLT_T
(
  ALERT_KEY                          VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  TEMPLATE_ID                        VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_ALERT_TMPLT_T ADD
(
  PRIMARY KEY ( ALERT_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_KPI_T
(
  KPI_KEY                            VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  KPIID                              VARCHAR2(128)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(128)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,  
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_KPI_T ADD
(
  PRIMARY KEY ( KPI_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.FGS_MONITOR_CONTEXT_T
(
  MC_KEY                             VARCHAR2(256)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  MCID                               VARCHAR2(256)       NOT NULL ,
  ENTITY_ID                          VARCHAR2(256)       NOT NULL ,
  ENTITY_TYPE                        VARCHAR2(20)        NOT NULL ,
  VISIBLE                            NUMBER(5,0)         NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.FGS_MONITOR_CONTEXT_T ADD
(
  PRIMARY KEY ( MC_KEY, ENTITY_ID, ENTITY_TYPE )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.MONPROPS_T
(
 CONTEXT	                     VARCHAR2(128)       NOT NULL ,
 PROPERTY	                     VARCHAR2(128)       NOT NULL ,
 CHARVALUE	                     VARCHAR2(512)                ,
 INTVALUE	                     NUMBER(10,0)                 ,
 DECVALUE	                     NUMBER                       , 
 TIMEVALUE	                     TIMESTAMP                    ,
 CLOBVALUE 	                     CLOB,
 BLOBVALUE 	                     BLOB,
 VERSION_ID                          NUMBER(5,0)
) TABLESPACE MONDSTS
  LOGGING;
  
ALTER TABLE @SCHEMA@.MONPROPS_T ADD
(
  PRIMARY KEY ( CONTEXT, PROPERTY )
  USING INDEX TABLESPACE MONIDXTS
);


---------- AUDIT LOG

CREATE TABLE @SCHEMA@.AUDIT_TRAIL_T
(
  AUDIT_ENTRY_ID                     NUMBER              NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER                       ,
  STEP                               VARCHAR(256)        NOT NULL ,
  AUDIT_TIMESTAMP                    TIMESTAMP           NOT NULL ,
  ARRIVAL_TIMESTAMP                  TIMESTAMP                    ,
  AUDIT_MSG_TYPE                     VARCHAR2(1)         NOT NULL ,
  HOSTNAME                           VARCHAR2(256)                ,
  PORT_NUMBER                        VARCHAR2(8)                  ,
  AUDIT_MESSAGE                      VARCHAR2(2048)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL
)  TABLESPACE MONDSTS
  LOGGING;
  
ALTER TABLE @SCHEMA@.AUDIT_TRAIL_T ADD
(
  PRIMARY KEY ( AUDIT_ENTRY_ID  )
  USING INDEX TABLESPACE MONIDXTS
);


CREATE SEQUENCE @SCHEMA@.AUDIT_TRAIL_SEQ INCREMENT BY 1 START WITH 1 NOCYCLE ORDER;


CREATE OR REPLACE TRIGGER @SCHEMA@.AUDIT_TRAIL_AUTONUM BEFORE INSERT ON @SCHEMA@.AUDIT_TRAIL_T FOR EACH ROW
BEGIN
   SELECT @SCHEMA@.AUDIT_TRAIL_SEQ.NEXTVAL
   INTO :NEW.AUDIT_ENTRY_ID
   FROM DUAL;
END;
/

CREATE TABLE @SCHEMA@.META_KPI_PERSONALIZED_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  USER_ID                            VARCHAR2(256)       NOT NULL ,
  TARGET                             NUMBER                       ,
  FGS_FILTER_HASH                    NUMBER(20,0)                 ,
  ENABLE_KPI_HISTORY                 NUMBER(5,0)         NOT NULL ,
  ENABLE_KPI_PREDICTIONS             NUMBER(5,0)         NOT NULL ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_PERSONALIZED_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, USER_ID )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_CACHE_T
(
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  MODEL_ID                           VARCHAR2(128)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  FGS_FILTER_HASH                    NUMBER(20,0)        NOT NULL ,
  KPI_CACHE_VALUE                    NUMBER                       ,
  KPI_LAST_CALCULATION_TIME          TIMESTAMP                    ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_CACHE_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, FGS_FILTER_HASH )
  USING INDEX TABLESPACE MONIDXTS
);

CREATE TABLE @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T
(
  KPI_RANGE_ID                       VARCHAR2(512)       NOT NULL ,
  KPI_ID                             VARCHAR2(384)       NOT NULL ,
  VERSION                            NUMBER(20,0)        NOT NULL ,
  USER_ID                            VARCHAR2(256)       NOT NULL ,
  DISPLAY_NAME                       VARCHAR2(1000)               ,
  START_VALUE                        NUMBER              NOT NULL ,
  END_VALUE                          NUMBER              NOT NULL ,
  COLOR                              VARCHAR2(7)                  ,
  ICON                               VARCHAR2(256)                ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T ADD
(
  PRIMARY KEY ( KPI_RANGE_ID, VERSION, USER_ID )
  USING INDEX TABLESPACE MONIDXTS
);



ALTER TABLE @SCHEMA@.KPI_HISTORY_T DROP PRIMARY KEY;


ALTER TABLE @SCHEMA@.KPI_HISTORY_T ADD
(
  FGS_FILTER_HASH    NUMBER(20,0)  DEFAULT 0 NOT NULL
);


ALTER TABLE @SCHEMA@.KPI_HISTORY_T ADD
(
  PRIMARY KEY ( KPI_ID, VERSION, KPI_DYNAMIC_FILTER_VALUE, PERIOD_TIMESTAMP, FGS_FILTER_HASH )
  USING INDEX TABLESPACE INDEXTS
);

ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T DROP PRIMARY KEY;


ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T ADD
(
  FGS_FILTER_HASH    NUMBER(20,0)  DEFAULT 0 NOT NULL
);


ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T ADD
(
  PRIMARY KEY ( KPI_PREDICTION_ID, VERSION, LAST_KPI_TIME, FGS_FILTER_HASH )
  USING INDEX TABLESPACE INDEXTS
);

ALTER TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T ADD
(
  FGS_LASTUPDATE_SECFILTER TIMESTAMP,
  FGS_LASTUPDATE_OBJSEC TIMESTAMP,
  FGS_LASTUPDATE_COGNOS TIMESTAMP
);


ALTER TABLE @SCHEMA@.META_MODEL_T 
  ADD(  ROOT_META_INF_CONTENTS    BLOB);
  
  
ALTER SEQUENCE @SCHEMA@.RECORDED_EVENTS_SEQ ORDER;
  

--ADD Cognos Changes

ALTER TABLE @SCHEMA@.META_MODEL_T 
        ADD (COGNOS_PACKAGE VARCHAR(1000));

ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T 
        ADD (COGNOS_NAMESPACE VARCHAR(1000));

ALTER TABLE @SCHEMA@.META_DIMENSION_T 
        ADD (COGNOS_NAME VARCHAR(1000));

ALTER TABLE @SCHEMA@.META_DIM_ATTR_T 
        ADD (COGNOS_NAME VARCHAR(1000));

ALTER TABLE @SCHEMA@.META_MEASURE_T 
        ADD (COGNOS_NAME VARCHAR(1000));

ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T 
        ADD (COGNOS_NAME  VARCHAR(1000));


CREATE SEQUENCE @SCHEMA@.ERROR_Q_SEQ INCREMENT BY 1 START WITH 1 NOCYCLE ORDER;



-- Changes for 8.0

ALTER TABLE @SCHEMA@.ACT_MGR_ALERTS_T 
        ADD (PRIORITY NUMBER(20,0));

CREATE TABLE @SCHEMA@.ACT_ALERT_COMMENTS_T
(
  ALERT_ID                           VARCHAR2(32)        NOT NULL ,
  COMMENT_OWNER                      VARCHAR2(32)        NOT NULL ,
  COMMENT_TIME                       TIMESTAMP           NOT NULL ,
  COMMENT_TEXT                       VARCHAR2(4000)               ,
  VERSION_ID                         NUMBER(5,0)         NOT NULL 
) TABLESPACE MONDSTS
  LOGGING;

ALTER TABLE @SCHEMA@.ACT_ALERT_COMMENTS_T ADD
(
  PRIMARY KEY ( ALERT_ID, COMMENT_OWNER, COMMENT_TIME )
  USING INDEX TABLESPACE MONIDXTS
);

ALTER TABLE @SCHEMA@.ACT_MGR_ALERTS_T 
        ADD (STATUS VARCHAR2(32));


ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T 
        ADD (HIDDEN NUMBER(5,0));        

UPDATE @SCHEMA@.META_MONITOR_CONTEXT_T SET HIDDEN = 0;

ALTER TABLE  @SCHEMA@.META_MONITOR_CONTEXT_T 
MODIFY	     HIDDEN
CONSTRAINT   META_MONITOR_CONTEXT_C1  NOT NULL;


ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T 
        ADD (HIDDEN NUMBER(5,0));        

UPDATE @SCHEMA@.META_MONITOR_METRIC_T SET HIDDEN = 0;

ALTER TABLE  @SCHEMA@.META_MONITOR_METRIC_T 
MODIFY	     HIDDEN
CONSTRAINT   META_MONITOR_METRIC_C1  NOT NULL;


ALTER TABLE @SCHEMA@.META_KPI_T
        ADD (HIDDEN NUMBER(5,0));        

UPDATE @SCHEMA@.META_KPI_T SET HIDDEN = 0;

ALTER TABLE  @SCHEMA@.META_KPI_T 
MODIFY	     HIDDEN
CONSTRAINT   META_KPI_C1  NOT NULL; 

COMMIT;
QUIT  