-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Scriptfile to upgrade database from v7.0 to v8.0 for SQL Server
--
-- ------------------------------------------------------------------------------------
-- Following variables need to be changed for customization before running this script:
-- @DB_NAME@ = the database name, eg MONITOR
-- @SCHEMA@  = the schema name, eg MONITOR
--
--
-- Run the script in the sqlcmd command line processor
--
-- USAGE:  sqlcmd -U <userid> -P <password> -e -i upgradeSchema700.sql
--
-- ------------------------------------------------------------------------------------
use @DB_NAME@
checkpoint
GO

DELETE FROM @SCHEMA@.SCHEMA_VERSION
GO

INSERT INTO @SCHEMA@.SCHEMA_VERSION (
        SCHEMA_VERSION, DATA_MIGRATION) 
        VALUES( 80, 0 )
GO


-- Changes for Monitor 7.5

ALTER TABLE @SCHEMA@.META_MODEL_T 
  ADD ROUTING_JARS  VARBINARY(MAX)
GO  


---------- FINE GRAINED SECURITY TABLES


CREATE TABLE @SCHEMA@.FGS_SECURITY_FILTER_T
(
    CONTEXT_KEY                        NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    MCID                               NVARCHAR(256)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    DISPLAY_INDICATION                 SMALLINT                           NOT NULL ,
    FILTER_VALUE                       TEXT                                   NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( CONTEXT_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_METRIC_T
(
    METRIC_KEY                         NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    MCID                               NVARCHAR(256)                      NOT NULL ,
    METRIC_ID                          NVARCHAR(256)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    VISIBLE                            SMALLINT                           NOT NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( METRIC_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_MEASURE_T
(
    MEASURE_KEY                        NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    MCID                               NVARCHAR(128)                      NOT NULL ,
    CUBE_ID                            NVARCHAR(128)                      NOT NULL ,
    MEASURE_ID                         NVARCHAR(128)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    VISIBLE                            SMALLINT                           NOT NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( MEASURE_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_DIMENSION_T
(
    DIMENSION_KEY                      NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    MCID                               NVARCHAR(128)                      NOT NULL ,
    CUBE_ID                            NVARCHAR(128)                      NOT NULL ,
    DIMENSION_ID                       NVARCHAR(128)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    VISIBLE                            SMALLINT                           NOT NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( DIMENSION_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_ALERT_TMPLT_T
(
    ALERT_KEY                          NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    TEMPLATE_ID                        NVARCHAR(128)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    VISIBLE                            SMALLINT                           NOT NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( ALERT_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_KPI_T
(
    KPI_KEY                            NVARCHAR(256)                      NOT NULL ,
    MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
    KPIID                              NVARCHAR(128)                      NOT NULL ,
    ENTITY_ID                          NVARCHAR(128)                      NOT NULL ,
    ENTITY_TYPE                        NVARCHAR(20)                       NOT NULL ,
    VISIBLE                            SMALLINT                           NOT NULL ,
    LAST_UPDATED                       DATETIME                                    ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( KPI_KEY, ENTITY_ID, ENTITY_TYPE )
)
GO


CREATE TABLE @SCHEMA@.FGS_MONITOR_CONTEXT_T
(
  MC_KEY                             NVARCHAR(256)                     NOT NULL ,
  MODEL_ID                           NVARCHAR(128)                     NOT NULL ,
  MCID                               NVARCHAR(256)                     NOT NULL ,
  ENTITY_ID                          NVARCHAR(128)                     NOT NULL ,
  ENTITY_TYPE                        NVARCHAR(20)                      NOT NULL ,
  VISIBLE                            SMALLINT                          NOT NULL ,
  LAST_UPDATED                       DATETIME                                   ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( MC_KEY, ENTITY_ID, ENTITY_TYPE )
) 
GO



CREATE TABLE @SCHEMA@.MONPROPS_T
(
 CONTEXT	                     NVARCHAR(128)                      NOT NULL ,
 PROPERTY	                     NVARCHAR(128)                      NOT NULL ,
 CHARVALUE	                     NVARCHAR(512)                               ,
 INTVALUE	                     INTEGER                                     ,
 DECVALUE	                     FLOAT(53)                                   , 
 TIMEVALUE	                     DATETIME                                    ,
 CLOBVALUE 	                     TEXT                                        , 
 BLOBVALUE 	                     VARBINARY(MAX)                              ,
 VERSION_ID                          SMALLINT                                    ,
 PRIMARY KEY ( CONTEXT, PROPERTY )
)
GO

---------- AUDIT LOG

CREATE TABLE @SCHEMA@.AUDIT_TRAIL_T
(
  AUDIT_ENTRY_ID                     BIGINT IDENTITY(1,1)               NOT NULL ,
  MODEL_ID                           NVARCHAR(128)                      NOT NULL ,
  VERSION                            BIGINT                                      ,
  STEP                               NVARCHAR(256)                      NOT NULL ,
  AUDIT_TIMESTAMP                    DATETIME                           NOT NULL ,
  ARRIVAL_TIMESTAMP                  DATETIME                                    ,
  AUDIT_MSG_TYPE                     NVARCHAR(1)                        NOT NULL ,
  HOSTNAME                           NVARCHAR(256)                               ,
  PORT_NUMBER                        NVARCHAR(8)                                 ,
  AUDIT_MESSAGE                      NVARCHAR(2048)                              ,
  VERSION_ID                         SMALLINT                           NOT NULL ,
  PRIMARY KEY ( AUDIT_ENTRY_ID )
)
GO


CREATE TABLE @SCHEMA@.META_KPI_PERSONALIZED_T
(
    KPI_ID                             VARCHAR(384)                       NOT NULL ,
    MODEL_ID                           VARCHAR(128)                       NOT NULL ,
    VERSION                            BIGINT                             NOT NULL ,
    USER_ID                            VARCHAR(256)                       NOT NULL ,
    TARGET                             FLOAT(53)                              NULL ,
    FGS_FILTER_HASH                    BIGINT                                 NULL ,
    ENABLE_KPI_HISTORY                 SMALLINT                           NOT NULL ,
    ENABLE_KPI_PREDICTIONS             SMALLINT                           NOT NULL ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( KPI_ID, VERSION, USER_ID )
)
GO

CREATE TABLE @SCHEMA@.META_KPI_CACHE_T
(
    KPI_ID                             VARCHAR(384)                       NOT NULL ,
    MODEL_ID                           VARCHAR(128)                       NOT NULL ,
    VERSION                            BIGINT                             NOT NULL ,
    FGS_FILTER_HASH                    BIGINT                             NOT NULL ,
    KPI_CACHE_VALUE                    FLOAT(53)                              NULL ,
    KPI_LAST_CALCULATION_TIME          DATETIME                               NULL ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( KPI_ID, VERSION, FGS_FILTER_HASH )
)
GO

CREATE TABLE @SCHEMA@.META_KPI_RANGE_PERSONALIZED_T
(
    KPI_RANGE_ID                       VARCHAR(512)                       NOT NULL ,
    KPI_ID                             VARCHAR(384)                       NOT NULL ,
    VERSION                            BIGINT                             NOT NULL ,
    USER_ID                            VARCHAR(256)                       NOT NULL ,
    DISPLAY_NAME                       VARCHAR(1000)                          NULL ,
    START_VALUE                        FLOAT(53)                          NOT NULL ,
    END_VALUE                          FLOAT(53)                          NOT NULL ,
    COLOR                              VARCHAR(7)                             NULL ,
    ICON                               VARCHAR(256)                           NULL ,
    VERSION_ID                         SMALLINT                           NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( KPI_RANGE_ID, VERSION, USER_ID )
)
GO


--block to drop primary key
DECLARE @pkname varchar(128)
DECLARE @dropsql as varchar(1024)

SET @pkname = (SELECT k.name
FROM sys.key_constraints k, sys.schemas s
WHERE k.schema_id = s.schema_id AND 
k.type = 'PK' AND s.name = '@SCHEMA@'
AND OBJECT_NAME(k.parent_object_id) = 'KPI_HISTORY_T')

SET @dropsql = 'ALTER TABLE @SCHEMA@.KPI_HISTORY_T 
DROP CONSTRAINT ' + @pkname

EXEC(@dropsql)

GO


ALTER TABLE @SCHEMA@.KPI_HISTORY_T ADD
  FGS_FILTER_HASH  BIGINT NOT NULL DEFAULT 0
GO  


ALTER TABLE @SCHEMA@.KPI_HISTORY_T ADD
  PRIMARY KEY ( KPI_ID, VERSION, KPI_DYNAMIC_FILTER_VALUE, PERIOD_TIMESTAMP, FGS_FILTER_HASH )
GO


--block to drop primary key
DECLARE @pkname varchar(128)
DECLARE @dropsql as varchar(1024)

SET @pkname = (SELECT k.name
FROM sys.key_constraints k, sys.schemas s
WHERE k.schema_id = s.schema_id AND 
k.type = 'PK' AND s.name = '@SCHEMA@'
AND OBJECT_NAME(k.parent_object_id) = 'KPI_TS_PREDICTION_T')

SET @dropsql = 'ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T 
DROP CONSTRAINT ' + @pkname

EXEC(@dropsql)

GO


ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T  ADD
  FGS_FILTER_HASH  BIGINT NOT NULL DEFAULT 0
GO  


ALTER TABLE @SCHEMA@.KPI_TS_PREDICTION_T ADD
  PRIMARY KEY ( KPI_PREDICTION_ID, VERSION, LAST_KPI_TIME, FGS_FILTER_HASH )
GO


ALTER TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T ADD FGS_LASTUPDATE_SECFILTER DATETIME
GO  

ALTER TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T ADD FGS_LASTUPDATE_OBJSEC DATETIME
GO  

ALTER TABLE @SCHEMA@.META_MODEL_UNVERSIONED_T ADD FGS_LASTUPDATE_COGNOS DATETIME
GO  


ALTER TABLE @SCHEMA@.META_MODEL_T 
  ADD ROOT_META_INF_CONTENTS  VARBINARY(MAX)
GO  


--ADD Cognos Changes

ALTER TABLE @SCHEMA@.META_MODEL_T 
  ADD COGNOS_PACKAGE NVARCHAR(1000)
GO

ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T 
  ADD COGNOS_NAMESPACE NVARCHAR(1000)
GO  

ALTER TABLE @SCHEMA@.META_DIMENSION_T 
  ADD COGNOS_NAME NVARCHAR(1000)
GO  

ALTER TABLE @SCHEMA@.META_DIM_ATTR_T 
  ADD COGNOS_NAME NVARCHAR(1000)
GO  

ALTER TABLE @SCHEMA@.META_MEASURE_T 
  ADD COGNOS_NAME NVARCHAR(1000)
GO  

ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T 
  ADD COGNOS_NAME NVARCHAR(1000)
GO  


-- Changes for 8.0

ALTER TABLE @SCHEMA@.ACT_MGR_ALERTS_T 
  ADD PRIORITY BIGINT
GO 

CREATE TABLE @SCHEMA@.ACT_ALERT_COMMENTS_T
(
    ALERT_ID                           NVARCHAR(32)                        NOT NULL ,
    COMMENT_OWNER                      NVARCHAR(32)                        NOT NULL ,
    COMMENT_TIME                       DATETIME                            NOT NULL ,
    COMMENT_TEXT                       NVARCHAR(4000)                          NULL ,
    VERSION_ID                         SMALLINT                            NOT NULL ,
    PRIMARY KEY NONCLUSTERED ( ALERT_ID, COMMENT_OWNER, COMMENT_TIME )
)
GO

ALTER TABLE @SCHEMA@.ACT_MGR_ALERTS_T 
  ADD STATUS NVARCHAR(32) NULL
GO 

ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T
  ADD HIDDEN SMALLINT NULL
GO 

UPDATE @SCHEMA@.META_MONITOR_CONTEXT_T SET HIDDEN = 0
GO

ALTER TABLE @SCHEMA@.META_MONITOR_CONTEXT_T 
    ADD CONSTRAINT META_MONITOR_CONTEXT_C1 CHECK (HIDDEN IS NOT NULL)
GO 

ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T
  ADD HIDDEN SMALLINT NULL
GO 

UPDATE @SCHEMA@.META_MONITOR_METRIC_T SET HIDDEN = 0
GO

ALTER TABLE @SCHEMA@.META_MONITOR_METRIC_T
    ADD CONSTRAINT META_MONITOR_METRIC_C1 CHECK (HIDDEN IS NOT NULL)
GO  

ALTER TABLE @SCHEMA@.META_KPI_T
  ADD HIDDEN SMALLINT NULL
GO 

UPDATE @SCHEMA@.META_KPI_T SET HIDDEN = 0
GO

ALTER TABLE @SCHEMA@.META_KPI_T
    ADD CONSTRAINT META_KPI_C1 CHECK (HIDDEN IS NOT NULL)
GO 