#  Licensed Materials - Property of IBM
#  5724-M23, 5724-M24
#  Copyright IBM Corporation 2007, 2011. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

import java
#-------------------------------------------------------------------------------
# This jython script installs the referenced EAR
#-------------------------------------------------------------------------------

def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

#######################################
# installEar subroutine
# location   - location where the EAR resides
# earName    - name of the ear in the location directory
# appName    - name of the Application
# nodeName   - name of the WAS node
# serverName - name of the WAS server
#######################################
#
# $AdminApp install <location>/<earName>.ear {-appname <appName> -node <nodeName> -server <serverName>}
# $AdminApp install c:/IBM/WebSphere/AppServer/systemApps/wimwmm.ear {-appname wimwmm -node hardyNode01 -server server1}
#
#######################################
def installSysApp ( location, earName, appName, nodeName, serverName ):
        rc = 0
        print "INFO:  Entering installSysApp"
        print "INFO:  location="+location+",earName="+earName+",appName="+appName+",nodeName="+nodeName+",serverName="+serverName

        #--------------------------------------------------------------
        # set up globals
        #--------------------------------------------------------------
        global AdminConfig
 
        #--------------------------------------------------------------
        # do some sanity checking 
        #     -- do we have a node by this name? 
        #--------------------------------------------------------------
        #cellname = AdminControl.getCell( )
        cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
        node = AdminConfig.getid("/Node:"+nodeName+"/" )
        print "INFO:  checking for existence of node "+nodeName
        if (len(node) == 0):
                print "INFO:  Error -- node not found for name "+nodeName
                rc = -1
                return rc
        #endIf 


        #---------------------------------------------------------
        # First, list the existing applications 
        #---------------------------------------------------------
        apps = AdminApp.list( )
        print "INFO:  Installed applications: "
        print apps

        # Check if App is already installed on the target server
        # Comparing the application name.
        appExist = "False"
        for tmpAppName in wsadminToList(apps):
                   if (cmp(tmpAppName, appName) == 0):
                           appExist = "True"
                           #Do nothing if it already installed
                           print "WARNING: The system application "+earName+".ear (ApplicationName: "+appName+") is already installed on server. We don't reinstall the system application."
                           return 0 
                #endIf 
        #endFor 

        #---------------------------------------------------------
        # Here is the ear file we'll be dealing with... 
        #---------------------------------------------------------
        defapp = location+"/"+earName+".ear"

        #---------------------------------------------------------
        # A very simple install command 
        #---------------------------------------------------------
        print "INFO:  Installation of System Application "+appName+" beginning."
        try:
            AdminApp.install(defapp, ["-appname", appName, "-node", nodeName, "-server", serverName] )
        except:
            print "Exception occurs in AdminApp.install."
            #TODO Need to figure it out how to ignore error when it already installed. 
            #AdminApp.list() doesn't list system applications.
            #rc = -1	
	
        # check for success
        # cannot do this for system app as is not visible to wsadmin
        print "INFO:  Installation of System Application "+appName+" completed."

        AdminConfig.save( )
        print "INFO:  Exiting installSysApp"	        
        return rc
#endDef 

#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------
#if ((len(sys.argv) != 5)):
#        print "installEar: this script requires 5 parameters: scriptname, location of ear, ear, application, node"
#        print "e.g.:     installEar.py /opt/WebSphere/VoiceServer/ear wvsapp WVS mynode"
#else:
#        installEar(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4] )
#endif 

