#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2008. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

import sys
import java


#
#
#
class Scope:
    CLUSTER_SCOPE = 1
    CELL_SCOPE = 2
    NODE_SCOPE = 3
    SERVER_SCOPE = 4
# instance variables are...
#    cell
#    node
#    server
#    cluster

    def __init__(self, cluster, cell, node, server):
        self.cluster = cluster
        self.cell = cell
        self.node = node
        self.server = server
        self.containmentPathMap = {}
        self.containmentPathMap[Scope.CLUSTER_SCOPE] = "/Cell:" + cell + "/Cluster:" + cluster + "/"
        self.containmentPathMap[Scope.CELL_SCOPE] = "/Cell:" + cell + "/"
        self.containmentPathMap[Scope.NODE_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/"
        self.containmentPathMap[Scope.SERVER_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/Server:" + server + "/"

    def getConfigId(self, scopeLevel):
        return AdminConfig.getid(self.getContainmentPath(scopeLevel))

    def getContainmentPath(self, scopeLevel):
        return self.containmentPathMap[scopeLevel]

#
#
#
def setJVMSystemProperties(scope, props):
    if (props == None):
        return

    serverCfgId = scope.getConfigId(Scope.SERVER_SCOPE)
    myJVM = AdminConfig.list("JavaVirtualMachine", serverCfgId)

    if (myJVM == None):
        print "No suitable JVM was found for Cell=" + scope.cell + ", Node=" + scope.node + ", Server=" + scope.server
    else:
        nameValPairs = props.items()
        for i in range(len(nameValPairs)):
            currProperty = [["name", nameValPairs[i][0]], ["value", nameValPairs[i][1]]]

            allPropsString = AdminConfig.list("TypedProperty", myJVM)
            allPropsArray = allPropsString.split(lineSeparator)

            print "Checking", allPropsArray, " for a match to",  props
            matched = "false"
            for j in range(len(allPropsArray)):
                if (allPropsArray[j] == ""):
                    continue

                systemPropertyName = AdminConfig.showAttribute(allPropsArray[j], "name")
                currPropertyName = nameValPairs[i][0]
                print "Checking: " + systemPropertyName + " against " + currPropertyName
                if (currPropertyName == systemPropertyName ):
                    print "Matched"
                    matched = "true"
                    print "Modifying: ", currProperty
                    AdminConfig.modify(allPropsArray[j],currProperty)
                    break

            if(matched != "true"):
                print "Creating:", currProperty
                AdminConfig.create("TypedProperty", myJVM, currProperty)

        AdminConfig.save()

#
#
#
def setJVMSystemProperty(scope, attr, value):
        print "trying to set property " + attr + " = " + value
        jvmProperties = {}
        jvmProperties[attr] = value
        setJVMSystemProperties(scope, jvmProperties)

