#  Licensed Materials - Property of IBM
#  5724-M23, 5724-M24
#  Copyright IBM Corporation 2007, 2011. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

#-------------------------------------------------------------------------------
# This jython script configures the Environmental Variables
#-------------------------------------------------------------------------------
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

#######################################
# environment subroutine
# cellName - name of the WAS Cell
# nodeName - name of the WAS Node
# serverName - name of the WAS server instance (i.e. server1)

# varName - name of the variable
# varValue - value of the variable
# varDescription -  description of the variable
#######################################
def environment(cellName, nodeName, serverName, varName, varValue, varDescription):
    print "INFO:  Entering environment"
    print "INFO:  varName="+varName+",varValue="+varValue+",varDescription="+varDescription
	
    id = None
    variableMapNodeId = None
    if(nodeName == None):	
        id = AdminConfig.getid("/Cell:"+cellName+"/" )
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellName+"/VariableMap:/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/"
    else:
        id = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/" )
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/VariableMap:/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/Node:"+nodeName+"/"
               
    wasVarDescription = ["description", varDescription]
    wasVarName = ["symbolicName", varName]
    wasVarValue = ["value", varValue]
    wasVarAttrs = [wasVarDescription, wasVarName, wasVarValue]
    if(cmp(varDescription,"") ==0):
        wasVarAttrs = [wasVarName, wasVarValue]
    #endif	
    installed    = "false"
    modified     = "false"

    varList = AdminConfig.list("VariableSubstitutionEntry", id )
    for tmpEntryId in wsadminToList(varList):
        #For Cell scope, we skip nodes
        if(nodeName == None):
            if(str(tmpEntryId).find("/nodes/") != -1):
                continue
            
        name = AdminConfig.showAttribute(tmpEntryId, "symbolicName")
        value = AdminConfig.showAttribute(tmpEntryId, "value")
        if (value != ""):
            print "%35s :  %-50s" % (name, value) 
        #endIf 
        if (cmp(name, varName) == 0):
            installed = "true"
            if (cmp(value, varValue) != 0):
                print "INFO: "+varName+" needs to be changed"
                AdminConfig.modify(tmpEntryId, wasVarAttrs )
                value = AdminConfig.showAttribute(tmpEntryId, "value" )
                modified = "true"
        	#endIf 
            break
        #endIf 
    #endFor 
    flagOk = "false"
    if (cmp(modified, "false") == 0):
        if (cmp(installed, "true") == 0):
            flagOk = "true"
            print "INFO: "+varName+" with the correct value is already there, no action taken."
        else:
            print "INFO:  "+varName+" needs to be created"
            variableEntryId = AdminConfig.create("VariableSubstitutionEntry", variableMapNodeId, wasVarAttrs )
            name = AdminConfig.showAttribute(variableEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(variableEntryId, "value" )
        #endElse 
    #endIf 

    # Check for success
    if (cmp(modified, "true") == 0 or cmp(installed, "false") == 0):
        varList = AdminConfig.list("VariableSubstitutionEntry", id )
        for tmpEntryId in wsadminToList(varList):
            #For Cell scope, we skip nodes
            if(nodeName == None):
                if(str(tmpEntryId).find("/nodes/") != -1):
                    continue
            name = AdminConfig.showAttribute(tmpEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(tmpEntryId, "value" )
            if (cmp(name, varName) == 0):
                if (cmp(value, varValue) == 0):
                    flagOk = "true"
                #endIf 
                break
            #endIf 
        #endFor 
    #endIf         
    if (cmp(flagOk, "true") == 0):
        AdminConfig.save( )
        print "INFO: "+varName+" successfully created/modified."
    else:
        print "ERROR: WebSphere Environment Variable "+varName+" could not be created/modified."
    #endElse 
    print "INFO:  Exiting environment"
    return
#endDef

def removeEnvironment(cellName, nodeName, serverName, varName):
    # Remove a WebSphere environment variable
    print "INFO:  Entering remove environment"
    print "INFO:  varName="+varName
    
    id = None
    if(nodeName == None):    
        id = AdminConfig.getid("/Cell:"+cellName+"/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/"
    else:
        id = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/Node:"+nodeName+"/"

    found = "false"
    varList = AdminConfig.list("VariableSubstitutionEntry", id )
    for tmpEntryId in wsadminToList(varList):
        #For Cell scope, we skip nodes
        if(nodeName == None):
            if(str(tmpEntryId).find("/nodes/") != -1):
                continue
            
        name = AdminConfig.showAttribute(tmpEntryId, "symbolicName")
        if (cmp(name, varName) == 0):
            print "Removing "+str(tmpEntryId)
            AdminConfig.remove(tmpEntryId)
            found = "true"
        #endIf 
    #endFor 
    
    if (cmp(found, "true") == 0):
        AdminConfig.save( )

    return