#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2008, 2009. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

import sys
import java
import string

def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split() #splits space-separated lists,
        else:
                tmpList = inStr.split("\n")   #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();         #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

class Scope:
    CLUSTER_SCOPE = 1
    CELL_SCOPE = 2
    NODE_SCOPE = 3
    SERVER_SCOPE = 4
# instance variables are...
#    cell
#    node
#    server
#    cluster
    
    def __init__(self, cluster, cell, node, server):
        self.cluster = cluster
        self.cell = cell
        self.node = node
        self.server = server
        self.containmentPathMap = {}
        self.containmentPathMap[Scope.CLUSTER_SCOPE] = "/Cell:" + cell + "/Cluster:" + cluster + "/"
        self.containmentPathMap[Scope.CELL_SCOPE] = "/Cell:" + cell + "/"
        self.containmentPathMap[Scope.NODE_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/"
        self.containmentPathMap[Scope.SERVER_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/Server:" + server + "/"

    def getConfigId(self, scopeLevel):
        return AdminConfig.getid(self.getContainmentPath(scopeLevel))

    def getContainmentPath(self, scopeLevel):
        return self.containmentPathMap[scopeLevel]


def setSystemProperties(scope, props):
    if (props == None):
        return
    
    serverCfgId = scope.getConfigId(Scope.SERVER_SCOPE)
    myJVM = AdminConfig.list("JavaVirtualMachine", serverCfgId)
    
    if (myJVM == None):
        print "No suitable JVM was found for Cell=" + scope.cell + ", Node=" + scope.node + ", Server=" + scope.server
    else:
        nameValPairs = props.items()
        for i in range(len(nameValPairs)):
            currProperty = [["name", nameValPairs[i][0]], ["value", nameValPairs[i][1]]]

            allPropsString = AdminConfig.list("TypedProperty", myJVM)
            allPropsArray = allPropsString.split(lineSeparator)
        
            print "Checking", allPropsArray, " for a match to",  props
            matched = "false"
            for j in range(len(allPropsArray)):
                if (allPropsArray[j] == ""):
                    continue
            
                systemPropertyName = AdminConfig.showAttribute(allPropsArray[j], "name")
                currPropertyName = nameValPairs[i][0]
                print "Checking: " + systemPropertyName + " against " + currPropertyName
                if (currPropertyName == systemPropertyName ):
                    print "Matched"
                    matched = "true"
                    print "Modifying: ", currProperty
                    AdminConfig.modify(allPropsArray[j],currProperty)
                    break
            
            if(matched != "true"):
                print "Creating:", currProperty
                AdminConfig.create("TypedProperty", myJVM, currProperty)
        
        AdminConfig.save()


cellName = sys.argv[0]
nodeName = sys.argv[1]
serverName = sys.argv[2]
wpsHost = sys.argv[3]
wpsRMIPort = sys.argv[4]
profilePath = sys.argv[5]

if(wpsHost == "NONE"):
    wpsHost = ""
if(wpsRMIPort == "NONE"):
    wpsRMIPort = ""

if  wpsHost == "WDPEOVERRIDE.qqq.qqq" and wpsRMIPort == "00000":
    wpsHost = "localhost"
    portDefFile = profilePath+'/properties/portdef.props'
    profileValsFile = profilePath+'/config.wbm/MonitorProfileVals.py'
    print "portDefFile:", portDefFile
    infile = open(portDefFile,"r")
    while infile:
        line = infile.readline()
        print "line:", line
        if line.find('BOOTSTRAP_ADDRESS') >= 0 :
            wpsRMIPort = line.split('=')[1].strip()
            print "wpsRMIPort:", wpsRMIPort
            break
        if not line : break
    infile = open(profileValsFile, "r")
    text = infile.read()
    infile.close()
    infile = open(profileValsFile, "w")
    text = text.replace('WDPEOVERRIDE.qqq.qqq','localhost')
    text = text.replace('00000',wpsRMIPort)
    infile.write(text)
    infile.close()


jvmProperties = {}
jvmProperties["DashboardBPCHost"] = wpsHost
jvmProperties["DashboardBPCRMIPort"] = wpsRMIPort

scope = Scope("clustername", cellName, nodeName, serverName)
print "scope="+str(scope)
setSystemProperties(scope, jvmProperties)
AdminConfig.save()
    
