#  Licensed Materials - Property of IBM
#  5724-M23,5724-M24 
#  Copyright IBM Corporation 2009, 2011. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

import java 

# get line separator 
lineSeparator = java.lang.System.getProperty('line.separator') 

def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split() #splits space-separated lists,
        else:
                tmpList = inStr.split("\n")   #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();         #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

S_COMMON_DB_DERBY_EMBEDDED = "DERBY_EMBEDDED"
S_COMMON_DB_DB2 = "DB2_DATASERVER"
S_COMMON_DB_ORACLE_11g = "Oracle11g"
S_COMMON_DB_ORACLE = "Oracle"
S_COMMON_DB_DB2_ZOS = "DB2UDBOS390"
S_COMMON_DB_DB2_ZOS_8 = "DB2UDBOS390_V8_1"
S_COMMON_DB_DB2_ZOS_9 = "DB2UDBOS390_V9_1"
S_COMMON_DB_MS_SQLSERVER = "MSSQLSERVER_MICROSOFT"

def environment(cellName, nodeName, serverName, varName, varValue, varDescription):
    print "INFO:  Entering environment"
    print "INFO:  varName="+varName+",varValue="+varValue+",varDescription="+varDescription
    
    id = None
    variableMapNodeId = None
    if(nodeName == None):    
        id = AdminConfig.getid("/Cell:"+cellName+"/" )
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellName+"/VariableMap:/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/"
    else:
        id = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/" )
        variableMapNodeId = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/VariableMap:/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/Node:"+nodeName+"/"
               
    wasVarDescription = ["description", varDescription]
    wasVarName = ["symbolicName", varName]
    wasVarValue = ["value", varValue]
    wasVarAttrs = [wasVarDescription, wasVarName, wasVarValue]
    if(cmp(varDescription,"") ==0):
        wasVarAttrs = [wasVarName, wasVarValue]
    #endif    
    installed    = "false"
    modified     = "false"

    varList = AdminConfig.list("VariableSubstitutionEntry", id )
    for tmpEntryId in wsadminToList(varList):
        #For Cell scope, we skip nodes
        if(nodeName == None):
            if(str(tmpEntryId).find("/nodes/") != -1):
                continue
            
        name = AdminConfig.showAttribute(tmpEntryId, "symbolicName")
        value = AdminConfig.showAttribute(tmpEntryId, "value")
        if (value != ""):
            print "%35s :  %-50s" % (name, value) 
        #endIf 
        if (cmp(name, varName) == 0):
            installed = "true"
            if (cmp(value, varValue) != 0):
                print "INFO: "+varName+" needs to be changed"
                AdminConfig.modify(tmpEntryId, wasVarAttrs )
                value = AdminConfig.showAttribute(tmpEntryId, "value" )
                modified = "true"
            #endIf 
            break
        #endIf 
    #endFor 
    flagOk = "false"
    if (cmp(modified, "false") == 0):
        if (cmp(installed, "true") == 0):
            flagOk = "true"
            print "INFO: "+varName+" with the correct value is already there, no action taken."
        else:
            print "INFO:  "+varName+" needs to be created"
            variableEntryId = AdminConfig.create("VariableSubstitutionEntry", variableMapNodeId, wasVarAttrs )
            name = AdminConfig.showAttribute(variableEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(variableEntryId, "value" )
        #endElse 
    #endIf 

    # Check for success
    if (cmp(modified, "true") == 0 or cmp(installed, "false") == 0):
        varList = AdminConfig.list("VariableSubstitutionEntry", id )
        for tmpEntryId in wsadminToList(varList):
            #For Cell scope, we skip nodes
            if(nodeName == None):
                if(str(tmpEntryId).find("/nodes/") != -1):
                    continue
            name = AdminConfig.showAttribute(tmpEntryId, "symbolicName" )
            value = AdminConfig.showAttribute(tmpEntryId, "value" )
            if (cmp(name, varName) == 0):
                if (cmp(value, varValue) == 0):
                    flagOk = "true"
                #endIf 
                break
            #endIf 
        #endFor 
    #endIf         
    if (cmp(flagOk, "true") == 0):
        AdminConfig.save( )
        print "INFO: "+varName+" successfully created/modified."
    else:
        print "ERROR: WebSphere Environment Variable "+varName+" could not be created/modified."
    #endElse 
    print "INFO:  Exiting environment"
    return
#endDef

def removeEnvironment(cellName, nodeName, serverName, varName):
    # Remove a WebSphere environment variable
    print "INFO:  Entering remove environment"
    print "INFO:  varName="+varName
    
    id = None
    if(nodeName == None):    
        id = AdminConfig.getid("/Cell:"+cellName+"/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/"
    else:
        id = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/" )
        print "INFO:  WebSphere Environment Variables for /Cell:"+cellName+"/Node:"+nodeName+"/"

    found = "false"
    varList = AdminConfig.list("VariableSubstitutionEntry", id )
    for tmpEntryId in wsadminToList(varList):
        #For Cell scope, we skip nodes
        if(nodeName == None):
            if(str(tmpEntryId).find("/nodes/") != -1):
                continue
            
        name = AdminConfig.showAttribute(tmpEntryId, "symbolicName")
        if (cmp(name, varName) == 0):
            print "Removing "+str(tmpEntryId)
            AdminConfig.remove(tmpEntryId)
            found = "true"
        #endIf 
    #endFor 
    
    if (cmp(found, "true") == 0):
        AdminConfig.save( )

    return

class Scope:
    CLUSTER_SCOPE = 1
    CELL_SCOPE = 2
    NODE_SCOPE = 3
    SERVER_SCOPE = 4
# instance variables are...
#    cell
#    node
#    server
#    cluster
    
    def __init__(self, cluster, cell, node, server):
        self.cluster = cluster
        self.cell = cell
        self.node = node
        self.server = server
        self.containmentPathMap = {}
        self.containmentPathMap[Scope.CLUSTER_SCOPE] = "/Cell:" + cell + "/Cluster:" + cluster + "/"
        self.containmentPathMap[Scope.CELL_SCOPE] = "/Cell:" + cell + "/"
        self.containmentPathMap[Scope.NODE_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/"
        self.containmentPathMap[Scope.SERVER_SCOPE] = "/Cell:" + cell + "/Node:" + node + "/Server:" + server + "/"

    def getConfigId(self, scopeLevel):
        return AdminConfig.getid(self.getContainmentPath(scopeLevel))

    def getContainmentPath(self, scopeLevel):
        return self.containmentPathMap[scopeLevel]

class AuthAliasInfo:
# instance variables are.... They are created in __init__ so they don't end up Class variables
#    name
#    desc
#    userId
#    password
    
    def __init__(self, name, desc, userId, password):
        print  "--- parms for datasourceAuthAliasInfo"
        print  "datasourceAuthAliasName         =  " + name
        print  "datasourceAuthAliasDesc         =  " + desc
        print  "datasourceAuthAliasUsername     =  " + userId
        print  "datasourceAuthAliasPassword     =  " + password
        self.name = name
        self.desc = desc
        self.userId = userId
        self.password = password

class JdbcProviderInfo:
# instance variables are.... They are created in __init__ so they don't end up Class variables
#    name
#    desc
#    implClass
#    driverFile
    
    def __init__(self, name, desc, implClass, driverFile, providerType):
        print  "--- parms for jdbcProviderInfo"
        print  "jdbcProviderName                =  " + name
        print  "jdbcProviderDesc                =  " + desc
        print  "jdbcProviderImplClass           =  " + implClass
        print  "jdbcProviderClasspath           =  " + driverFile
        print  "providerType                    =  " + providerType
        self.name = name
        self.desc = desc
        self.implClass = implClass
        self.driverFile = driverFile
        self.providerType = providerType

class DatasourceInfo:
# instance variables are.... They are created in __init__ so they don't end up Class variables
#    name
#    desc
#    jndiName
#    helperClassname
#    maxConnections
#    props #map
#    providerName
#    authAliasName
    
    def __init__(self, name, desc, jndiName, helperClassname, maxConnections, props, providerName, authAliasName, providerType):
        print  "--- parms for DatasourceInfo"
        print  "datasourceName                 =  " + name
        print  "datasourceDesc                 =  " + desc
        print  "datasourceJndi                 =  " + jndiName
        print  "datasourceHelperClassname      =  " + helperClassname
        print  "datasourceMaxConnections       =  " + maxConnections
        #print  "datasourceProps               =  " + props
        print  "jdbcProviderName               =  " + providerName
        print  "datasourceAuthAliasName        =  " + authAliasName
        print  "providerType                   =  " + providerType
        self.name = name
        self.desc = desc
        self.jndiName = jndiName
        self.helperClassname = helperClassname
        self.maxConnections = maxConnections
        self.props = props
        self.providerName = providerName
        self.authAliasName = authAliasName
        self.providerType = providerType
    
    def getProps(self):
        ret = []
        nameValPairs = self.props.items()
        for i in range(len(nameValPairs)):
            ret.append([["name", nameValPairs[i][0]], ["value", nameValPairs[i][1]]])
        return ret

def getJdbcProviderConfigId(scope, scopeLevel, providerName):
    providerConfigId = AdminConfig.getid(scope.getContainmentPath(scopeLevel) + "JDBCProvider:" + providerName + "/")
    
    if (providerConfigId == ""):
        print "Did not find JDBC Provider: " + providerName
        return None
    else:
        print "Found JDBC Provider: " + providerConfigId
        return providerConfigId

def createJdbcProvider(scope, scopeLevel, providerInfo):
    providerConfigId = getJdbcProviderConfigId(scope, scopeLevel, providerInfo.name)
    
    if (providerConfigId == None): # go ahead and create it
        db2ProviderImplClass = "com.ibm.db2.jcc.DB2XADataSource"
        providerCreateArgs = None
        if (providerInfo.implClass == db2ProviderImplClass):
            providerCreateArgs = [["name", providerInfo.name], ["providerType", providerInfo.providerType], ["description", providerInfo.desc], ["implementationClassName", providerInfo.implClass], ["classpath", providerInfo.driverFile], ["nativepath", "${DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH}"]]
        else:
            providerCreateArgs = [["name", providerInfo.name], ["providerType", providerInfo.providerType], ["description", providerInfo.desc], ["implementationClassName", providerInfo.implClass], ["classpath", providerInfo.driverFile]]
        parent = scope.getConfigId(scopeLevel)
        print "Create JDBC Provider with:", providerCreateArgs
        providerConfigId = AdminConfig.create("JDBCProvider", parent, providerCreateArgs)
    
    return providerConfigId

def getAuthenticationAliasConfigId(aliasName):
    print "List all Aliases"
    allAuthAliasString = AdminConfig.list("JAASAuthData")
    allAuthAliasArray = allAuthAliasString.split(lineSeparator)
    aliasConfigId = None
    
    print "Checking Aliases for a match to " + aliasName
    for i in range(len(allAuthAliasArray)):
        print "allAuthAliasArray[i]="+str(allAuthAliasArray[i])
        if(len(allAuthAliasArray[i]) > 0):
            if (AdminConfig.showAttribute(allAuthAliasArray[i], "alias") == aliasName):
                print "Found Alias: " + allAuthAliasArray[i]
                aliasConfigId = allAuthAliasArray[i]
    
    return aliasConfigId

def createAuthenticationAlias(authAliasInfo):
    aliasConfigId = getAuthenticationAliasConfigId(authAliasInfo.name)
    
    if (aliasConfigId == None): # go ahead and create it
        aliasCreateArgs = [["alias", authAliasInfo.name], ["description", authAliasInfo.desc], ["userId", authAliasInfo.userId], ["password", authAliasInfo.password]]
        parent = AdminConfig.getid("/Security:/")
        print "Create Alias with: alias: " + authAliasInfo.name + " description: " + authAliasInfo.desc + " userId: " + authAliasInfo.userId
        aliasConfigId = AdminConfig.create("JAASAuthData", parent, aliasCreateArgs)
    
    return aliasConfigId

# Queries WAS for datasource existance
def getDatasourceConfigId(scope, scopeLevel, providerName, datasourceName):
    datasourceConfigId = AdminConfig.getid(scope.getContainmentPath(scopeLevel) + "JDBCProvider:" + providerName + "/DataSource:" + datasourceName + "/")
    if (datasourceConfigId == ""):
        print "Did not find Datasource: " + datasourceName + " under " + providerName
        return None
    else:
        print "Found Datasource: " + datasourceConfigId + " under " + providerName
        return datasourceConfigId
    
def createDatasource(scope, scopeLevel, datasourceInfo):
    print ">> createDatasource"
    datasourceConfigId = getDatasourceConfigId(scope, scopeLevel, datasourceInfo.providerName, datasourceInfo.name)
    
    if (datasourceConfigId == None): # go ahead and create it
        nameAttr = ["name", datasourceInfo.name]
        providerTypeAttr = ["providerType", datasourceInfo.providerType]
        descAttr = ["description", datasourceInfo.desc]
        jndiAttr = ["jndiName", datasourceInfo.jndiName]
        statementCacheSizeAttr = ["statementCacheSize","100"]
        helpAttr = ["datasourceHelperClassname", datasourceInfo.helperClassname]
        authAttr = ["authDataAlias", datasourceInfo.authAliasName]
        authMechAttr = ["authMechanismPreference", "BASIC_PASSWORD"]
        mappingAttr = ["mapping", [["authDataAlias", datasourceInfo.authAliasName], ["mappingConfigAlias", "DefaultPrincipalMapping"]]]
        
        datasourceCreateArgs = [nameAttr, providerTypeAttr, descAttr, jndiAttr, statementCacheSizeAttr, helpAttr, authAttr, authMechAttr, mappingAttr]
        
        providerId = getJdbcProviderConfigId(scope, scopeLevel, datasourceInfo.providerName)
        if (providerId != None):
            print "Got the provider"
            print "Create Datasource with:", datasourceCreateArgs
            datasourceConfigId = AdminConfig.create("DataSource", providerId, datasourceCreateArgs)
            
            # now update the pool
            print "Update the pool for:", datasourceCreateArgs
            datasourcePool = AdminConfig.list("ConnectionPool", datasourceConfigId)
            AdminConfig.modify(datasourcePool, [["maxConnections", datasourceInfo.maxConnections]])
            
            # now update the properties
            print "Update the properties for:", datasourceCreateArgs
            propSetConfigId = AdminConfig.create("J2EEResourcePropertySet", datasourceConfigId, [])
            props = datasourceInfo.getProps()
            for i in range(len(props)):
                AdminConfig.create("J2EEResourceProperty", propSetConfigId, props[i])

def setSystemProperties(scope, props):
    if (props == None):
        return
    
    serverCfgId = scope.getConfigId(Scope.SERVER_SCOPE)
    myJVM = AdminConfig.list("JavaVirtualMachine", serverCfgId)
    
    if (myJVM == None):
        print "No suitable JVM was found for Cell=" + scope.cell + ", Node=" + scope.node + ", Server=" + scope.server
    else:
        nameValPairs = props.items()
        for i in range(len(nameValPairs)):
            currProperty = [["name", nameValPairs[i][0]], ["value", nameValPairs[i][1]]]

            allPropsString = AdminConfig.list("TypedProperty", myJVM)
            allPropsArray = allPropsString.split(lineSeparator)
        
            print "Checking", allPropsArray, " for a match to",  props
            matched = "false"
            for j in range(len(allPropsArray)):
                if (allPropsArray[j] == ""):
                    continue
            
                systemPropertyName = AdminConfig.showAttribute(allPropsArray[j], "name")
                currPropertyName = nameValPairs[i][0]
                print "Checking: " + systemPropertyName + " against " + currPropertyName
                if (currPropertyName == systemPropertyName ):
                    print "Matched"
                    matched = "true"
                    print "Modifying: ", currProperty
                    AdminConfig.modify(allPropsArray[j],currProperty)
                    break
            
            if(matched != "true"):
                print "Creating:", currProperty
                AdminConfig.create("TypedProperty", myJVM, currProperty)
        
        AdminConfig.save()
def removeSystemProperties(scope, props):
    if (props == None):
        return
    
    serverCfgId = scope.getConfigId(Scope.SERVER_SCOPE)
    myJVM = AdminConfig.list("JavaVirtualMachine", serverCfgId)
    
    if (myJVM == None):
        print "No suitable JVM was found for Cell=" + scope.cell + ", Node=" + scope.node + ", Server=" + scope.server
    else:
        allPropsString = AdminConfig.list("TypedProperty", myJVM)
        allPropsArray = allPropsString.split(lineSeparator)
        propConfigId = None
        
        nameValPairs = props.items()
        print "Checking", allPropsArray, " for a match to",  props
        for i in range(len(allPropsArray)):
            if (allPropsArray[i] == ""):
                continue
            
            currPropertyName = AdminConfig.showAttribute(allPropsArray[i], "name")
            for j in range(len(nameValPairs)):
                currCheckPropertyName = nameValPairs[j][0]
                print "Checking: " + currPropertyName + " against " + currCheckPropertyName
                if (currPropertyName == currCheckPropertyName):
                    print "Matched"
                    AdminConfig.remove(allPropsArray[i])

def setJDBCDriverPathVar(scopeString,cellName,nodeName,wbmDBType,JDBC_DRIVER_PATH):
    serverName = ""
    if(wbmDBType == S_COMMON_DB_DB2 or wbmDBType.startswith(S_COMMON_DB_DB2_ZOS):
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"DB2UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"DB2390_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"DB2_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "NODE"):
            environment(cellName,nodeName,serverName,"DB2UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,nodeName,serverName,"UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,nodeName,serverName,"DB2390_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,nodeName,serverName,"DB2_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    
    elif(wbmDBType == S_COMMON_DB_ORACLE_11g or wbmDBType.startswith(S_COMMON_DB_ORACLE)):
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"ORACLE_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "NODE"):
            environment(cellName,nodeName,serverName,"ORACLE_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")

    elif(wbmDBType == S_COMMON_DB_MS_SQLSERVER):
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"MICROSOFT_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"MICROSOFT_JDBC_DRIVER_NATIVEPATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "NODE"):
            environment(cellName,nodeName,serverName,"MICROSOFT_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,nodeName,serverName,"MICROSOFT_JDBC_DRIVER_NATIVEPATH",JDBC_DRIVER_PATH,"")


def createJDBCProviderComponent(scopeString,cellName,nodeName,serverName,wbmDBType,jdbcProviderName,jdbcProviderDesc,JDBC_DRIVER_PATH):
    scope = Scope("clusterName", cellName, nodeName, serverName)
    
    providerType = ""
    jdbcProviderImplClass = None
    jdbcProviderClassPath = None
    if (wbmDBType == S_COMMON_DB_DERBY_EMBEDDED):
        providerType = "Derby JDBC Provider"
        jdbcProviderImplClass = "org.apache.derby.jdbc.EmbeddedXADataSource"
        jdbcProviderClassPath = "${DERBY_JDBC_DRIVER_PATH}/derby.jar"
        environment(cellName,nodeName,serverName,"DERBY_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"DERBY_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    elif(wbmDBType == S_COMMON_DB_DB2 or wbmDBType.startswith(S_COMMON_DB_DB2_ZOS)):
        providerType = "DB2 Universal JDBC Driver Provider"
        jdbcProviderImplClass = "com.ibm.db2.jcc.DB2XADataSource"
        jdbcProviderClassPath = "${DB2UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc4.jar;${UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc_license_cu.jar;${DB2UNIVERSAL_JDBC_DRIVER_PATH}/db2jcc_license_cisuz.jar"
        environment(cellName,nodeName,serverName,"DB2UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        environment(cellName,nodeName,serverName,"UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        environment(cellName,nodeName,serverName,"DB2390_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        environment(cellName,nodeName,serverName,"DB2_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"DB2UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"UNIVERSAL_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"DB2390_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
            environment(cellName,None,serverName,"DB2_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    elif(wbmDBType == S_COMMON_DB_ORACLE_11g or wbmDBType.startswith(S_COMMON_DB_ORACLE)):
        providerType = "Oracle JDBC Driver"
        jdbcProviderImplClass = "oracle.jdbc.xa.client.OracleXADataSource"
        jdbcProviderClassPath = "${ORACLE_JDBC_DRIVER_PATH}/ojdbc6.jar"
        environment(cellName,nodeName,serverName,"ORACLE_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"ORACLE_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
    elif(wbmDBType == S_COMMON_DB_MS_SQLSERVER):
        providerType = "Microsoft SQL Server JDBC Driver"
        jdbcProviderImplClass = "com.microsoft.sqlserver.jdbc.SQLServerXADataSource"
        jdbcProviderClassPath = "${MICROSOFT_JDBC_DRIVER_PATH}/sqljdbc4.jar"
        environment(cellName,nodeName,serverName,"MICROSOFT_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        if(scopeString == "CELL"):
            environment(cellName,None,serverName,"MICROSOFT_JDBC_DRIVER_PATH",JDBC_DRIVER_PATH,"")
        
    jdbcProviderInfo = JdbcProviderInfo(jdbcProviderName, jdbcProviderDesc, jdbcProviderImplClass, jdbcProviderClassPath, providerType)
    
    # Creates the JDBC Provider.
    if(scopeString =="CELL"):
        providerConfigId = createJdbcProvider(scope, Scope.CELL_SCOPE, jdbcProviderInfo)
    elif(scopeString =="NODE"):
        providerConfigId = createJdbcProvider(scope, Scope.NODE_SCOPE, jdbcProviderInfo)
    elif(scopeString =="SERVER"):
        providerConfigId = createJdbcProvider(scope, Scope.SERVER_SCOPE, jdbcProviderInfo)
    
    AdminConfig.save()
    
def createJDBCAuthAlias(datasourceAuthAliasName,datasourceAuthAliasDesc,datasourceAuthAliasUsername,datasourceAuthAliasPassword):
    datasourceAuthAliasInfo = AuthAliasInfo(datasourceAuthAliasName, datasourceAuthAliasDesc, datasourceAuthAliasUsername, datasourceAuthAliasPassword)
    print "Create " + datasourceAuthAliasInfo.name + " if needed"
    createAuthenticationAlias(datasourceAuthAliasInfo)
    AdminConfig.save()

def createJDBCDatasource(scopeString,cellName,nodeName,serverName,wbmDBType, jdbcProviderName, datasourceAuthAliasName, datasourceName, datasourceDesc, datasourceJndi, datasourceProps, datasourceMaxConnections):
    scope = Scope("clusterName", cellName, nodeName, serverName)
    
    providerType = ""
    datasourceHelperClassname = None
    if (wbmDBType == S_COMMON_DB_DERBY_EMBEDDED):
        providerType = "Derby JDBC Provider"
        datasourceHelperClassname = "com.ibm.websphere.rsadapter.DerbyDataStoreHelper"
    elif(wbmDBType == S_COMMON_DB_DB2 or wbmDBType.startswith(S_COMMON_DB_DB2_ZOS)):
        providerType = "DB2 Universal JDBC Driver Provider"
        datasourceHelperClassname = "com.ibm.websphere.rsadapter.DB2UniversalDataStoreHelper"
    elif(wbmDBType == S_COMMON_DB_ORACLE_11g or wbmDBType.startswith(S_COMMON_DB_ORACLE)):
        providerType = "Oracle JDBC Driver"
        datasourceHelperClassname = "com.ibm.websphere.rsadapter.Oracle11gDataStoreHelper"
    elif(wbmDBType == S_COMMON_DB_MS_SQLSERVER):
        providerType = "Microsoft SQL Server JDBC Driver"
        datasourceHelperClassname = "com.ibm.websphere.rsadapter.MicrosoftSQLServerDataStoreHelper"
    
    #TODO should do it during datasource creation check for linux and type 2
    # Set WAS Environment variable for DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH
    #if(DB2_JDBC_DRIVER_NATIVEPATH != ""):
    #    environment(cellName,None,serverName,"DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH",DB2_JDBC_DRIVER_NATIVEPATH,"")
    #    environment(cellName,nodeName,serverName,"DB2UNIVERSAL_JDBC_DRIVER_NATIVEPATH",DB2_JDBC_DRIVER_NATIVEPATH,"")

    datasourceInfo =  DatasourceInfo(datasourceName, datasourceDesc, datasourceJndi, datasourceHelperClassname, datasourceMaxConnections, datasourceProps, jdbcProviderName, datasourceAuthAliasName, providerType)
    
    # Creates the JDBC datasource.
    if(scopeString =="CELL"):
        datasourceConfigId = createDatasource(scope, Scope.CELL_SCOPE, datasourceInfo)
    elif(scopeString =="NODE"):
        datasourceConfigId = createDatasource(scope, Scope.NODE_SCOPE, datasourceInfo)
    elif(scopeString =="SERVER"):
        datasourceConfigId = createDatasource(scope, Scope.SERVER_SCOPE, datasourceInfo)

    AdminConfig.save()

def getDatasourceProps(wbmDBType, databaseName, datasourceDriverType, datasourceServerName, datasourcePortNumber, extraPropNames, extraPropValues):
    #Add custom property for webSphereDefaultIsolationLevel:2 (Cursor Stability (CS))
    datasourceProps = {"webSphereDefaultIsolationLevel":"2"}
    if(wbmDBType == S_COMMON_DB_DERBY_EMBEDDED):
        datasourceProps["databaseName"] = databaseName
    if(wbmDBType == S_COMMON_DB_DB2 or wbmDBType.startswith(S_COMMON_DB_DB2_ZOS)):
        datasourceProps["databaseName"] = databaseName
        datasourceProps["driverType"] = datasourceDriverType
        datasourceProps["serverName"] = datasourceServerName
        datasourceProps["portNumber"] = datasourcePortNumber
    if (wbmDBType == S_COMMON_DB_ORACLE_11g or wbmDBType.startswith(S_COMMON_DB_ORACLE)):
        #type 2 = OCI, type 4 = thin.  
        if(datasourceDriverType == "2"):
            datasourceProps["URL"] = "jdbc:oracle:oci:@"+datasourceServerName+":"+datasourcePortNumber+":"+databaseName
        else:
            datasourceProps["URL"] = "jdbc:oracle:thin:@"+datasourceServerName+":"+datasourcePortNumber+":"+databaseName

    if (wbmDBType == S_COMMON_DB_MS_SQLSERVER):
        datasourceProps["databaseName"] = databaseName
        datasourceProps["serverName"] = datasourceServerName
        datasourceProps["portNumber"] = datasourcePortNumber
        
        
    #Add extra properties
    if(extraPropNames != None and extraPropValues != None):
        extraPropNameList =extraPropNames.split(',')
        extraPropValueList =extraPropValues.split(',')
        for i in range(len(extraPropNameList)):
            print "adding "+extraPropNameList[i]+" "+extraPropValueList[i]
            datasourceProps[extraPropNameList[i]] = extraPropValueList[i] 
    
    return datasourceProps

#############################################
#            MAIN METHOD                    #
#############################################
FileName = "jdbcCommandManager.py"

print "entry"
#============================================
# Display usage if input parameters incorrect 
#============================================ 
if (len(sys.argv) < 1):
        print " "
        print "================================================================="
        print "Usage for file: "+FileName
        print ""
        print "Input Parameters: <command> <scopeString> <cellName> <nodeName> <serverName> <wbmDBType> <jdbcProviderName> <jdbcProviderDesc> <jdbcProviderClassPath>"
        print ""
        print "<command> is a command to execute. Valid commands are createJDBCProvider, createJDBCAuthAlias, createJDBCDatasource"
        print "<scopeString> is the string of CELL, NODE or SERVER."
        print "<cellName> is the WebSphere Application Server cell name."
        print "<nodeName> is the WebSphere Application Server node name."
        print "<serverName> is the name of the server in WebSphere Application Server."
        print "<wbmDBType> is the type of databae product."
        print "<jdbcProviderName> is the name of the JDBC provider."
        print "<jdbcProviderDesc> is the description of the JDBC provider."
        print "<jdbcProviderClassPath> is the path of the folder where JDBC driver jars sit."
        print ""
        print "================================================================="
        raise RuntimeError, "Invalid number of parameters."
#endIf 

command = sys.argv[0]

if(command == "createJDBCProvider"):
    print "Run createJDBCProvider command."
    scopeString = sys.argv[1]
    cellName = sys.argv[2]
    nodeName = sys.argv[3]
    serverName = sys.argv[4]
    wbmDBType = sys.argv[5]
    jdbcProviderName = sys.argv[6]
    jdbcProviderDesc = sys.argv[7]
    wbmDBJDBCClasspath = sys.argv[8]

    createJDBCProviderComponent(scopeString,cellName,nodeName,serverName,wbmDBType,jdbcProviderName,jdbcProviderDesc,wbmDBJDBCClasspath)

elif(command == "deleteJDBCProvider"):
    print "Run deleteJDBCProvider command."
    scopeString = sys.argv[1]
    cellName = sys.argv[2]
    nodeName = sys.argv[3]
    serverName = sys.argv[4]
    jdbcProviderName = sys.argv[5]

    print "Removing Datasource Provider: " + jdbcProviderName
    scope = Scope("clusterName", cellName, nodeName, serverName)

    configId = None
    if(scopeString =="CELL"):
        configId = getJdbcProviderConfigId(scope, Scope.CELL_SCOPE, jdbcProviderName)
    elif(scopeString =="NODE"):
        configId = getJdbcProviderConfigId(scope, Scope.NODE_SCOPE, jdbcProviderName)
    elif(scopeString =="SERVER"):
        configId = getJdbcProviderConfigId(scope, Scope.SERVER_SCOPE, jdbcProviderName)

    if (configId == None):
        print "Datasource Provider not present.  Continuing"
    else:
        AdminConfig.remove(configId)
        print "Datasource Provider removed"
        AdminConfig.save()
            
elif(command == "createJDBCAuthAlias"):
    print "Run createJDBCAuthAlias command."
    datasourceAuthAliasName = sys.argv[1]
    datasourceAuthAliasDesc = sys.argv[2]
    datasourceAuthAliasUsername = sys.argv[3]
    datasourceAuthAliasPassword = sys.argv[4]

    createJDBCAuthAlias(datasourceAuthAliasName,datasourceAuthAliasDesc,datasourceAuthAliasUsername,datasourceAuthAliasPassword)

elif(command == "deleteJDBCAuthAlias"):
    print "Run deleteJDBCAuthAlias command."
    datasourceAuthAliasName = sys.argv[1]
    print "Removing Authentication Alias: " + datasourceAuthAliasName
    configId = getAuthenticationAliasConfigId(datasourceAuthAliasName)
    if (configId == None):
        print "Authentication Alias not present.  Continuing"
    else:
        AdminConfig.remove(configId)
        print "Authentication Alias removed"
        AdminConfig.save()
        
elif(command == "createJDBCDatasource"):
    print "Run createJDBCDatasource command."
    scopeString = sys.argv[1]
    cellName = sys.argv[2]
    nodeName = sys.argv[3]
    serverName = sys.argv[4]
    wbmDBType = sys.argv[5]
    jdbcProviderName = sys.argv[6]
    datasourceAuthAliasName = sys.argv[7]
    datasourceName = sys.argv[8]
    datasourceDesc = sys.argv[9]
    datasourceJndi = sys.argv[10]
    databaseName = sys.argv[11]
    datasourceDriverType = sys.argv[12]
    datasourceServerName = sys.argv[13]
    datasourcePortNumber = sys.argv[14]
    datasourceMaxConnections = sys.argv[15]

    #Check for extra custom properties
    extraPropNames = None
    extraPropValues = None
    if(len(sys.argv) >= 17):
        extraPropNames = sys.argv[16]
        extraPropValues = sys.argv[17]
        
    datasourceProps = getDatasourceProps(wbmDBType, databaseName, datasourceDriverType, datasourceServerName, datasourcePortNumber, extraPropNames, extraPropValues)
    
    createJDBCDatasource(scopeString,cellName,nodeName,serverName,wbmDBType, jdbcProviderName, datasourceAuthAliasName, datasourceName, datasourceDesc, datasourceJndi, datasourceProps, datasourceMaxConnections)

elif(command == "setJDBCDriverPathVar"):
    print "Run setJDBCDriverPathVar command."
    scopeString = sys.argv[1]
    nodeName = sys.argv[2]
    wbmDBType = sys.argv[3]
    wbmDBJDBCClasspath = sys.argv[4]

    cellName = AdminConfig.showAttribute(AdminConfig.list("Cell"), "name")
    print "cellName="+cellName
    
    setJDBCDriverPathVar(scopeString,cellName,nodeName,wbmDBType,wbmDBJDBCClasspath)
    
else:
    print "Invalid command:"+command
    
AdminConfig.save()
