#***************************************************************
# Licensed Material - Property of IBM
#
# 5724-M24
#
# Copyright IBM Corp. 2008.  All rights reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM
# Corp.
#***************************************************************

import java
import re
import tempfile
import os.path
import time

from java.util import ResourceBundle
from java.text import MessageFormat
from java.lang import Boolean
from java.util import Properties
from java.util import List
from java.util import LinkedList
from java.util import Hashtable
from java.io import FileInputStream
from java.io import FileOutputStream
from java.util.jar import *
from javax.management import Attribute;
from javax.management import AttributeList;
from com.ibm.websphere.management.application import AppManagement;
from com.ibm.websphere.management.application import AppManagementProxy;


from JythonUtilitiesBundleKeys import JythonUtilitiesBundleKeys as JBK
from Consts import Consts as CONSTS



rb = ResourceBundle.getBundle(JBK.BUNDLE_NAME)





class ConfigObjectHolder:
	AdminApp = None
	AdminControl = None
	AdminConfig = None
	AdminTask = None
	

class CellScope:
	def __init__(self, cell):
		self.cell = cell
		
	def __str__(self):
		return "Cell:" + str(self.cell)
	
	def getContainmentPath(self):
		return "/Cell:" + str(self.cell) + "/"

	def getQueryPath(self):
		return "/Cell:" + str(self.cell) + "/"
	
class NodeScope:
	def __init__(self, cell, node):
		self.cell = cell
		self.node = node
		
	def __str__(self):
		return "Cell:" + str(self.cell) + " Node:" + str(self.node)
	
	def getContainmentPath(self):
		return "/Cell:" + str(self.cell) + "/Node:" + str(self.node) + "/"

class ServerScope:
	def __init__(self, cell, node, server):
		self.cell = cell
		self.node = node
		self.server = server
		
	def __str__(self):
		return "Cell:" + str(self.cell) + " Node:" + str(self.node) + " Server:" + str(self.server)
	
	def getContainmentPath(self):
		return "/Cell:" + str(self.cell) + "/Node:" + str(self.node) + "/Server:" + str(self.server) + "/"

	def getQueryPath(self):
		return "cell=" + str(self.cell) + ",node=" + str(self.node) + ",server=" + str(self.server) + "/"

class ClusterScope:
	def __init__(self, cell, cluster):
		self.cell = cell
		self.cluster = cluster
		
	def __str__(self):
		return "Cell:" + str(self.cell) + " Cluster:" + str(self.cluster)
	
	def getContainmentPath(self):
		return "/ServerCluster:" + str(self.cluster) + "/"
	
class ProcessScope:
	def __init__(self, cell, node, process):
		self.cell = cell
		self.node = node
		self.process = process
		
	def __str__(self):
		return "Cell:" + str(self.cell) + " Node:" + str(self.node) + " Process:" + str(self.process)
	
	def getMBeanQuery(self):
		return "cell=" + str(self.cell) + ",node=" + str(self.node) + ",process=" + str(self.process)
	

class Version:
	def __str__(self):
		if (self.first == None):
			return "INVALID Version"
		
		return str(self.first) + "." + str(self.second) + "." + str(self.third) + "." + str(self.fourth)
	
	def __init__(self, version):
		if (version):
			vals = version.split(".")
			if (len(vals) >= 1):
				self.first = int(vals[0])
			else:
				self.first = 0
			
			if (len(vals) >= 2):
				self.second = int(vals[1])
			else:
				self.second = 0
			
			if (len(vals) >= 3):
				self.third = int(vals[2])
			else:
				self.third = 0
			
			if (len(vals) >= 4):
				self.fourth = int(vals[3])
			else:
				self.fourth = 0
	
	def __cmp__(self, other):
		diff = self.first - other.first
		if (diff != 0):
			return diff
		
		diff = self.second - other.second
		if (diff != 0):
			return diff
		
		diff = self.third - other.third
		if (diff != 0):
			return diff
		
		diff = self.fourth - other.fourth
		return diff

	

class WSAdminUtils:
	def getAttribute(self, cfgID, attributeName):
		attr = ConfigObjectHolder.AdminConfig.show(cfgID, attributeName)
		
		regex = "\\[" + attributeName + " (?P<value>.+)\\]"
		compiled = re.compile(regex)
		matchResult = compiled.match(attr)
		
		if (matchResult != None):
			fullValue = matchResult.group("value")
			strippedOfQuotes = fullValue
			
			if (fullValue[0] == "\""):
				strippedOfQuotes = fullValue[1:len(fullValue)-1]
			
			if (strippedOfQuotes[0] == "["):
				strippedOfBrackets = strippedOfQuotes[1:len(strippedOfQuotes)-1]
				arrayedValuesMessedUpChars = strippedOfBrackets.split(")")
				fixedArrayedValues = []
				
				for badArrayedValue in arrayedValuesMessedUpChars:
					if (len(badArrayedValue) < 1):
						continue
					
					if (badArrayedValue[0] == " "):
						fixedArrayedValues.append(badArrayedValue[1:] + ")")
					else:
						fixedArrayedValues.append(badArrayedValue + ")")
				
				return fixedArrayedValues
				
			else:
				return strippedOfQuotes
		
		return None
	
	def getAllEventGroupLists(self):
		unmassagedList = ConfigObjectHolder.AdminConfig.list("EventGroupProfileList").splitlines()
		ret = []
		for entry in unmassagedList:
			ret.append(entry[1:len(entry)-1])
		
		return ret
	
	def getEventGroupList(self, scope):
		ancestor = ConfigObjectHolder.AdminConfig.getid(str(scope.getContainmentPath()))
		unmassagedList = ConfigObjectHolder.AdminConfig.list("EventGroupProfileList", ancestor).splitlines()

		if (len(unmassagedList) != 1):
			return None
		else:
			return unmassagedList[0][1:len(unmassagedList[0])-1]
	
	def getAllEventGroups(self, eventGroupList):
		unmassagedList = ConfigObjectHolder.AdminConfig.list("EventGroupProfile", eventGroupList).splitlines()
		ret = []
		for entry in unmassagedList:
			ret.append(entry[1:len(entry)-1])
		
		return ret
	
	def getArchivingEventGroup(self, eventGroupList):
		unmassagedList = ConfigObjectHolder.AdminConfig.list("EventGroupProfile", eventGroupList).splitlines()
		ret = None
		for entry in unmassagedList:
			cfgID = entry.replace('"', '')
			name = self.getAttribute(cfgID, "eventGroupName")

			if (name == CONSTS.ARCHIVING_EVENT_GROUP_NAME):
				ret = cfgID
				break
		
		return ret
	
	def enableEventArchiving(self, scope):
		owningList = self.getEventGroupList(scope)
		routingDefinition = self.getDatasourceJNDIName() + ":" + self.getMonitorSchemaName() + ":true"
		
		compatibilityFalse = [["description", "Compatibility mode with previous event service transmission protocol"], ["name", "compatibilityMode"], ["type", "java.lang.Boolean"], ["required", Boolean("false")], ["value", Boolean("false")]]
		selectorClass = [["description", "Fully qualified name of a custom event selector filter class"], ["name", "customEventSelectorFilterClass"], ["type", "java.lang.String"], ["required", Boolean("false")], ["value", "com.ibm.wbimonitor.router.record.ceiext.EventRecordingSelector"]]
		
		params = [["eventGroupName", CONSTS.ARCHIVING_EVENT_GROUP_NAME], ["eventSelectorString", routingDefinition], ["persistEvents", Boolean("false")]]
		
		newGroup = ConfigObjectHolder.AdminConfig.create("EventGroupProfile", owningList, params)

		propSetConfigId = ConfigObjectHolder.AdminConfig.create("J2EEResourcePropertySet", newGroup, [])
		ConfigObjectHolder.AdminConfig.create("J2EEResourceProperty", propSetConfigId, compatibilityFalse)
		ConfigObjectHolder.AdminConfig.create("J2EEResourceProperty", propSetConfigId, selectorClass)
		
		ConfigObjectHolder.AdminConfig.save()
		
		print
		self.synchronizeConfig()
		print
		
		self.reloadCEIConfig()
	
	def getDatasourceJNDIName(self):
		# determine a datasource name based on the cell name
		return "jdbc/wbm/MonitorDatabase"
	
	def getMonitorSchemaName(self):
		# determine the schema name
		return "Monitor"
	
	def disableEventArchiving(self, scope):
		owningList = self.getEventGroupList(scope)
		groupToDelete = self.getArchivingEventGroup(owningList)
		
		ConfigObjectHolder.AdminConfig.remove(str(groupToDelete))
		ConfigObjectHolder.AdminConfig.save()
		
		print
		self.synchronizeConfig()
		print
				
		self.reloadCEIConfig()
	
	def reloadCEIConfig(self):
		print 
		self.stopApp("EventService")
		print
		
		ceiReloaders = ConfigObjectHolder.AdminControl.queryNames("type=ConfigUtil,*").splitlines()
		
		for ceiReloader in ceiReloaders:
			try:
				print MessageFormat.format(rb.getString(JBK.RELOADING_CEI), [str(self.determineScopeFromMBean(str(ceiReloader)))])
				
				ConfigObjectHolder.AdminControl.invoke(ceiReloader, "reloadResources")
			except Exception, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_RELOADING_CEI), [str(ceiReloader), str(e)])
			except Warning, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_RELOADING_CEI), [str(ceiReloader), str(e)])
			except:
				print MessageFormat.format(rb.getString(JBK.ERROR_RELOADING_CEI), [str(ceiReloader), "?"])
		
		print
		self.startApp("EventService")
		print
	
	def stopApp(self, appName):
		appManagers = ConfigObjectHolder.AdminControl.queryNames("type=ApplicationManager,*").splitlines()
		
		for appManager in appManagers:
			try:
				# make sure the app exists on this app manager
				appManagerScope = self.determineScopeFromMBean(str(appManager))
				
				query = appManagerScope.getMBeanQuery() + ",name=" + appName + ",*"
				apps = ConfigObjectHolder.AdminControl.queryNames(query).splitlines()
				if (apps):
					print MessageFormat.format(rb.getString(JBK.STOPPING_APP), [appName, str(appManagerScope)])
					ConfigObjectHolder.AdminControl.invoke(appManager, "stopApplication", appName)
								
			except Exception, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_STOPPING_APP), [appName, str(appManager), str(e)])
			except Warning, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_STOPPING_APP), [appName, str(appManager), str(e)])
			except:
				print MessageFormat.format(rb.getString(JBK.ERROR_STOPPING_APP), [appName, str(appManager), "?"])
	
	def startApp(self, appName):
		appManagers = ConfigObjectHolder.AdminControl.queryNames("type=ApplicationManager,*").splitlines()
		
		for appManager in appManagers:
			try:
				# make sure the app exists on this app manager
				appManagerScope = self.determineScopeFromMBean(str(appManager))
				
				query = appManagerScope.getMBeanQuery() + ",name=" + appName + ",*"
				apps = ConfigObjectHolder.AdminControl.queryNames(query).splitlines()
				if (apps):
					print MessageFormat.format(rb.getString(JBK.STARTING_APP), [appName, str(appManagerScope)])
					ConfigObjectHolder.AdminControl.invoke(appManager, "startApplication", appName)
								
			except Exception, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_STARTING_APP), [appName, str(appManager), str(e)])
			except Warning, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_STARTING_APP), [appName, str(appManager), str(e)])
			except:
				print MessageFormat.format(rb.getString(JBK.ERROR_STARTING_APP), [appName, str(appManager), "?"])
	
	
	def updateAppFile(self, appName, jarFilePath):
		jarFileName = os.path.basename(jarFilePath)
		
		try:
			print 
			self.stopApp(appName)
			print 
			
			print MessageFormat.format(rb.getString(JBK.UPDATING_FILE), [appName, jarFileName, jarFilePath, str(e)])
			ConfigObjectHolder.AdminApp.update(appName, 'file', ['-operation', 'update', '-contents', jarFilePath, 'contenturi', jarFileName])
			
			ConfigObjectHolder.AdminConfig.save()
			
			print
			self.synchronizeConfig()
			print
			
			self.startApp(appName)
			
		except Exception, e:
			print MessageFormat.format(rb.getString(JBK.ERROR_UPDATING_FILE), [appName, jarFileName, jarFilePath, str(e)])
	
	def updateAppFiles(self, appName, zipFilePath):
		try:
			print 
			self.stopApp(appName)
			print 
			
			print MessageFormat.format(rb.getString(JBK.UPDATING_FILES), [appName, zipFilePath])
			ConfigObjectHolder.AdminApp.update(appName, 'partialapp', ['-contents', zipFilePath])
			
			ConfigObjectHolder.AdminConfig.save()
			
			print
			self.synchronizeConfig()
			print
			
			self.startApp(appName)

		except Exception, e:
			print MessageFormat.format(rb.getString(JBK.ERROR_UPDATING_FILES), [appName, zipFilePath, str(e)])
	
	def exportApp(self, appName, directory):
		localTime = time.localtime()
		dateString = "%.4i%.2i%.2i%.2i%.2i%.2i%.2i" % (str(localTime[0]), str(localTime[1]), str(localTime[2]), str(localTime[3]), str(localTime[4]), str(localTime[5]), str(localTime[6]))
		
		fullFilename = directory + '/' + appName + "_backup_" + dateString + ".ear"
		try:
			
			print MessageFormat.format(rb.getString(JBK.EXPORTING_APP), [appName, fullFilename])
			ConfigObjectHolder.AdminApp.export(appName, fullFilename)

		except Exception, e:
			print MessageFormat.format(rb.getString(JBK.ERROR_EXPORTING_APP), [appName, fullFilename, str(e)])
	
	def getAppFiles(self, appName, uris):
		try:
			appManagement = AppManagementProxy.getJMXProxyForClient(ConfigObjectHolder.AdminControl.getAdminClient())
			result = []
			
			for uri in uris:
				result.append(appManagement.getApplicationContents(appName, uri, Hashtable(), None))
			
			return result
			
		except Exception, e:
			print MessageFormat.format(rb.getString(JBK.ERROR_GETTING_FILES), [appName, str(uris), str(e)])
	
	KNOWN_LIBRARY_JARS = ["com.ibm.wbimonitor.server.common.jar", "com.ibm.wbimonitor.server.moderator.jar", "com.ibm.wbimonitor.server.modellogic.jar", "com.ibm.wbimonitor.server.moderator.serialmt.jar", "com.ibm.wbimonitor.server.moderator.serialst.jar"]
	def determineAppLibraryCodeVersionFiles(self, appName):
		try:
			jars = self.getAppFiles(appName, WSAdminUtils.KNOWN_LIBRARY_JARS)
			biggestVersionSoFar = None
			valset = 0
			count = 0
			
			for jar in jars:
				tName = tempfile.mktemp(suffix=(appName +  "_" + str(WSAdminUtils.KNOWN_LIBRARY_JARS[count]) + ".jar"))
				outstream = FileOutputStream(tName)
				outstream.write(jar)
				outstream.close()
				
				jarFile = JarFile(tName, Boolean("true"))
				manifest = jarFile.getManifest()
				bundleVersion = Version(manifest.getMainAttributes().getValue("Bundle-Version"))
				
				# TODO for some reason biggestVersionSoFar == None produces an error
				if (valset == 0):
					biggestVersionSoFar = bundleVersion
					valset = 1;
				elif (bundleVersion > biggestVersionSoFar):
					biggestVersionSoFar = bundleVersion
				
				jarFile.close()
				
				os.remove(tName)				
			
			return biggestVersionSoFar

		except Exception, e:
			print MessageFormat.format(rb.getString(JBK.ERROR_GETTING_FILES), [appName, str(WSAdminUtils.KNOWN_LIBRARY_JARS), str(e)])
	
	
	def synchronizeConfig(self):
		nodeAgents = ConfigObjectHolder.AdminControl.queryNames("type=NodeSync,*").splitlines()

		for nodeAgent in nodeAgents:
			try:
				# make sure the app exists on this app manager
				nodeAgentScope = self.determineScopeFromMBean(str(nodeAgent))
				
				print MessageFormat.format(rb.getString(JBK.SYNCHRONIZING_CONFIG), [str(nodeAgentScope)])
				ConfigObjectHolder.AdminControl.invoke(nodeAgent, "sync")
								
			except Exception, e:
				print MessageFormat.format(rb.getString(JBK.ERROR_SYNCHRONIZING_CONFIG), [str(nodeAgent), str(e)])
	
	
	MIN_WAS_VERSION_FOR_EVENT_ARCHIVING = Version("6.1.0.15")
	MIN_WBM_VERSION_FOR_EVENT_ARCHIVING = Version("6.2.0.0")
	def canWASSupportEventArchiving(self, eventGroupList):
		scope = self.determineScope(eventGroupList)
		nodes = self.determineNodes(scope)
		
		if (nodes):
			# check WAS version
			minWASVersion = self.getMinVersion(nodes, self.getWASVersion)
			if (minWASVersion):
				if (minWASVersion >= WSAdminUtils.MIN_WAS_VERSION_FOR_EVENT_ARCHIVING):
					# we passed the WAS check
					return 1
					
				else:
					return 0 # false
				
			else:
				return 0 # false
			
		else:
			# no nodes, so there's nothing to check
			return 0 # false
	
	def canWBMSupportEventArchiving(self, eventGroupList):
		scope = self.determineScope(eventGroupList)
		nodes = self.determineNodes(scope)
		
		if (nodes):
			# check WBM version
			minWBMVersion = self.getMinVersion(nodes, self.getWBMVersion)
			if (minWBMVersion):
				if (minWBMVersion >= WSAdminUtils.MIN_WBM_VERSION_FOR_EVENT_ARCHIVING):
					# we passed the WAS check
					return 1
					
				else:
					return 0 # false
				
			else:
				return 0 # false
			
		else:
			# no nodes, so there's nothing to check
			return 0 # false
	
	def determineNodes(self, scope):
		scopeClass = scope.__class__
		
		if (scopeClass == CellScope):
			return []
		elif (scopeClass == NodeScope):
			return [scope]
		elif (scopeClass == ServerScope):
			return [NodeScope(scope.cell, scope.node)]
		elif (scopeClass == ClusterScope):
			members = self.getClusterMembers(scope)
			ret = []
			
			for member in members:
				ret.append(NodeScope(scope.cell, self.getAttribute(member, "nodeName")))
			
			return ret
		else:
			return None
	
	def getClusterMembers(self, scope):
		if (scope.__class__ == ClusterScope):
			clusterID = ConfigObjectHolder.AdminConfig.getid(scope.getContainmentPath())
			ret = self.getAttribute(clusterID, "members")
			
			return ret
		else:
			return None
	
	def getMinVersion(self, nodes, versionGetterMethod):
		minVersionSoFar = None
		
		for node in nodes:
			currVersion = versionGetterMethod(node)
			
			if (currVersion):
				if (minVersionSoFar):
					if (currVersion < minVersionSoFar):
						minVersionSoFar = currVersion
				else:
					minVersionSoFar = currVersion
			else:
				return None
		
		return minVersionSoFar
	
	WAS_VERSION_KEY = "com.ibm.websphere.baseProductVersion"
	def getWASVersion(self, node):
		props = self.getNodeMetadataProperties(node)
		version = props.getProperty(WSAdminUtils.WAS_VERSION_KEY)
		ret = Version(version)
		
		return ret
	
	
	WBM_VERSION_KEY = "com.ibm.websphere.wbmFeaturePackProductVersion"
	def getWBMVersion(self, node):
		props = self.getNodeMetadataProperties(node)
		version = props.getProperty(WSAdminUtils.WBM_VERSION_KEY)
		ret = None
		
		if (version):
			ret = Version(version)
		
		return ret
	
	def getNodeMetadataProperties(self, node):
		theFile = tempfile.mktemp(suffix=".properties")
		ConfigObjectHolder.AdminConfig.extract("cells/" + node.cell + "/nodes/" + node.node + "/node-metadata.properties", str(theFile))
		
		props = Properties()
		infile = FileInputStream(str(theFile))
		props.load(infile)
		infile.close()
		
		os.remove(str(theFile))
		
		return props
	
	
	scopeName = "scope"
	cellName = "cell"
	nodeName = "node"
	serverName = "server"
	clusterName = "cluster"
	scopePattern = ".*\\((?P<" + scopeName + ">.+)\\|.+"
	scopeCompiled = re.compile(scopePattern)
	cellPattern = "^cells/(?P<" + cellName + ">[^/]+)$"
	cellCompiled = re.compile(cellPattern)
	nodePattern = "^cells/(?P<" + cellName + ">[^/]+)/nodes/(?P<" + nodeName + ">[^/]+)$"
	nodeCompiled = re.compile(nodePattern)
	serverPattern = "^cells/(?P<" + cellName + ">[^/]+)/nodes/(?P<" + nodeName + ">[^/]+)/servers/(?P<" + serverName + ">[^/]+)$"
	serverCompiled = re.compile(serverPattern)
	clusterPattern = "^cells/(?P<" + cellName + ">[^/]+)/clusters/(?P<" + clusterName + ">[^/]+)$"
	clusterCompiled = re.compile(clusterPattern)
	
	def determineScope(self, configID):
		matchResult = WSAdminUtils.scopeCompiled.match(configID)
		if (matchResult == None):
			return None
		else:
			scope = matchResult.group(WSAdminUtils.scopeName)
			if (scope == None):
				return None
			else:
				cellMatchResult = WSAdminUtils.cellCompiled.match(scope)
				if (cellMatchResult != None):
					cellName = cellMatchResult.group(WSAdminUtils.cellName);
					return CellScope(cellName)
				
				nodeMatchResult = WSAdminUtils.nodeCompiled.match(scope)
				if (nodeMatchResult != None):
					cellName = nodeMatchResult.group(WSAdminUtils.cellName);
					nodeName = nodeMatchResult.group(WSAdminUtils.nodeName);
					return NodeScope(cellName, nodeName)
				
				serverMatchResult = WSAdminUtils.serverCompiled.match(scope)
				if (serverMatchResult != None):
					cellName = serverMatchResult.group(WSAdminUtils.cellName);
					nodeName = serverMatchResult.group(WSAdminUtils.nodeName);
					serverName = serverMatchResult.group(WSAdminUtils.serverName);
					return ServerScope(cellName, nodeName, serverName)
				
				clusterMatchResult = WSAdminUtils.clusterCompiled.match(scope)
				if (clusterMatchResult != None):
					cellName = clusterMatchResult.group(WSAdminUtils.cellName);
					clusterName = clusterMatchResult.group(WSAdminUtils.clusterName);
					return ClusterScope(cellName, clusterName)
				
				return None

	cellName = "cell"
	nodeName = "node"
	serverName = "server"
	clusterName = "cluster"
	cellPattern = "^cells/(?P<" + cellName + ">[^/]+)$"
	cellCompiled = re.compile(cellPattern)
	nodePattern = "^cells/(?P<" + cellName + ">[^/]+)/nodes/(?P<" + nodeName + ">[^/]+)$"
	nodeCompiled = re.compile(nodePattern)
	serverPattern = "^cells/(?P<" + cellName + ">[^/]+)/nodes/(?P<" + nodeName + ">[^/]+)/servers/(?P<" + serverName + ">[^/]+)$"
	serverCompiled = re.compile(serverPattern)
	clusterPattern = "^cells/(?P<" + cellName + ">[^/]+)/clusters/(?P<" + clusterName + ">[^/]+)$"
	clusterCompiled = re.compile(clusterPattern)

	def determineScopeFromMBean(self, mbean):
		nameValuePairs = self.determineNameValuePairsFromMBean(mbean)
		
		return ProcessScope(nameValuePairs["cell"], nameValuePairs["node"], nameValuePairs["process"])
	
	
	mbeanNameValuesName = "nameValues"
	mbeanNameValuesPattern = "WebSphere:(?P<" + mbeanNameValuesName + ">.+)"
	mbeanNameValuesCompiled = re.compile(mbeanNameValuesPattern)

	def determineNameValuePairsFromMBean(self, mbean):
		matchResult = WSAdminUtils.mbeanNameValuesCompiled.match(mbean)
		if (matchResult == None):
			return None
		else:
			nameValueString = matchResult.group(WSAdminUtils.mbeanNameValuesName)
			if (nameValueString == None):
				return None
			else:
				nameValuesList = nameValueString.split(",")
				nameValuesDict = {}
				
				for nameValue in nameValuesList:
					nameAndValue = nameValue.split("=")
					nameValuesDict[nameAndValue[0]] = nameAndValue[1]
				
				return nameValuesDict
	
	
	