#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2007, 2008. All Rights Reserved.
#  US Government Users Restricted Rights- Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
"""Reusable system utilty functions
"""

import sys, os
from os import environ
from java.lang import System
from win.path import removePathSpace
from com.ibm.wbmonitor.install.common.log import ILogger
from util.wbmlogger import WBMLogger


__all__ = []

CN = "system"

def commandExists(cmd, shellExt):

   """Check is the command exist or not
   """
   log = WBMLogger.log

   fullCmd = cmd
   if (len(shellExt) > 1):
     if (System.getProperty("os.name").startswith("Windows")):
        fullCmd += shellExt[0]
     else:
        fullCmd += shellExt[1]
   return os.path.exists(fullCmd)

def runCommand(argv, atCurShell, shellExt, removeSpaceFlags=[]):
  """Run command and return return-code, STDOUT and STDERR
      in a list. 
  """
  log = WBMLogger.log
  MN = "runCommand"

  rc = []
  if (len(argv) == 0):
    return rc

  import tempfile
  stdout =  tempfile.mktemp()
  stderr =  tempfile.mktemp()

  cmd = argv[0]
  if (System.getProperty("os.name").startswith("Windows")):
   
    if (len(shellExt) > 1):
        cmd += shellExt[0]
    if ((len(removeSpaceFlags) == 0) or (removeSpaceFlags[0] == 1)):
      cmd = removePathSpace(cmd)

    i = 1
    for arg in argv[1:]: 
      if ((len(removeSpaceFlags) == 0) or (removeSpaceFlags[i] == 1)):
        arg = removePathSpace(arg) 

      cmd += " " + arg 
      i += 1

  else:
    if (len(shellExt) > 1):
        cmd += shellExt[1]

    if (atCurShell):
      cmd = ". " + cmd
    for arg in argv[1:]: 
      cmd += " " +  arg
  cmdonly = cmd
  cmd += " > " + stdout + " 2> " + stderr

  log.msg(ILogger.TYPE_INFO, CN, MN, cmdonly)
  rc.append(os.system(cmd))

  input = open(stdout, "r")
  stdoutLines = input.readlines()
  input.close()
  os.remove(stdout)
  rc.append(stdoutLines)

  input = open(stderr, "r")
  stderrLines = input.readlines()
  input.close()
  os.remove(stderr)
  rc.append(stderrLines)

  if (len(rc) > 0):
    result = "return code " + str(rc[0])
    result = "STDOUT: " 
    log.msg(ILogger.TYPE_INFO, CN, MN, result)
    for line in rc[1]:
      log.msg(ILogger.TYPE_INFO, CN, MN, line)
    result = "STDERR: " 
    log.msg(ILogger.TYPE_INFO, CN, MN, result)
    for line in rc[2]:
      log.msg(ILogger.TYPE_INFO, CN, MN, line)
  else:
    #should never come here
    result = "Cannot run the command"
  return rc 


def commandSuccess(rc):
  if (len(rc[2]) == 0 ):
    return 1
  if (len(rc[2]) > 1 ):
    return 0

  lines = rc[2]
  if (lines[0].strip == ""):
    return 1

  return 0

