######################################################################
#
#  Licensed Materials - Property of IBM
#  5724-M24
#  Copyright IBM Corporation 2008,2011. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#  
######################################################################

import sys
import jarray
from java.lang import *;
from java.util import Properties
from java.io import FileInputStream
from javax.crypto import BadPaddingException
from javax.crypto import Cipher
from javax.crypto import IllegalBlockSizeException
from javax.crypto import NoSuchPaddingException
from javax.crypto import SecretKey
from javax.crypto import SecretKeyFactory
from javax.crypto.spec import PBEKeySpec
from javax.crypto.spec import PBEParameterSpec
from com.ibm.misc import BASE64Decoder
from com.ibm.misc import BASE64Encoder

# return code, 0 for success, -1 for failure
rc = 0;

#--------------------------------------------------------------------------
# get the directory where WAS is installed
#--------------------------------------------------------------------------
def getWASHome():
	return getPropertyValue('was.install.root')
#--------------------------------------------------------------------------
# get the profile root
#--------------------------------------------------------------------------	
def getUserInstallRoot():
	return getPropertyValue('user.install.root')
#------------------------------------------------------------------------------
# get the property value by the the property name; If property not available, return ''
#------------------------------------------------------------------------------
def getPropertyValue(propertyName):
	propertyValue = java.lang.System.getProperty(propertyName)
	if(propertyValue == None):
		return ''
	return propertyValue
#------------------------------------------------------------------------------
# get the property value of configuring Cognos; If property not available, return None
#------------------------------------------------------------------------------	
def getCognosPropertiesFromFile():
	properties = Properties()
	inputFile = FileInputStream(installDir + '/scripts.wbm/migration/CognosConfig/CognosConfig.properties')
	properties.load(inputFile)
	isConfigCognos = properties.getProperty('configCognos')
	if (isConfigCognos == 'YES'):
		 congosProp = {'dbName':properties.getProperty('dbName'),'dbUserName':properties.getProperty('dbUserName'),'dbUserPassword':getDecodedPassword(properties.getProperty('dbUserPassword')),'adminUserName':properties.getProperty('adminUserName'),'adminPassword':getDecodedPassword(properties.getProperty('adminPassword')),'cluster':properties.getProperty('cluster'),'node':properties.getProperty('node'),'server':properties.getProperty('server')}
		 inputFile.close()
		 return congosProp
	else:
		return None

#------------------------------------------------------------------------------
# Print the property value of configuring Cognos for trace
#------------------------------------------------------------------------------	
def printCognosProperties():
	if(cognosProperties == None):
		print "No pre-defined cognos config found."
	else:
		print "Cognos pre-defined properties is: {DB Name:",cognosProperties['dbName'], " DB User Name:" ,cognosProperties['dbUserName'] ," DB User Password:******** Admin User Name:", cognosProperties['adminUserName'] ," Admin Password:******** Cluster:",cognosProperties['cluster'] ," Node:" ,cognosProperties['node'] ," Server:" ,cognosProperties['server'] ,"}"

#--------------------------------------------------------------------------
# Check if user indicate where to deploy cognos service explicitly
# pre-defined config:<target_server>/scripts.wbm/migration/CognosConfig/CognosConfig.properties
#--------------------------------------------------------------------------
def ifSetCognosDeploymentLocation():
	locationHasSet = 0
	if(cognosProperties == None):
		return locationHasSet
	if(cognosProperties['cluster'] != None or (cognosProperties['node'] != None and cognosProperties['server'] != None)):
		print "The location where to deploy Cognos was defined - ", "Cluster:",cognosProperties['cluster']," Node",cognosProperties['node']," Server",cognosProperties['server']
		locationHasSet = 1
	else:
		print "The location where to deploy Cognos was not defined."
	return locationHasSet

#--------------------------------------------------------------------------
# Check if current server/cluster is equals with location info about where to 
# deploy cognos service in pre-defined cognos config user has set
# pre-defined config:<target_server>/scripts.wbm/migration/CognosConfig/CognosConfig.properties
#--------------------------------------------------------------------------	
def checkNeedCognosDeploymentByLocation(cluster,node,server):
	needDeploy = 0
	if(cognosProperties == None):
		return needDeploy
	if(cognosProperties['node'] == node and cognosProperties['server'] == server and cognosProperties['cluster'] == cluster):
		needDeploy = 1
		print "Cognos need to be deployed on", " Cluster:",cluster," Node",node," Server",server
	else:
		print "Cognos don't need to be deployed on", " Cluster:",cluster," Node",node," Server",server
		
	return needDeploy

#------------------------------------------------------------------------------
# Decode the password string got from CognocConfig.properties file
#------------------------------------------------------------------------------
def getDecodedPassword(toDecrypt):
	if (toDecrypt == None or toDecrypt == ""):
		return None

	restoredPassword = ""
	PBE_PASSWORD = "this provides a simple roasting"
	salt = jarray.array([java.lang.Byte(int('0xc7',16)),java.lang.Byte(int('0x73',16)),java.lang.Byte(int('0x21',16)),java.lang.Byte(int('0x8c',16)),java.lang.Byte(int('0x7e',16)),java.lang.Byte(int('0xc8',16)),java.lang.Byte(int('0xee',16)),java.lang.Byte(int('0x99',16))],'b')
	count = 20
	soDecoder = BASE64Decoder()
	soRaw = soDecoder.decodeBuffer(toDecrypt)
	pbeParamSpec = PBEParameterSpec(salt, count)			
	pbeKeySpec = PBEKeySpec(PBE_PASSWORD)
	keyFac = SecretKeyFactory.getInstance("PBEWithMD5AndDES")
	pbeKey = keyFac.generateSecret(pbeKeySpec)
	pbeCipher = Cipher.getInstance("PBEWithMD5AndDES")
	pbeCipher.init(Cipher.DECRYPT_MODE, pbeKey, pbeParamSpec)
	restoredtext = pbeCipher.doFinal(soRaw)	
	restoredPassword = restoredtext.tostring()	
	return restoredPassword  
#------------------------------------------------------------------------------
# a utility method
#------------------------------------------------------------------------------
def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList

#..............................................................................
# check status of Event Service
#..............................................................................
def showEventServiceStaus(cell,node,server,cluster):
	print "\n**************** showEventServiceStaus() start ****************"
	if(cluster == None):		
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster

	if(cluster != None):
		status = AdminTask.showEventServiceStatus(['-clusterName', cluster])
	else:
		status = AdminTask.showEventServiceStatus(['-nodeName', node, '-serverName',server])

	deployed = 0
	statusStrings = status.splitlines()
	for statusStr in statusStrings:
		if(statusStr.find('deployed true') >= 0):
			deployed = 1
			break
	if ( deployed == 0 ):
		print "Event Service is not deployed here"
	else:
		print "Event Service is deployed here"
	print "\n**************** showEventServiceStaus() end ****************"
	return deployed
	
#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
	exist = 0
	#applications = AdminApp.list().split()
	applications = AdminApp.list().splitlines()
	for application in applications:
		if (application == applicationName):
			exist = 1
			return exist
	return exist
#------------------------------------------------------------------------------
# uninstall the application with the specified parameter
#------------------------------------------------------------------------------
def uninstallApplication(applicationName):
	print "\n**************** uninstallApplication() start ****************"
	result = 0
	try:
		AdminApp.uninstall(applicationName)
	except:
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error happened when trying to uninstall the application: " + applicationName
		result = -1
	AdminConfig.save()
	print "**************** uninstallApplication() end ****************"
	return result
#------------------------------------------------------------------------------
# perform app update
#------------------------------------------------------------------------------	
def updateApplication(applicationName, newAppLocation):
	# format: AdminApp.update('app1', 'app', '[-operation update -contents c:/apps/app1/newApp1.ear 
	# -usedefaultbindings -nodeployejb -BindJndiForEJBNonMessageBinding 
	# [ ["Increment EJB module" Increment Increment.jar,META-INF/ejb-jar.xml Inc]]]')
	print "\n**************** updateApplication() start ****************"
	print "Application Name" + applicationName + " New Application Location:" + newAppLocation
	result = 0
	try:
		AdminApp.update(applicationName, 'app', '[-operation update -contents "' + newAppLocation + '" ]')
	except:
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error happened when trying to update the application: " + applicationName
		result = -1
	else:
		AdminConfig.save()
	print "**************** updateApplication() end ****************"
	return result

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplication(applicationName, earLocation, cell, node, server, options):
	print "\n**************** installApplication() start ****************"
	result = 0

	#1.IF: the application was not installed yet, install the application 
	if(checkApplicationExist(applicationName) == 0):
		#deployOptions = '-appname ' + applicationName + ' -cell ' +  cell + ' -node ' + node + ' -server ' + server + '-MapModulesToServers' + ''
		deployOptions = ['-appname', applicationName ,'-cell',cell,'-node',node,'-server',server]
		print "Options added to deploy:"
		print options
		# add MapModulesToServers options to deploy options
		if ( options != ''):
			deployOptions.extend(options)
		parameters = deployOptions
		print parameters
		try:
			AdminApp.install(earLocation, parameters)
		except:
			type, value, traceback = sys.exc_info()
			print type
			print value
			print traceback
			print "Exception happened when trying to install application " + applicationName
			result = -1
		else:
			AdminConfig.save()
	print "**************** installApplication() end ****************"
	return result;
	
#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplicationCluster(applicationName, earLocation, cell, cluster, options):
	print "\n**************** installApplicationCluster() start ****************"
	result = 0
	#1.IF: the application was not installed yet, install the application 
	if(checkApplicationExist(applicationName) == 0):
		deployOptions = ['-appname', applicationName ,'-cell',cell,'-cluster',cluster]
		print "Options added to deploy:"
		print options
		if ( options != ''):
			deployOptions.extend(options)
		parameters = deployOptions
		try:
			AdminApp.install(earLocation, parameters)
		except:
			type, value, traceback = sys.exc_info()
			print type
			print value
			print traceback
			print "Exception happened when trying to install application " + applicationName
			result = -1
		else:
			AdminConfig.save()
	print "**************** installApplicationCluster() end ****************"
	return result
#------------------------------------------------------------------------------
# check whether the newTarget already exist in oldTarget;If true,return 0; otherwise, return 1
#------------------------------------------------------------------------------
def checkTargetExist(oldTarget,newTarget):
	result = 1
	targetList = oldTarget.split('+')
	for target in targetList:
		if(target == newTarget):
			result = 0
	return result
#------------------------------------------------------------------------------
# get the current cell
#------------------------------------------------------------------------------
def getCell():
	cellName = ''
	cellList = wsadminToList(AdminConfig.list('Cell'))
	for cell in cellList:
		cellName = AdminConfig.showAttribute(cell,'name')
	return cellName

#------------------------------------------------------------------------------
# get the servers defined in specified cell and node; need to filter out the node agent, cluster member
#------------------------------------------------------------------------------
def getServerList(cell,node):
	print "\n**************** getServerList() start ****************"
	serverNameList = []
	nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
	if(nodeId == ''):
		return serverNameList
	serverList = wsadminToList(AdminConfig.list('Server',nodeId))
	for server in serverList:
		serverName = AdminConfig.showAttribute(server,'name')
		print "Server list - Name: "
		print server
		clusterName = AdminConfig.showAttribute(server,'clusterName')
		if(clusterName == None):
			if(serverName != 'nodeagent'):
				serverNameList.append(serverName)
	print "**************** getServerList() end ****************"
	return serverNameList
	
#------------------------------------------------------------------------------
# get the standalone server name, the default value is server1
#------------------------------------------------------------------------------
def getSAServerName(cell,node):
	print "\n**************** getSAServerName() start ****************"
	serverName = 'server1'	
	serverList = getServerList(cell,node)			
	for server in serverList:
		serverName = server
		break
	print "Server Name: " + serverName
	print "**************** getSAServerName() end ****************"
	return serverName

#------------------------------------------------------------------------------
# get the location of installable  application
#------------------------------------------------------------------------------	
def getInstallableAppsLocation(cell,node):
	return getWASHome() + '/installableApps.wbm/'

#------------------------------------------------------------------------------
# check whether the update is required for current node; If true, return 0; Otherwise, return 1
# 1. If the node is dmgr node, do nothing. The app upgrade is done for the first custom node with server specified
# 2. cluster upgrade.
# 3. no need for stand alone server
#------------------------------------------------------------------------------
def checkWhetherUpdateRequired():
	print "\n**************** checkWhetherUpdateRequired() start ****************"
	result = 1
	nodeType = getPropertyValue('bpm.upgrade.node.type')
	print "node type is: "
	print nodeType
	updatedCluster = getUpdatedCluster()
	print "update cluster is: "
	print updatedCluster
	
	if(updatedCluster != ''):
		result = 0
	elif (nodeType == 'managed'):
		result = 0
	print "**************** checkWhetherUpdateRequired() end ****************"
	return result

#------------------------------------------------------------------------------
# get the current node name which is migrated
#------------------------------------------------------------------------------
def getUpdatedNode():
	return getPropertyValue('local.node')

#------------------------------------------------------------------------------
# get the current cluster name which is migrated, if not available return ''
#------------------------------------------------------------------------------
def getUpdatedCluster():
	return getPropertyValue('bpm.upgrade.cluster.name')
	
#------------------------------------------------------------------------------
# Check wether old mappings exist on current upgrade server/cluster
#------------------------------------------------------------------------------	
def checkOldAppMapping(oldAppName,cell,node,server,cluster):
	oldAppModuleList = AdminApp.listModules(oldAppName,'-server')
	moduleList = oldAppModuleList.splitlines()
	oldAppInfoList = moduleList[0].split('#')
	oldAppTarget = oldAppInfoList[2]
	
	if (cluster == None):
		target = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server
	else:
		target = 'WebSphere:cell=' + cell + ',cluster=' + cluster  
	result = checkTargetExist(oldAppTarget,target)
	if(result == 0):
		return 1
	else:
		# not exist
		return 0	

#------------------------------------------------------------------------------
# Check wether old app exist, and mappings exist on current target
#------------------------------------------------------------------------------	
def checkOldAppOnTarget(oldAppName,target):
	if ( checkApplicationExist(oldAppName) == 0 ):
		return 0
	oldAppModuleList = AdminApp.listModules(oldAppName,'-server')
	moduleList = oldAppModuleList.splitlines()
	oldAppInfoList = moduleList[0].split('#')
	oldAppTarget = oldAppInfoList[2]
  
	result = checkTargetExist(oldAppTarget,target)
	if(result == 0):
		return 1
	else:
		# not exist
		return 0

# ######################################
# setAllAuthenticated subroutine
# app - name of the application 
# roll - name of the user roll
# ######################################
def setAllAuthenticated ( app, roll ):
	print "INFO:  Entering setAllAuthenticated"
	print "INFO:  app="+app+",roll="+roll

	#--------------------------------------------------------------
	# set up globals
	#--------------------------------------------------------------
	global AdminConfig
	global AdminApp

	#---------------------------------------------------------
	# A very simple install command 
	#---------------------------------------------------------
	print "INFO:  Set the AllAuthenticated Security to role: "+roll+" for the "+app+" application."
	try:
		AdminApp.edit(app, '[ -MapRolesToUsers [[ '+roll+' AppDeploymentOption.No AppDeploymentOption.Yes "" "" ]]]' )
		print "INFO:  AllAuthenticated set successfully for the "+roll+" role for the "+app+" application."
		AdminConfig.save( )
	except:
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		rc = -1
		print "INFO:  Exception caught executing AdminApp.edit() to set the AllAuthenticated Security to role: "+roll+" for the "+app+" application."

	print "INFO:  Exiting setAllAuthenticated"	        
#endDef 

#------------------------------------------------------------------------------
# Create resources needed by Emitter rest services
#------------------------------------------------------------------------------	
def createEmitterRestResources(cellName, clusterName, nodeName, serverName):
	print "\n**************** createEmitterRestResources() start ****************"
	busName = "MONITOR."+cellName+".Bus"
	print "bus name is: " + busName
	cellConfigId=cellName+"(cells/"+cellName+"|cell.xml)" 
	print "cell config id is: "
	print cellConfigId
	
	# get bus member cluster or node server
	try:
		busMembersStr = AdminTask.listSIBusMembers('[-bus ' + busName + ' ]')
		busMembersList = wsadminToList(busMembersStr)
		print "\nBus members :"
		print busMembersList
		busMemberCluster = AdminConfig.showAttribute(busMembersList[0], 'cluster')
		print "Bus member cluster:"
		print busMemberCluster
		busMemberNode = AdminConfig.showAttribute(busMembersList[0], 'node')
		print "Bus member node:"
		print busMemberNode
		busMemberServer = AdminConfig.showAttribute(busMembersList[0], 'server')
		print "Bus member server:"
		print busMemberServer
	except:  
		type, value, traceback = sys.exc_info()
		rc = -1
		print type
		print value
		print traceback
		print "warning: failed to get bus member information"

	#puts "Creating jms error destination queue"
	try:    
		if ( (busMemberCluster == None) or (busMemberCluster == '') ):
			MEListInfo = AdminTask.listSIBEngines('[-bus  ' + busName + ' -server ' + serverName + ' -node ' + nodeName + ']')
			MEList = wsadminToList(MEListInfo)
			print "\nmessage engine list:"
			print MEList
			subscriptHome = AdminConfig.showAttribute(MEList[0], 'name') # ME name
			print "Durable Subscription Home: "
			print subscriptHome
			optList=["-bus",busName,"-name","MonitorEventEmitterErrorQueue","-description","Error Queue for Monitor JMS Event Emitter Service","-type","Queue","-node",nodeName,"-server",serverName,"-reliability","ASSURED_PERSISTENT"]
		else:
			MEListInfo = AdminTask.listSIBEngines('[-bus  ' + busName + ' -cluster ' + busMemberCluster + ']')
			MEList = wsadminToList(MEListInfo)
			print "\nmessage engine list:"
			print MEList
			subscriptHome = AdminConfig.showAttribute(MEList[0], 'name') # ME name
			print "Durable Subscription Home: "
			print subscriptHome
			optList=["-bus",busName,"-name","MonitorEventEmitterErrorQueue","-description","Error Queue for Monitor JMS Event Emitter Service","-type","Queue","-cluster",busMemberCluster,"-reliability","ASSURED_PERSISTENT"]
		print "\nCreating the MonitorEventEmitterErrorQueue destination with the following arguments"
		print optList
		AdminTask.createSIBDestination(optList)
		AdminConfig.save()
		print "Done creating the MonitorEventEmitterErrorQueue destination"
	except: 
		type, value, traceback = sys.exc_info()
		rc = -1
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEventEmitterErrorQueue destination"

	#puts "Creating jms destination queue"
	try:    
		if ( (busMemberCluster == None) or (busMemberCluster == '') ):
			optList=["-bus",busName,"-name","MonitorEventEmitterQueue","-description","Queue for Monitor JMS Event Emitter Service","-type","Queue","-node",nodeName,"-server",serverName,"-reliability","ASSURED_PERSISTENT","-exceptionDestination","MonitorEventEmitterErrorQueue"]
		else:
			optList=["-bus",busName,"-name","MonitorEventEmitterQueue","-description","Queue for Monitor JMS Event Emitter Service","-type","Queue","-cluster",busMemberCluster,"-reliability","ASSURED_PERSISTENT","-exceptionDestination","MonitorEventEmitterErrorQueue"]
		
		print "\nCreating the MonitorEventEmitterQueue destination with the following arguments"
		print optList
		AdminTask.createSIBDestination(optList)
		AdminConfig.save()
		print "Done creating the MonitorEventEmitterQueue destination"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEventEmitterQueue destination"

	#puts "Creating error queue connection factory"
	try:    
		optList=["-name","MonitorEmitterErrorQConnFactory","-description","Queue Connection factory used by the JMS MonitorEventEmitterErrorQueue","-jndiName","jms/MonitorEventEmitter/ErrorQConnFactory","-type","queue","-authDataAlias","MonitorBusAuth ","-busName",busName,"-nonPersistentMapping","ExpressNonPersistent", "-persistentMapping","ReliablePersistent","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred","-connectionProximity","Bus","-shareDataSourceWithCMP","false"]
		print "\nCreating the MonitorEmitterErrorQ connection factory with the following cellConfigId"
		print cellConfigId
		print "Creating the MonitorEmitterErrorQ connection factory with the following optList"
		print optList
		AdminTask.createSIBJMSConnectionFactory(cellConfigId,optList)
		AdminConfig.save()
		print "Done creating the MonitorEmitterErrorQ connection factory"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEmitterErrorQ connection factory"

	#puts "Creating queue connection factory"
	try:    
		optList=["-name","MonitorEmitterQueueConnFactory","-description","Queue Connection factory used by the JMS MonitorEventEmitterQueue","-jndiName","jms/MonitorEventEmitter/QueueConnFactory","-type","queue","-authDataAlias","MonitorBusAuth ","-busName",busName,"-nonPersistentMapping","ExpressNonPersistent", "-persistentMapping","ReliablePersistent","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred","-connectionProximity","Bus","-shareDataSourceWithCMP","false"]
		print "\nCreating the MonitorEmitterQueueConnFactory with the following cellConfigId"
		print cellConfigId
		print "Creating the MonitorEmitterQueueConnFactory with the following optList"
		print optList
		AdminTask.createSIBJMSConnectionFactory(cellConfigId,optList)
		AdminConfig.save()
		print "Done creating the MonitorEmitterQueue connection factory"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEmitterQueueConnFactory"

	#puts "Creating JMS queue"
	try:    
		optList=["-name","MonitorEventEmitterQueue","-description","JMS Queue for the Monitor JMS Event Emitter service","-jndiName","jms/MonitorEventEmitter/Queue","-queueName","MonitorEventEmitterQueue","-deliveryMode","Application ","-readAhead","AsConnection","-busName",busName]
		print "\nCreating the MonitorEventEmitterQueue JMS queue with the following cellConfigId"
		print cellConfigId
		print "Creating the MonitorEventEmitterQueue JMS queue with the following optList"
		print optList
		AdminTask.createSIBJMSQueue(cellConfigId,optList)
		AdminConfig.save()
		print "Done creating the MonitorEventEmitterQueue JMS queue"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEventEmitterQueue JMS queue"

	#puts "Creating JMS error queue"
	try:    
		optList=["-name","MonitorEventEmitterErrorQueue","-description","JMS Error Queue for the Monitor JMS Event Emitter service","-jndiName","jms/MonitorEventEmitter/ErrorQueue","-queueName","MonitorEventEmitterErrorQueue","-deliveryMode","Application","-readAhead","AsConnection","-busName",busName]
		print "\nCreating the MonitorEventEmitterErrorQueue JMS queue with the following cellConfigId"
		print cellConfigId
		print "Creating the MonitorEventEmitterErrorQueue JMS queue with the following arguments"
		print optList
		AdminTask.createSIBJMSQueue(cellConfigId,optList)
		AdminConfig.save()
		print "Done creating the MonitorEventEmitterErrorQueue JMS queue"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEventEmitterErrorQueue JMS queue"

	#puts "Creating activation spec"
	try:    
		optList=["-name","MonitorEventEmitterActivationSpec","-description","Activation Specification used by the JMS Event Emitter Service","-jndiName","jms/MonitorEventEmitter/ActivationSpec","-destinationJndiName","jms/MonitorEventEmitter/Queue ","-acknowledgeMode","Auto-acknowledge","-authenticationAlias","MonitorBusAuth","-busName",busName,"-durableSubscriptionHome",subscriptHome,"-maxBatchSize","1","-maxConcurrency","1", "-subscriptionDurability","NonDurable","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred ","-shareDataSourceWithCMP","false"]
		print "\nCreating the MonitorEventEmitterActivationSpec with the following cellConfigId"
		print cellConfigId
		print "Creating the MonitorEventEmitterActivationSpec with the following arguments"
		print optList
		AdminTask.createSIBJMSActivationSpec(cellConfigId,optList)
		AdminConfig.save()
		print "Done creating the MonitorEventEmitterActivationSpec"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the MonitorEventEmitterActivationSpec"

	providerid=AdminConfig.getid("/Cell:"+cellName+"/EventInfrastructureProvider:/")
	try:		
		Name = ['name', 'EmitterFactoryForREST']
		JndiName = ['jndiName', 'com/ibm/monitor/EmitterFactoryForREST']
		Description = ['description', 'Emitter Factory for REST Emitter']
		TransactionMode = ['preferredTransactionMode', 'false']
		SynchronizationMode = ['preferredSynchronizationMode', 'true']
		SyncJNDIName = ['synchronousTransmissionProfileJNDIName', 'com/ibm/events/configuration/bus-transmission/Default']
		filteringEnabled = ['filteringEnabled', 'false']

		optList1 = [Name,JndiName,Description,TransactionMode,SynchronizationMode,SyncJNDIName,filteringEnabled]
		print "\nCreating the EmitterFactoryForREST with the following providerid"
		print providerid
		print "Creating the EmitterFactoryForREST with the following optList1"
		print optList1
		AdminConfig.create('EmitterFactoryProfile', providerid, optList1)
		AdminConfig.save()
		print "Done creating the EmitterFactoryForREST"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the EmitterFactoryForREST"

	try:		
		Name = ['name', 'EmitterFactory']
		JndiName = ['jndiName', 'com/ibm/monitor/EmitterFactory']
		Description = ['description', 'Emitter Factory for MDB Emitter']
		TransactionMode = ['preferredTransactionMode', 'false']
		SynchronizationMode = ['preferredSynchronizationMode', 'true']
		SyncJNDIName = ['synchronousTransmissionProfileJNDIName', 'com/ibm/events/configuration/bus-transmission/Default']
		filteringEnabled = ['filteringEnabled', 'false']

		optList2 = [Name,JndiName,Description,TransactionMode,SynchronizationMode,SyncJNDIName,filteringEnabled]
		print "\nCreating the EmitterFactoryForMDB with the following providerid"
		print providerid
		print "Creating the EmitterFactoryForMDB with the following optList2"
		print optList2
		AdminConfig.create('EmitterFactoryProfile', providerid, optList2)
		AdminConfig.save()
		print "Done creating the EmitterFactoryForMDB"
	except:    
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "warning: failed to create the EmitterFactoryForMDB"	
	
	print "**************** createEmitterRestResources() end ****************"

#------------------------------------------------------------------------------
# Create resources needed by Lombardi Emitter services for migration 70 > 75 standalone
# After 70 > 75 profile migration, the resources.xml are overwritten and these configurations are gone
#------------------------------------------------------------------------------	
def createLombardiEmitterResources(cellName, nodeName):
	print "\n**************** createLombardiEmitterResources() start ****************"
	busName = "MONITOR."+cellName+".Bus"
	print "bus name is: " + busName
	cellConfigId = cellName+"(cells/"+cellName+"|cell.xml)" 
	print "cell config id is: "
	print cellConfigId
	server = getSAServerName(cellName, nodeName)
	subscriptHome=nodeName+"."+server+"-MONITOR."+cellName+".Bus" 
	
	#    * Create error queue destination on MONITOR SI bus for the JMS emitter.
	#    
	#        - Type � Queue
	#        - Name � LombardiJMSEmitterErrorQueue
	#
    	try:    
		optList=["-bus",busName,"-name","LombardiJMSEmitterErrorQueue","-description","Error queue for the Lombardi JMS event emitter","-type","Queue","-node",nodeName,"-server",server,"-reliability","ASSURED_PERSISTENT"]
		print "\nCreating the LombardiJMSEmitterErrorQueue destination with the following arguments"
		print optList
		AdminTask.createSIBDestination(optList)
		print "Done creating the LombardiJMSEmitterErrorQueue destination"
	except:    
		rc = -1
		print "warning: failed to create the LombardiJMSEmitterErrorQueue destination"

	# 
	#    * Create queue destination on MONITOR SI bus for Lombardi events.
	#   
	#       - Type � Queue
	#       - Name � LombardiEventEmitterInputQueue
	#
	try:    
		optList=["-bus",busName,"-name","LombardiEventEmitterInputQueue","-description","Input queue for the Lombardi JMS event emitter","-type","Queue","-node",nodeName,"-server",server,"-reliability","ASSURED_PERSISTENT","-exceptionDestination","LombardiJMSEmitterErrorQueue"]
		print "\nCreating the LombardiEventEmitterInputQueue destination with the following arguments"
		print optList
		AdminTask.createSIBDestination(optList)
		print "Done creating the LombardiEventEmitterInputQueue destination"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmitterInputQueue destination"

	#     * Create Lombardi Event Emission JMS Queue
	#
	#        - Name � LombardiEventEmissionQueue
	#        - JNDI name � jms/com.ibm.lombardi/EventEmissionQueue (points to same queue as above)
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#        - Queue name � LombardiEventEmissionQueue
	#
	try:    
		optList=["-name","LombardiEventEmissionQueue","-description","Output queue for Lombardi event emission","-jndiName","jms/com.ibm.lombardi/EventEmissionQueue","-queueName","LombardiEventEmitterInputQueue","-deliveryMode","Application ","-readAhead","AsConnection","-busName",busName]
		print "\nCreating the LombardiEventEmissionQueue JMS queue with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmissionQueue JMS queue with the following optList"
		print optList
		AdminTask.createSIBJMSQueue(cellConfigId,optList)
		print "Done creating the LombardiEventEmissionQueue JMS queue"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmissionQueue JMS queue"

	#    * Create Lombardi Event Emission JMS Queue Connection Factory
	#    
	#        - Name � LombardiEventEmissionQCF
	#        - JNDI name � jms/com.ibm.lombardi/EventEmissionQueueFactory
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#
	try:    
		optList=["-name","LombardiEventEmissionQCF","-description","Queue connection factory for Lombardi event emission","-jndiName","jms/com.ibm.lombardi/EventEmissionQueueFactory","-type","queue","-containerAuthAlias","MonitorBusAuth","-authDataAlias","MonitorBusAuth","-busName",busName,"-nonPersistentMapping","ExpressNonPersistent", "-persistentMapping","ReliablePersistent","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred","-connectionProximity","Bus","-shareDataSourceWithCMP","false"]
		print "\nCreating the LombardiEventEmissionQCF with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmissionQCF with the following optList"
		print optList
		AdminTask.createSIBJMSConnectionFactory(cellConfigId,optList)
		print "Done creating the LombardiEventEmissionQCF connection factory"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmissionQCF"

	#     * Create JMS Queue for the Lombardi JMS Event Emitter to use as input
	#
	#        - Name � LombardiEventEmitterInputQueue
	#        - JNDI name � jms/com.ibm.lombardi/JMSEmitterInput
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#        - Queue name � LombardiEventEmitterInputQueue
	#
	try:    
		optList=["-name","LombardiEventEmitterInputQueue","-description","Input queue for the Lombardi event emitter","-jndiName","jms/com.ibm.lombardi/JMSEmitterInput","-queueName","LombardiEventEmitterInputQueue","-deliveryMode","Application ","-readAhead","AsConnection","-busName",busName]
		print "\nCreating the LombardiEventEmitterInputQueue JMS queue with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmitterInputQueue JMS queue with the following optList"
		print optList
		AdminTask.createSIBJMSQueue(cellConfigId,optList)
		print "Done creating the LombardiEventEmitterInputQueue JMS queue"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmitterInputQueue JMS queue"

	#     * Create JMS Queue Connection Factory for the Lombardi JMS Event Emitter to use as input
	#    
	#        - Name � LombardiEventEmitterInputQCF
	#        - JNDI name � jms/com.ibm.lombardi/JMSEmitterInputQueueFactory
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#
	try:    
		optList=["-name","LombardiEventEmitterInputQCF","-description","Input queue connection factory for the Lombardi event emitter","-jndiName","jms/com.ibm.lombardi/JMSEmitterInputQueueFactory","-type","queue","-containerAuthAlias","MonitorBusAuth","-authDataAlias","MonitorBusAuth","-busName",busName,"-nonPersistentMapping","ExpressNonPersistent", "-persistentMapping","ReliablePersistent","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred","-connectionProximity","Bus","-shareDataSourceWithCMP","false"]
		print "\nCreating the LombardiEventEmitterInputQCF with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmitterInputQCF with the following optList"
		print optList
		AdminTask.createSIBJMSConnectionFactory(cellConfigId,optList)
		print "Done creating the LombardiEventEmitterInputQCF connection factory"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmitterInputQCF"

	#     * Create JMS Activation Specification for the Lombardi JMS Event Emitter to use as input
	#    
	#        - Name � LombardiEventEmitterInputAS
	#        - JNDI name � jms/com.ibm.lombardi/JMSEmitterInputActivationSpec
	#        - Description �
	#        - Destination type � Queue
	#        - Destination JNDI name � jms/com.ibm.lombardi/JMSEmitterInput
	#        - Bus name � MONITOR.cellname.Bus
	#        - Authentication Alias � MonitorBusAuth
	#
	try:    
		optList=["-name","LombardiEventEmitterInputAS","-description","Activation specification used by the Lombardi event emitter","-jndiName","jms/com.ibm.lombardi/JMSEmitterInputActivationSpec","-destinationJndiName","jms/com.ibm.lombardi/JMSEmitterInput","-acknowledgeMode","Auto-acknowledge","-authenticationAlias","MonitorBusAuth","-busName",busName,"-durableSubscriptionHome",subscriptHome,"-maxBatchSize","1","-maxConcurrency","1", "-subscriptionDurability","NonDurable","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred ","-shareDataSourceWithCMP","false"]
		print "\nCreating the LombardiEventEmitterInputAS with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmitterInputAS with the following arguments"
		print optList
		AdminTask.createSIBJMSActivationSpec(cellConfigId,optList)
		print "Done creating the LombardiEventEmitterInputAS"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmitterInputAS"

	#     * Create JMS Queue for the Lombardi JMS Event Emitter to use for errors
	#
	#        - Name � LombardiEmitterErrorQueue
	#        - JNDI name � jms/com.ibm.lombardi/JMSEmitterError
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#        - Queue name � LombardiEmitterErrorQueue
	#
	try:    
		optList=["-name","LombardiEmitterErrorQueue","-description","Error queue for the Lombardi event emitter","-jndiName","jms/com.ibm.lombardi/JMSEmitterError","-queueName","LombardiEventEmitterInputQueue","-deliveryMode","Application ","-readAhead","AsConnection","-busName",busName]
		print "\nCreating the LombardiEmitterErrorQueue JMS queue with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEmitterErrorQueue JMS queue with the following optList"
		print optList
		AdminTask.createSIBJMSQueue(cellConfigId,optList)
		print "Done creating the LombardiEmitterErrorQueue JMS queue"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEmitterErrorQueue JMS queue"

	#     * Create JMS Queue Connection Factory for the Lombardi JMS Event Emitter to use for errors
	#    
	#        - Name � LombardiEventEmitterErrorQCF
	#        - JNDI name � jms/com.ibm.lombardi/JMSEmitterErrorQueueFactory
	#        - Description �
	#        - Bus name � MONITOR.cellname.Bus
	#
	try:    
		optList=["-name","LombardiEventEmitterErrorQCF","-description","Error queue connection factory for the Lombardi event emitter","-jndiName","jms/com.ibm.lombardi/JMSEmitterErrorQueueFactory","-type","queue","-containerAuthAlias","MonitorBusAuth","-authDataAlias","MonitorBusAuth","-busName",busName,"-nonPersistentMapping","ExpressNonPersistent", "-persistentMapping","ReliablePersistent","-readAhead","Default","-targetType","BusMember","-targetSignificance","Preferred","-connectionProximity","Bus","-shareDataSourceWithCMP","false"]
		print "\nCreating the LombardiEventEmitterErrorQCF with the following cellConfigId"
		print cellConfigId
		print "Creating the LombardiEventEmitterErrorQCF with the following optList"
		print optList
		AdminTask.createSIBJMSConnectionFactory(cellConfigId,optList)
		print "Done creating the LombardiEventEmitterErrorQCF connection factory"
	except:    
		rc = -1
		print "warning: failed to create the LombardiEventEmitterErrorQCF"

	
	#    * Create event emitter factory for use by the JMS emitter.
	#        - Name � LombardiEventEmitterFactory
	#        - JNDI name � cei/com.ibm.lombardi/EmitterFactory
	#        - Description �
	#        - Support event service transmission � Checked
	#        - Prefer event service transmission � Checked
	#        - JNDI name � cei/com.ibm.lombardi/EmitterFactory.
	#
	try:		
		Name = ['name', 'LombardiEventEmitterFactory']
		JndiName = ['jndiName', 'cei/com.ibm.lombardi/EmitterFactory']
		Description = ['description', 'Emitter Factory for the Lombardi Emitter']
		TransactionMode = ['preferredTransactionMode', 'false']
		SynchronizationMode = ['preferredSynchronizationMode', 'true']
		SyncJNDIName = ['synchronousTransmissionProfileJNDIName', 'com/ibm/events/configuration/bus-transmission/Default']
		filteringEnabled = ['filteringEnabled', 'false']
		
		optList = [Name,JndiName,Description,TransactionMode,SynchronizationMode,SyncJNDIName,filteringEnabled]	
		providerid=AdminConfig.getid("/Cell:"+cellName+"/EventInfrastructureProvider:/")	
		print "\nCreating the LombardiEventEmitterFactory with the following providerid"
		print providerid
		print "Creating the LombardiEventEmitterFactory with the following optList1"
		print optList
		AdminConfig.create('EmitterFactoryProfile', providerid, optList)
		print "Done creating the LombardiEventEmitterFactory"
	except:    
		rc = -1
		print "warning: failed to create the EmitterFactoryForLombardi"

	AdminConfig.save()
	
	# Create event emitter factory EmitterFactoryForJMS that is new in 75
	createEmitterFactoryForJMS(cellName)
	
	# reinstall IBM_BPM_EMITTER_SERVICE
	try:
		bpmEventEmitter = "IBM_BPM_EMITTER_SERVICE"
		bpmEventEmitterEar = "EmitterServices.ear"
		installableAppsLocation = getInstallableAppsLocation(cellName,nodeName)
		print "installable apps location : " + installableAppsLocation
		
		uninstallApplication(bpmEventEmitter)
		extraAppInstallOptions = []
		# set option to set context roots of web modules
		contextRootMapEventEmitter = [ '.*', '.*', '/unsupported']
		contextRootOptions = [ contextRootMapEventEmitter ]
		extraAppInstallOptions.extend(['-CtxRootForWebMod', contextRootOptions])
		
		# set option to map resource references to EJBs
		resourceRefMapJMSEmitterFactory = [ 'EventEmitterMDB', '.*', '.*', '.*', 'com.ibm.events.emitter.EmitterFactory', 'cei/com.ibm.lombardi/EmitterFactory', '', '', '' ]
		resourceRefMapRESTEmitterFactory = [  'EventEmitterREST', '', 'eventEmitter.war,WEB-INF/web.xml', 'EmitterFactoryForREST', 'com.ibm.events.emitter.EmitterFactory', 'cei/com.ibm.lombardi/EmitterFactory', '', '', '' ]
		resourceRefMaps = [ resourceRefMapJMSEmitterFactory,  resourceRefMapRESTEmitterFactory ]
		extraAppInstallOptions.extend(['-MapResRefToEJB', resourceRefMaps])
		
		# set option to map JMS message bindings
		jmsBindingMDB = [ 'EventEmitterMDB', '.*', 'EventEmitterMDB.jar,META-INF/ejb-jar.xml', '', 'jms/com.ibm.lombardi/JMSEmitterInputActivationSpec', 'jms/com.ibm.lombardi/JMSEmitterInput', 'MonitorBusAuth' ]
		jmsBindingOptions = [ jmsBindingMDB ]
		extraAppInstallOptions.extend(['-BindJndiForEJBMessageBinding', jmsBindingOptions]);
		print "INFO: Application installation options: ",extraAppInstallOptions
		
		installApplication(bpmEventEmitter, installableAppsLocation + bpmEventEmitterEar, cellName, nodeName, server, extraAppInstallOptions)
		AdminApp.edit(bpmEventEmitter, '[ -MapRolesToUsers [[ eventemitters AppDeploymentOption.No AppDeploymentOption.Yes "" "" ]]]' )
		AdminConfig.save()
		print "Done reinstall " + bpmEventEmitter
	except:    
		rc = -1
		print "warning: failed to reinstall " + bpmEventEmitter
		
	# reinstall IBM_WBM_MM_DEPLOYMENT_SERVICE
	try:
		bpmDeployService = "IBM_WBM_MM_DEPLOYMENT_SERVICE"
		bpmDeployServiceEar = "IBM_WBM_MM_DEPLOYMENT_SERVICE.ear"
		installableAppsLocation = getInstallableAppsLocation(cellName,nodeName)
		print "installable apps location : " + installableAppsLocation
		
		uninstallApplication(bpmDeployService)
		installApplication(bpmDeployService, installableAppsLocation + bpmDeployServiceEar, cellName, nodeName, server, '')
		AdminConfig.save()
		print "Done reinstall " + bpmDeployService
	except:    
		rc = -1
		print "warning: failed to reinstall " + bpmDeployService
		 
	print "**************** createLombardiEmitterResources() end ****************"

#------------------------------------------------------------------------------
# Create event emitter factory EmitterFactoryForJMS that is new in 75	
#------------------------------------------------------------------------------	
def createEmitterFactoryForJMS(cellName):
	print "\n**************** createEmitterFactoryForJMS() start ****************"
	try:		
		Name = ['name', 'EmitterFactoryForJMS']
		JndiName = ['jndiName', 'com/ibm/monitor/EmitterFactoryForJMS']
		Description = ['description', 'Emitter Factory for JMS Emitter ']
		TransactionMode = ['preferredTransactionMode', 'false']
		SynchronizationMode = ['preferredSynchronizationMode', 'true']
		SyncJNDIName = ['synchronousTransmissionProfileJNDIName', 'com/ibm/events/configuration/bus-transmission/Default']
		filteringEnabled = ['filteringEnabled', 'false']

		optList = [Name,JndiName,Description,TransactionMode,SynchronizationMode,SyncJNDIName,filteringEnabled]
		providerid=AdminConfig.getid("/Cell:"+cellName+"/EventInfrastructureProvider:/")
		print "\nCreating the EmitterFactoryForMDB with the following providerid"
		print providerid
		print "Creating the EmitterFactoryForMDB with the following optList"
		print optList
		AdminConfig.create('EmitterFactoryProfile', providerid, optList)
		AdminConfig.save()
		print "Done creating the EmitterFactoryForJMS"
	except:    
		rc = -1
		print "warning: failed to create the EmitterFactoryForJMS"

	print "**************** createEmitterFactoryForJMS() end ****************"
#...............................................................
# isWebServerTarge
#...............................................................
def isWebServerTarget(target):
	# oldTarget =  'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server  parsed  the node and server, at last to judge the server is applicaiton server or not.
	print "\n**************** isWebServerTarget() start ****************"
	if(target.find('cluster=')!=-1):
		print "**************** isWebServerTarget() end ****************"
		return 0
	targetUnits = target.split(',')
	cellInfo = targetUnits[0]
	nodeInfo = targetUnits[1]
	serverInfo = targetUnits[2]
	cellName = cellInfo[15:len(cellInfo)]
	nodeName = nodeInfo[5:len(nodeInfo)]
	serverName = serverInfo[7:len(serverInfo)]
	nodeId = AdminConfig.getid('/Cell:'+cellName+'/Node:'+nodeName)
	entriesStr = AdminConfig.list('ServerEntry',nodeId)
	entries = wsadminToList(entriesStr)
	for entry in entries:
		currentServerName = AdminConfig.showAttribute(entry,'serverName')
		if(currentServerName==serverName):
			serverType = AdminConfig.showAttribute(entry,'serverType')
			if(serverType=='WEB_SERVER'):
				print "**************** isWebServerTarget() end ****************"
				return 1
	print "**************** isWebServerTarget() end ****************"
	return 0	

#...............................................................
# get old target from  app modue list by URI
#...............................................................	
def getOldTargetByURI(moduleURI, moduleList):
	oldTarget = ''
	for module in moduleList:
		moduleInfoList = module.split('#')
		uri = moduleInfoList[1].replace('+', ',')
		if ( uri == moduleURI ):
			oldTarget = moduleInfoList[2]
	return oldTarget
#.............................................................................	
# deleteWebServerMapping
#..............................................................................
def deleteWebServerMapping(appModuleMapping):
	print "\n**************** deleteWebServerMapping() start ****************"
	for appInfo in appModuleMapping:
		appName = appInfo[0]
		print "Application name: "
		print appName
		moduleMappingList = appInfo[1]
		print "Module information list: "
		print moduleMappingList
		
		if(checkApplicationExist(appName)==0):
			print "Application " + appName + " not exist"
			continue
		
		# get module list from server env
		oldAppModuleList = AdminApp.listModules(appName,'-server')
		moduleList = oldAppModuleList.splitlines()
		print "Module list:"
		print moduleList
		
		for moduleInfo in moduleMappingList:
			moduleName = moduleInfo[0]
			print "module name is: "
			print moduleName
			moduleURI = moduleInfo[1]
			print "module uri is: "
			print moduleURI
			oldTarget = getOldTargetByURI(moduleURI, moduleList)
			print "Old target is: "
			print oldTarget
			# if no app installed, or no mapping found for this module
			if ( oldTarget == '' ):
				continue
			oldTargetList = oldTarget.split('+')
			newTargetList = []
			for target in oldTargetList:
				if(isWebServerTarget(target)==0):
					newTargetList.append(target)
			print 'The new target list is :'
			print newTargetList
			
			if(newTargetList==[]):
				try:
					AdminApp.uninstall(appName)
				except:
					rc = -1
					print "Error happened when trying to uninstall the application: " + appName
				else:
					AdminConfig.save()
				break
			
			sep ='+'
			newTarget = sep.join(newTargetList)
			print 'New target is: '
			print newTarget
			
			# edit app to delete web server mapping
			try:
				if ( oldTarget != newTarget):
					print "Edit application server mappings"
					detailOption = [[moduleName,moduleURI,newTarget]]
					options = ['-MapModulesToServers',detailOption]
					AdminApp.edit(appName, options)
			except:
				rc = -1
				print "exception happened when trying to edit target mapping"
			else:
				AdminConfig.save()
	print "**************** deleteWebServerMapping() end ****************"
#.............................................................................	
# Update module mapping and app information to V7
#..............................................................................
def updateModuleMappings(appModuleMapping):
	print "\n**************** updateModuleMappings() start ****************"
	# Update module information, and save old server mappings
	for i in range( len(appModuleMapping) ):
		appInfo = appModuleMapping[i]
		appName = appInfo[0]
		print "Application name: "
		print appName
		moduleMappingList = appInfo[1]
		
		if (checkApplicationExist(appName) != 0 ):
			oldAppModuleList = AdminApp.listModules(appName,'-server')
		else:
			continue
		moduleList = oldAppModuleList.splitlines()
		
		for moduleInfo in moduleMappingList:
			moduleName = moduleInfo[0]
			moduleURI = moduleInfo[1]
			# get old target 
			oldTarget = getOldTargetByURI(moduleURI, moduleList)
			
			# update module URI for app IBM_WBM_ACTIONSERVICES
			if (appName == 'IBM_WBM_ACTIONSERVICES'):
				if (moduleName == 'Action Manager BootStrap'):
					# new URI for IBM_WBM_ACTIONSERVICES, module Action Manager BootStrap
					moduleURI = 'ActionManagerBootstrap_EJB.jar,META-INF/ejb-jar.xml'
					moduleInfo[1] = moduleURI
				elif (moduleName == 'Action Manager Dispatcher'):
					# new URI for IBM_WBM_ACTIONSERVICES, module Action Manager Dispatcher
					moduleURI = 'ActionManagerMDB_EJB.jar,META-INF/ejb-jar.xml'
					moduleInfo[1] = moduleURI
				# URI should be able to retrieved for both old and new Action Service modules
				if (oldTarget == ''):
					oldTarget = getOldTargetByURI(moduleURI, moduleList)
			
			# set server mapping to appModuleMapping
			moduleInfo[2] = oldTarget
		
			# if it's Emitter Service, rename the app name to IBM_WBM_EMITTER_SERVICES, and update module/mapping
			newEmitterServiceMapping = ['IBM_WBM_EMITTER_SERVICES', [ \
				['EventEmitterMDB', 'EventEmitterMDB.jar,META-INF/ejb-jar.xml', ''], \
				['EventEmitterREST', 'eventEmitter.war,WEB-INF/web.xml', ''] ]]
			if ( appName == 'IBM_WBM_EMITTER_REST_SERVICES'):
				target = appInfo[1][0][2]
				print "Emitter service target is: "
				print target
				newEmitterServiceMapping[1][0][2] = target
				newEmitterServiceMapping[1][1][2] = target
				print "New emitter service module mapping list: "
				print newEmitterServiceMapping
				appModuleMapping[i] = newEmitterServiceMapping
	print "**************** updateModuleMappings() end ****************"

#.............................................................................	
# get MapModulesToServers options for an app
#..............................................................................		
def getDeployOptions(appModuleMapping, appName):
	options = ''
	for appInfo in appModuleMapping:
		app = appInfo[0]
		if ( app == appName ):
			moduleMappingList = appInfo[1]
			options = ['-MapModulesToServers', moduleMappingList]
	return options
#.............................................................................	
# Update appModuleMapping to V7 information
#..............................................................................
def updateAppModuleInfo(appModuleMapping):
	print "\n**************** updateAppModuleInfo() start ****************"
	# Update module information
	for i in range( len(appModuleMapping) ):
		appInfo = appModuleMapping[i]
		appName = appInfo[0]
		print "\nApplication name: "
		print appName
		moduleMappingList = appInfo[1]
		
		for moduleInfo in moduleMappingList:
			moduleName = moduleInfo[0]
			moduleURI = moduleInfo[1]
			
			# update module URI for app IBM_WBM_ACTIONSERVICES
			if (appName == 'IBM_WBM_ACTIONSERVICES'):
				if (moduleName == 'Action Manager BootStrap'):
					# new URI for IBM_WBM_ACTIONSERVICES, module Action Manager BootStrap
					moduleURI = 'ActionManagerBootstrap_EJB.jar,META-INF/ejb-jar.xml'
					moduleInfo[1] = moduleURI
				elif (moduleName == 'Action Manager Dispatcher'):
					# new URI for IBM_WBM_ACTIONSERVICES, module Action Manager Dispatcher
					moduleURI = 'ActionManagerMDB_EJB.jar,META-INF/ejb-jar.xml'
					moduleInfo[1] = moduleURI
		
			# if it's Emitter Service, rename the app name to IBM_WBM_EMITTER_SERVICES, and update module/mapping
			newEmitterServiceMapping = ['IBM_WBM_EMITTER_SERVICES', [ \
				['EventEmitterMDB', 'EventEmitterMDB.jar,META-INF/ejb-jar.xml', ''], \
				['EventEmitterREST', 'eventEmitter.war,WEB-INF/web.xml', ''] ]]
			if ( appName == 'IBM_WBM_EMITTER_REST_SERVICES'):
				appModuleMapping[i] = newEmitterServiceMapping
	print "**************** updateAppModuleInfo() end ****************"	
#.............................................................................	
# Create CMP factory for datasource
#..............................................................................	
def createCmpFactory(cell, datasourceName, datasourceid, authalias):
	print "\n**************** createCmpFactory() start ****************"
	print "\nList all Resource Adapters"
	resourceAdapterString = AdminConfig.list("J2CResourceAdapter", AdminConfig.getid('/Cell:' + cell + '/'))
	resourceAdapterArray = resourceAdapterString.split(lineSeparator)
	resourceAdapterConfigId = None
	
	print "Checking Resource Adapters for a match to " + "WebSphere Relational Resource Adapter"
	for i in range(len(resourceAdapterArray)):
		if (resourceAdapterArray[i].find("WebSphere Relational Resource Adapter") != -1):
			print "Found Resource Adapter: " + "WebSphere Relational Resource Adapter"
			resourceAdapterConfigId = resourceAdapterArray[i]
	
	nameAttr = ["name", datasourceName + "_CMP_Factory"]
	authMechAttr = ["authMechanismPreference", "BASIC_PASSWORD"]
	dataAttr = ["cmpDatasource", datasourceid]
	authAttr = ["authDataAlias", authalias]
	
	cmpFactoryCreateArgs = [nameAttr, authMechAttr, dataAttr, authAttr]

	print "Create CMP Factory with", cmpFactoryCreateArgs
	try:
	    cmpFactoryConfigId = AdminConfig.create("CMPConnectorFactory", resourceAdapterConfigId, cmpFactoryCreateArgs)
	    AdminConfig.save()
	except:
	    print "error: CMP create failed."
	print "\n**************** createCmpFactory() end ****************"
	return cmpFactoryConfigId
	
#.............................................................................	
# Update data source
#..............................................................................	
def updateDataSource(cell):
	print "\n**************** updateDataSource() start ****************"	
	# update datasource for Monitor_<cell>_Routing_Database, according to Monitor_Database datasource
	# "authDataAlias"
	# "datasourceHelperClassname"
	# "providerType"
	# "propertySet J2EEResourcePropertySet"
	routingDataSourceName = 'Monitor_'+cell+'_Routing_Database'
	print "\nUpdating datasource " + routingDataSourceName
	try:
		# get properties from monitor data source
		monitorDataSourceId = AdminConfig.getid('/DataSource:Monitor_Database/')
		print "\nmonitor DataSource Id: " + monitorDataSourceId
		authDataAlias = AdminConfig.showAttribute(monitorDataSourceId, 'authDataAlias')
		print "authDataAlias: " + authDataAlias
		datasourceHelperClassname = AdminConfig.showAttribute(monitorDataSourceId, 'datasourceHelperClassname')
		print "datasourceHelperClassname: " + datasourceHelperClassname
		# propertySet is a list of J2EEResourcePropertySet objects
		propertySet = AdminConfig.showAttribute(monitorDataSourceId, 'propertySet')
		print "propertySet: " + propertySet
		#get detail properties for propertySet
		resourcePropertyList = wsadminToList(AdminConfig.showAttribute(propertySet, 'resourceProperties'))
		print "J2EEResourceProperty list: "
		print resourcePropertyList
	
		# update data source attributes
		print "\nUpdating datasource attributes..."
		#get routing data source id
		routingDataSourceId = AdminConfig.getid('/DataSource:'+routingDataSourceName+'/')
		AdminConfig.modify(routingDataSourceId, [['authDataAlias', authDataAlias], \
			['datasourceHelperClassname', datasourceHelperClassname]])
		print "Datasource attributes updated."
		# update propertySet
		oldPropertySet = AdminConfig.showAttribute(routingDataSourceId, 'propertySet')
		# get detail properties for propertySet
		oldResourcePropertyList = wsadminToList(AdminConfig.showAttribute(oldPropertySet, 'resourceProperties'))
		print "\nOld J2EEResourceProperty list: "
		print oldResourcePropertyList
		# remove old properties
		for oldProperty in oldResourcePropertyList:
			AdminConfig.remove(oldProperty)
		AdminConfig.save()
		print "Old J2EEResourceProperty removed."
		
		# update routing data source with new properties
		print "\nCreating new J2EEResourceProperty..."
		for resourceProperty in resourcePropertyList:
			name = AdminConfig.showAttribute(resourceProperty, 'name')
			value = AdminConfig.showAttribute(resourceProperty, 'value')
			required = AdminConfig.showAttribute(resourceProperty, 'required')
			print "Property to be created:"
			print "name: " + name
			print "value: " + value
			print "required: " + required
			AdminConfig.create('J2EEResourceProperty', oldPropertySet, [ ['name', name], ['value', value], ['required', required] ])
			print "New J2EEResourceProperty created."
		AdminConfig.save()
		print "New J2EEResourceProperty create completed."
		
		# configure container managed persistence(CMP) for data source
		# print "\nConfiguring container managed persistence(CMP) for data source..."
		# createCmpFactory(cell, routingDataSourceName, routingDataSourceId, authDataAlias)
		# print "Container managed persistence(CMP) configured."
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error updating datasource " + routingDataSourceName
	else:
		AdminConfig.save()
		print "DataSource " + routingDataSourceName + " updated."
	print "**************** updateDataSource() end ****************"
#.............................................................................	
# Update for Monitor stand alone profile
#..............................................................................	
def updateStandAloneProfile(cell, node):
	print "\n**************** updateStandAloneProfile() start ****************"
	serverName = getSAServerName(cell, node)
	if (ifRemoveABX == 'YES'):
		removeAbxApp(cell, node, serverName, None)
		removeAbxLibrary(cell, node)
	else:
		updateAbxApp(cell, node, serverName, None)
	print "\n**************** updateStandAloneProfile() end ****************"

#..............................................................................
# Update rest service application, methods
#..............................................................................
# Update rest gateway application
def updateRESTGatewayApplication(cell, node, server, cluster):
	print "\n**************** updateRESTGatewayApplication() start ****************"
	if(cluster == None):
		if(server == None):
			server = getSAServerName(cell, node)
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster
	
	restGatewayApp = 'REST Services Gateway'
	gatewayFlag = 0
	exist = 0
	# check rest gateway application
	applications = AdminApp.list().splitlines()
	for application in applications:
		if (application == restGatewayApp):
			exist = 1
			break
	
	if ( exist == 0 ):
		print "\nOld app " + restGatewayApp + " does not exist"
	elif (cluster == None) and (checkOldAppMapping(restGatewayApp,cell,node,server,None) == 0):
		print "Old mapping for " + restGatewayApp + " on server " + server + " does not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(restGatewayApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + restGatewayApp + " on cluster " + cluster + " does not exist"
	else:
		gatewayFlag = 1
	
	# use admin task to update rest gateway application
	if (gatewayFlag == 0):
		if(cluster == None):
			print "\nRest gateway application does not exist, or rest gateway application does not mapped to server " + server
		else:
			print "\nRest gateway application does not exist, or rest gateway application does not mapped to cluster " + cluster
	else:
		print "calling admin task updateRESTGatewayService..."
		try:
			if(cluster == None):
				AdminTask.updateRESTGatewayService(['-nodeName', node, '-serverName', server, '-enable', 'true'])
			else:
				AdminTask.updateRESTGatewayService(['-clusterName', cluster, '-enable', 'true'])
		except:
			rc = -1
			type, value, traceback = sys.exc_info()
			print type
			print value
			print traceback
			print "Error calling admin task updateRESTGatewayService"
		else:
			AdminConfig.save()
			print "Done admin task updateRESTGatewayService."
	
	print "**************** updateRESTGatewayApplication() end ****************"
#endDef

# Remove rest gateway configuration
def removeRESTGatewayConfig():
	print "\n**************** removeRESTGatewayConfig() start ****************"

	restDeploymentTargetsList = getRESTDeploymentTargets()
	for restDeploymentTarget in restDeploymentTargetsList:
		webModulesList = wsadminToList(AdminConfig.list("WBIRestWebModule", restDeploymentTarget))			
		for webModule in webModulesList:
			currentWebModuleName = AdminConfig.showAttribute(webModule, "name")
			if (currentWebModuleName == "rest.gateway.war"):
				AdminConfig.remove(webModule)
				AdminConfig.save()
				print 'Remove rest gate way configuration for ' + currentWebModuleName
				break	
	
	print "**************** removeRESTGatewayConfig() end ****************"
#endDef

# check wether REST Services Gateway already installed and mapped to current target
def isRestGatewayConfigured(old_restAppName, target):
        print "The current target is ", target 
	if ( checkOldAppOnTarget(old_restAppName, target) == 0 ):
		return 'false'
	else:
		return 'true'

# Migrate rest service gateway
def migrateRESTGatewayServices():
    print ""
    print "migrating REST services configuration";
    print ""
    
    old_restAppName = 'IBM_WBM_REST_SERVICES'
    # update Rest services Gateway application binaries  
    #if (checkApplicationExist("REST Services Gateway") != 0):
    #    updateRESTGatewayApplication();
    # refresh current REST Services Gateway
    #refreshSystemRESTServices();
    
    # update Rest services Gateway application binaries  
    if (checkApplicationExist(old_restAppName) == 0):
    	print "Old REST Services application not exist."
        return
    
    # update REST Services Gateway app on targets that old app IBM_WBM_REST_SERVICES found
    # Result format: IBM_WBM_ACTIONSERVICES#monAAMMDB.jar+META-INF/ejb-jar.xml#
    # WebSphere:cell=ibmmon44Node06Cell,node=ibmmon44Node02,server=server1
    oldAppModuleList = AdminApp.listModules(old_restAppName,'-server')
    moduleList = oldAppModuleList.splitlines()
    oldAppInfoList = moduleList[0].split('#')
    # format: WebSphere:cell=ibmmon44Node06Cell,node=ibmmon44Node02,server=server1
    # +WebSphere:cell=ibmmon44Node06Cell,node=ibmmon44Node02,server=server1
    oldAppTarget = oldAppInfoList[2]
    oldTargetList = oldAppTarget.split('+')
    print ""
    print "Rest service target list:"
    print oldTargetList
    print ""
    
    # uninstall old rest service application
    uninstallApplication(old_restAppName)
    
    for target in oldTargetList:
    	if ( not (isRestGatewayConfigured(old_restAppName, target) == 'true') ):
    		# WebSphere:cell=ibmmon44Node06Cell,node=ibmmon44Node02,server=server1
    		targetStringList = ((target.split(':'))[1]).split(',')
    		print "Current target: ", targetStringList
    		cellName = targetStringList[0].split('=')[1]
    		print "Configure REST Services Gateway on cell: ", cellName
    		if len(targetStringList) == 3:
    			nodeName = targetStringList[1].split('=')[1]
    			serverName = targetStringList[2].split('=')[1]
    			print "Configure REST Services Gateway on Node: ", nodeName, " ,Server: " , serverName
    			AdminTask.updateRESTGatewayService(['-nodeName', nodeName, '-serverName', serverName, '-enable', 'true'])
    			AdminConfig.save()
    		elif len(targetStringList) == 2:
    			clusterName = targetStringList[1].split('=')[1]
    			print "Configure REST Services Gateway on cluster: " , clusterName
    			AdminTask.updateRESTGatewayService(['-clusterName', clusterName, '-enable', 'true'])
    			AdminConfig.save()
    		else:
    			print "The following target is not valid: ", targetStringList

    print "migrated REST services configuration";
    print ""
#endDef


def getRESTDeploymentTargets():
    print ""
    
    restDeploymentTargetsList = wsadminToList(AdminConfig.list("WBIRestDeploymentTarget"));
    
    print "The value of restDeploymentTargetsList is ", restDeploymentTargetsList
    print ""
    return  restDeploymentTargetsList;  
#endDef

def getRestWebModules(restDeploymentTarget):
    print ""

    webModulesList = wsadminToList(AdminConfig.list("WBIRestWebModule", restDeploymentTarget));
    
    print "The value of webModulesList is ", webModulesList
    print ""
    return  webModulesList; 
#endDef

def registerDefaultRESTServices(businessSpaceClusterName, businessSpaceNodeName, businessSpaceServerName):
    print ""
    
    print "The value of businessSpaceClusterName is", businessSpaceClusterName;
    print "The value of businessSpaceNodeName is", businessSpaceNodeName;
    print "The value of businessSpaceServerName is", businessSpaceServerName;
    
    bfmRestAlreadyRegistered = "False";
    taskRestAlreadyRegistered = "False";
    
    restDeploymentTargetsList = getRESTDeploymentTargets();
    
    for restDeploymentTarget in restDeploymentTargetsList:
        contextID = AdminConfig.showAttribute(restDeploymentTarget, "contextID");
        print ""
        print "The current deployment target context ID is ", contextID 
        contextIDValues = contextID.split(":");
                
        webModulesList = getRestWebModules(restDeploymentTarget);
                
        for webModule in webModulesList:
            print ""
            
            currentWebModuleName = AdminConfig.showAttribute(webModule, "name");
            print "The value of currentWebModuleName is", currentWebModuleName 
        
            applicationName = AdminConfig.showAttribute(webModule, "applicationName");
            print "The value of applicationName is ", applicationName
                                            
            default = AdminConfig.showAttribute(webModule, "default");
            print "The value of default is ", default
            
            if not (default == None) and (default == "true"):
                print ""
                
                if (currentWebModuleName == "bfmrestapi.war"):
                    if (bfmRestAlreadyRegistered == "True"):
                        continue
                    else:
                        bfmRestAlreadyRegistered = "True"
                                        
                if (currentWebModuleName == "taskrestapi.war"):
                    if (taskRestAlreadyRegistered == "True"):
                        continue
                    else:
                        taskRestAlreadyRegistered = "True"


                if len(contextIDValues) == 6:               
                    nodeName = contextIDValues[3];
                    serverName = contextIDValues[5];                    
                    print "The value of nodeName is ", nodeName, " - serverName is ", serverName
                    
                    print "Registering the REST service endpoints."
                    if (isEmptyString(businessSpaceClusterName) == "False"):
                        status = AdminTask.registerRESTServiceEndpoint(['-businessSpaceClusterName', businessSpaceClusterName, '-nodeName', nodeName, '-serverName', serverName])
                        print status;
                    else:
                        status = AdminTask.registerRESTServiceEndpoint(['-businessSpaceNodeName', businessSpaceNodeName, '-businessSpaceServerName', businessSpaceServerName, '-nodeName', nodeName, '-serverName', serverName])
                        print status;
                    
                elif len(contextIDValues) == 4:
                    clusterName = contextIDValues[3];
                    print "The value of clusterName is ", clusterName
                    
                    print "Registering the REST service endpoints."
                    if (isEmptyString(businessSpaceClusterName) == "False"):
                        status = AdminTask.registerRESTServiceEndpoint(['-businessSpaceClusterName', businessSpaceClusterName, '-clusterName', clusterName])
                        print status;
                    else:
                        status = AdminTask.registerRESTServiceEndpoint(['-businessSpaceNodeName', businessSpaceNodeName, '-businessSpaceServerName', businessSpaceServerName, '-clusterName', clusterName])
                        print status;
                    
                else:
                    print ""
                    print "The following context ID is not valid : ", contextID
                    continue
    print ""   
#endDef

def isEmptyString(value):
    stringValue = String(value);                            
    if (stringValue.length() != 0):
        return "False";
    else:
        return "True";
#endDef

def getCellContext():
    print ""
    cellContext = ""
    
    try:
        cellContext =  "/Cell:" + System.getProperty("local.cell");        
    except:
        print "Cannot get the cell context. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value      
    
    print "The value of cellContext is ", cellContext
    print ""
    return cellContext
#endDef

def getNodeContext(userContext):
    print ""
    nodeContext = ""
    
    if (userContext == "use-local-server"):
        nodeName = System.getProperty("local.node");
    else:
        nodeName = getNamedValue(userContext, "nodes");
    
    if (isEmptyString(nodeName) == "False"):
        nodeContext =  "/Node:" + nodeName;
    
    print "The value of nodeContext is ", nodeContext
    print ""
    return nodeContext
#endDef

def getNamedValue(nameValuePairs, name):
    value = "";
    
    nameValuePairsArray = nameValuePairs.split(":");
    
    index = 0;
    while index < len(nameValuePairsArray):
        if nameValuePairsArray[index] == name and ((index + 1) < len(nameValuePairsArray)):
            value = nameValuePairsArray[index + 1];
            break;
        else:
            index = index + 2; 
    
    return value;
#endDef

def restServicesRegistrationMigration(cluster):
	try:
	        if (cluster != ''):
	                registerDefaultRESTServices(cluster, "", "");        
	                AdminConfig.save();                                        
	        else:        
	            print "Migrating the distributed custom profile"
	            
	            nodeID = AdminConfig.getid(getCellContext() + getNodeContext("use-local-server"));            
	            print ""
	            print "The value of nodeID is ", nodeID;
	
	            servers = wsadminToList(AdminConfig.list("Server", nodeID));
	            
	            for server in servers:
	                serverType = AdminConfig.showAttribute(server, "serverType");
	                print ""
	                print "The value of serverType is ", serverType
	                
	                if (serverType == "APPLICATION_SERVER"):
	                    serverName = AdminConfig.showAttribute(server, "name");
	                    print "The value of serverName is ", serverName
	                    
	                    clusterName = AdminConfig.showAttribute(server, "clusterName");
	                    print "The value of clusterName is ", clusterName
	                    
	                    if (not (clusterName)) or (isEmptyString(clusterName) == "True"):
	                        print "Migrating server ", serverName, " on Node ", System.getProperty("local.node");
	                        
	                        try:
	                            registerDefaultRESTServices("", System.getProperty("local.node"), serverName);
	                            
	                            AdminConfig.save();
	                        except:
	                            print "Cannot migrate REST services configuration. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value                        
	                    else:
	                        print "Skipping the migration for cluster members"             
	except:
	    print "Cannot migrate REST services registration. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value 

#..............................................................................
# End: Update rest service application, methods
#..............................................................................

#.............................................................................	
# Update the jdbc driver location from universalDriver.wbm\lib to jdbcdrivers\DB2
#..............................................................................	
def updateJDBCDriverLocation(cell,node):
	print "\n**************** updateJDBCDriverLocation() start ****************"
	if(node == None):
		print 'cell: ' + cell
	else:
		print "cell: " + cell + " node: " + node

	try:			
		if(node == None):
			cellId = AdminConfig.getid('/Cell:' + cell + '/')
			varSubstitutions = AdminConfig.list('VariableSubstitutionEntry',cellId).split(java.lang.System.getProperty('line.separator'))
		else:
			nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
			varSubstitutions = AdminConfig.list('VariableSubstitutionEntry',nodeId).split(java.lang.System.getProperty('line.separator'))
	
		nodeVarStart = '(cells/' + cell + '/nodes/'
		varList = []
		for varSubst in varSubstitutions:
			if (node == None):
				if not(varSubst.startswith(nodeVarStart)):
					varList.append(varSubst)				
			else:
				if varSubst.startswith(nodeVarStart):
					varList.append(varSubst)
			
		wasInsVarName = 'WAS_INSTALL_ROOT'
		for varSubst in varList:		
			getVarName = AdminConfig.showAttribute(varSubst, 'symbolicName')
			if (getVarName == wasInsVarName):
				wasInsVarValue = AdminConfig.showAttribute(varSubst, 'value')
				break
		if (wasInsVarValue == None):
			wasInsVarValue = getWASHome()
		wasInsVarValue = wasInsVarValue.replace('\\', '/')
		if not(wasInsVarValue.endswith('/')):
			wasInsVarValue = wasInsVarValue + '/'
			
		oldLocation = []
		oldLocation.append('${WAS_INSTALL_ROOT}/universalDriver.wbm/lib')
		oldLocation.append('${WAS_INSTALL_ROOT}\\universalDriver.wbm\\lib')
		oldLocation.append(wasInsVarValue + 'universalDriver.wbm/lib')
		oldLocation.append(wasInsVarValue.replace('/', '\\') + 'universalDriver.wbm\\lib')
		
		newLocation = []
		newLocation.append('${WAS_INSTALL_ROOT}/jdbcdrivers/DB2')
		newLocation.append('${WAS_INSTALL_ROOT}/jdbcdrivers/DB2')
		newLocation.append(wasInsVarValue + 'jdbcdrivers/DB2')
		newLocation.append(wasInsVarValue + 'jdbcdrivers/DB2')	
		
		for varSubst in varList:		
			getVarValue = AdminConfig.showAttribute(varSubst, 'value')
			getVarName = AdminConfig.showAttribute(varSubst, 'symbolicName')			
			index = 0
			for varValue in oldLocation:
				if (getVarValue == varValue):
					AdminConfig.modify(varSubst,[['value', newLocation[index]]])
					print 'modify variable: ' + getVarName + varSubst
					print 'old location: ' + getVarValue + ' new location: ' + newLocation[index]
					break
				index = index + 1
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error updating jdbc driver location"
	else:
		AdminConfig.save()
	
	print "**************** updateJDBCDriverLocation() end ****************"
#endDef

#.............................................................................	
# Update the old library of abx poi-2.5.1-final-20040804.jar to poi-3.1-FINAL-20080629.jar
#..............................................................................	
def updateAbxLibrary(cell,node):
	print "\n**************** updateAbxLibrary() start ****************"
	print "cell: " + cell + " node: " + node

	try:			
		nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
		libList = AdminConfig.list('Library',nodeId).split(java.lang.System.getProperty('line.separator'))	
		for lib in libList:		
			libName = AdminConfig.showAttribute(lib, 'name')			
			if (libName == 'abx_poi-2.5.1-final-20040804'):
				print 'modify library: ' + lib
				AdminConfig.modify(lib,[['classPath', '']])
				AdminConfig.modify(lib,[['classPath', '${ABX_INSTALL_DIR}/lib/poi-3.1-FINAL-20080629.jar']])
				print 'new classpath: ' + '${ABX_INSTALL_DIR}/lib/poi-3.1-FINAL-20080629.jar'			
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error updating alphablox library"
	else:
		AdminConfig.save()
	
	print "**************** updateAbxLibrary() end ****************"
#endDef

#.............................................................................	
# remove the old library of abx as ABX is removed from Monitor V8.0 and later
#..............................................................................	
def removeAbxLibrary(cell,node):
	print "\n**************** removeAbxLibrary() start ****************"
	print "cell: " + cell + " node: " + node

	try:		
		#Remove ABX lib from cell scope
		nodeId = AdminConfig.getid('/Cell:' + cell)
		libList = AdminConfig.list('Library',nodeId).split(java.lang.System.getProperty('line.separator'))	
		for lib in libList:				
			if (lib.startswith('abx_') or lib.startswith('ABX_')):
				print 'removing ABX library: ' + lib
				AdminConfig.remove(lib)	
		#Remove ABX lib from node scope
		nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
		libList = AdminConfig.list('Library',nodeId).split(java.lang.System.getProperty('line.separator'))	
		for lib in libList:				
			if (lib.startswith('abx_') or lib.startswith('ABX_')):
				print 'removing ABX library: ' + lib
				AdminConfig.remove(lib)							
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error removing alphablox library"
	else:
		AdminConfig.save()
	
	print "**************** removeAbxLibrary() end ****************"
#endDef

#.............................................................................	
# remove the cognos service for standalone server
#..............................................................................	
def removeCognosService(cell,node,server,cluster):
	print "\n**************** removeCognosService() start ****************"
	if(cluster == None):
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster
	
	try:
		if(cluster == None):
			AdminTask.wbmRemoveCognosService('[-node ' + node + ' -server ' + server + ']')
		else:
			AdminTask.wbmRemoveCognosService('[-cluster ' + cluster + ']')
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error calling admin task wbmDeployCognosService"
	else:
		AdminConfig.save()	
	print "End of admin task wbmRemoveCognosService call"
	
	print "**************** removeCognosService() end ****************"
#endDef


#.............................................................................	
# Deploy the cognos service
#..............................................................................	
def deployCognosService(cell,node,server,cluster):
	print "\n**************** deployCognosService() start ****************"
	if(cluster == None):
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster
		
	try:
		if(cluster == None):
			AdminTask.wbmDeployCognosService('[-node ' + node + ' -server ' + server + ' -dbName ' + cognosProperties['dbName'] + ' -dbUserName ' + cognosProperties['dbUserName'] +' -dbUserPassword ' + cognosProperties['dbUserPassword'] +' -adminUserName ' + cognosProperties['adminUserName'] +' -adminPassword ' + cognosProperties['adminPassword'] +']')
		else:
			AdminTask.wbmDeployCognosService('[-cluster ' + cluster + ' -dbName ' + cognosProperties['dbName'] + ' -dbUserName ' + cognosProperties['dbUserName'] +' -dbUserPassword ' + cognosProperties['dbUserPassword'] +' -adminUserName ' + cognosProperties['adminUserName'] +' -adminPassword ' + cognosProperties['adminPassword']  +']')
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error calling admin task wbmDeployCognosService"
	else:
		AdminConfig.save()
	print "End of admin task wbmDeployCognosService call"
	
	print "**************** deployCognosService() end ****************"
#endDef


#.............................................................................	
# update the alphablox application
#..............................................................................	
def updateAbxApp(cell, node, server, cluster):
	print "\n**************** updateAbxApp() start ****************"
	if(cluster == None):
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster

	# updata ABX related application and configuration
	alphabloxPlatformApp = 'AlphabloxPlatform'
	alphabloxStudioApp = 'ApplicationStudio'
	abxFlag = 0
	# check AlphabloxPlatform app, if exist, uninstall it, and set abx flag to ture, we will redeploy it.
	if ( checkApplicationExist(alphabloxPlatformApp) == 0 ):
		print "\nOld app " + alphabloxPlatformApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxPlatformApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxPlatformApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxPlatformApp)
	
	# check AlphabloxStudio app, if exist, uninstall it
	if ( checkApplicationExist(alphabloxStudioApp) == 0 ):
		print "Old app " + alphabloxStudioApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxStudioApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxStudioApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxStudioApp)
	
	# use ABX admin task to configure ABX on cluster
	if (abxFlag == 0):
		if(cluster == None):
			print "\nABX applications not exist, or ABX applications not mapped to server " + server
		else:
			print "\nABX applications not exist, or ABX applications not mapped to cluster " + cluster
	else:
		print "calling admin task wbmMigrateAlphabloxService..."
		try:
			if(cluster == None):
				AdminTask.wbmMigrateAlphabloxService('[-node ' + node + ' -server ' + server + ']')
			else:
				AdminTask.wbmMigrateAlphabloxService('[-cluster ' + cluster +']')
		except:
			rc = -1
			type, value, traceback = sys.exc_info()
			print type
			print value
			print traceback
			print "Error calling admin task wbmMigrateAlphabloxService"
		else:
			AdminConfig.save()
		print "End of admin task wbmMigrateAlphabloxSevice call"
	
		# update the abx endpoint	
		if(cluster == None):
			updateAbxEndpoint(node, server, None)
		else:
			updateAbxEndpoint(None, None, cluster)
	
	print "**************** updateAbxApp() end ****************"
#endDef

#################################################################################################
#remove ABX in source env if existed, and check if we need to deploy cognos service
##################################################################################################
def removeAbxAppAndDeployCognosService(cell, node, server, cluster):
	print "\n**************** removeAbxAppAndDeployCognosService() start ****************"
	if(cluster == None):
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster

	# updata ABX related application and configuration
	alphabloxPlatformApp = 'AlphabloxPlatform'
	alphabloxStudioApp = 'ApplicationStudio'
	abxFlag = 0
	# check AlphabloxPlatform app, if exist, uninstall it, and set abx flag to ture, we will redeploy it.
	if ( checkApplicationExist(alphabloxPlatformApp) == 0 ):
		print "\nOld app " + alphabloxPlatformApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxPlatformApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxPlatformApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxPlatformApp)
	
	# check AlphabloxStudio app, if exist, uninstall it
	if ( checkApplicationExist(alphabloxStudioApp) == 0 ):
		print "Old app " + alphabloxStudioApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxStudioApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxStudioApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxStudioApp)
	
	# use Cognos admin task to configure Cognos Service on cluster
	if (abxFlag == 0):
		if(cluster == None):
			print "\nABX applications not exist, or ABX applications not mapped to server " + server
		else:
			print "\nABX applications not exist, or ABX applications not mapped to cluster " + cluster
		#In the case there is no ABX deployed on current server/cluster in source env and no cognos service deployed in the cell
		#If there is pre-defined cognos config and set location where to deploy cognos explicitly, and the setted location is 
		#current server/cluster, then we will deploy cognos service here, otherwise, don't deploy cognos
		if(cognosProperties != None and checkApplicationExist('IBM Cognos') == 0 and checkNeedCognosDeploymentByLocation(cluster,node,server) == 1):
			print "Cognos pre-defined config found and set cognos deployment location, will deploy cognos."
			deployCognosService(cell, node, server, cluster)
		else:
			print "Cognos pre-defined config not found or not set cognos deployment location, and ABX not deployed on the node, so will not deploy cognos."
	else:
		#In the case there is ABX deployed on current server/cluster in source env, and no cognos service deployed in the cell
		#1.If there is pre-defined cognos config and set location where to deploy cognos explicitly, and the setted location is 
		#current server/cluster, then we will deploy cognos service here. 
		#2.If user didn't set location where to deploy cognos service in pre-defined cognos config,we will deploy cognos here
		if (cognosProperties != None and checkApplicationExist('IBM Cognos') == 0):
			if(ifSetCognosDeploymentLocation() ==0):
				print "Will deploy cognos servive where Alphablox was deployed."
				deployCognosService(cell, node, server, cluster)
			elif(checkNeedCognosDeploymentByLocation(cluster,node,server) == 1):
				print "Will deploy cognos servive based on pre-defined cognos config."
				deployCognosService(cell, node, server, cluster)
		else:
			print "No congos config pre-defined or Cognos has been deployed, will not deploy cognos service"
	
	
	
	print "**************** removeAbxAppAndDeployCognosService() end ****************"
#endDef

def removeAbxApp(cell, node, server, cluster):
	print "\n**************** removeAbxApp() start ****************"
	if(cluster == None):
		print 'cell: ' + cell + ' node: ' + node + ' server: ' + server
	else:
		print 'cell: ' + cell + ' cluster: ' + cluster

	# updata ABX related application and configuration
	alphabloxPlatformApp = 'AlphabloxPlatform'
	alphabloxStudioApp = 'ApplicationStudio'
	abxFlag = 0
	# check AlphabloxPlatform app, if exist, uninstall it, and set abx flag to ture, we will redeploy it.
	if ( checkApplicationExist(alphabloxPlatformApp) == 0 ):
		print "\nOld app " + alphabloxPlatformApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxPlatformApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxPlatformApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxPlatformApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxPlatformApp)
	
	# check AlphabloxStudio app, if exist, uninstall it
	if ( checkApplicationExist(alphabloxStudioApp) == 0 ):
		print "Old app " + alphabloxStudioApp + " not exist"
	elif (cluster == None) and (checkOldAppMapping(alphabloxStudioApp,cell,node,server,None) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on server " + server + " not exist"
	elif (not(cluster == None)) and (checkOldAppMapping(alphabloxStudioApp,cell,None,None,cluster) == 0):
		print "Old mapping for " + alphabloxStudioApp + " on cluster " + cluster + " not exist"
	else:
		abxFlag = 1
		uninstallApplication(alphabloxStudioApp)
	
	
	print "**************** removeAbxApp() end ****************"
#endDef

#.............................................................................	
# update the alphablox endpoint
#..............................................................................	
def updateAbxEndpoint(node, server, cluster):
	print "\n**************** updateAbxEndpoint() start ****************"
	if(cluster == None):
		print 'node: ' + node + ' server: ' + server
	else:
		print 'cluster: ' + cluster

	try:
		wasInstall = getWASHome()
		wasInstall = wasInstall.replace('\\', '/')
		if not(wasInstall.endswith('/')):
			wasInstall = wasInstall + '/'
		abxEndpoint = wasInstall + 'Alphablox/endpoints'
		if (cluster == None):
			AdminTask.updateEndpointBinding('[-nodeName ' + node + ' -serverName ' + server + ' -endpointBindingDirectoryName "' + abxEndpoint + '" -clean false -save false ]')
		else:
			AdminTask.updateEndpointBinding('[-clusterName ' + cluster + ' -endpointBindingDirectoryName "' + abxEndpoint + '" -clean false -save false ]')
	except:
		rc = -1
		type, value, traceback = sys.exc_info()
		print type
		print value
		print traceback
		print "Error calling admin task updateEndpointBinding"
	else:
		AdminConfig.save()
	print "End of admin task updateEndpointBinding call"
	
	print "**************** updateAbxEndpoint() end ****************"
#endDef

#.............................................................................	
# Update applications for Monitor profile 6.x to 8.0 migration
#..............................................................................	
def updateWBMProfile():
	print "\n**************** updateWBMProfile() start ****************"
	# update application list
	# IBM_WBM_DATA_SERVICES, V6.2
	# IBM_WBM_MOBILE_DASHBOARD, V6.1.2, V6.2+
	# IBM_WBM_REST_SERVICES, V6.1 - V6.2+
	# IBM_WBM_ACTIONSERVICES, V6.1 - V6.2+
	# IBM_WBM_MSS_SERVICE, ignored, only in stand-alone env
	# IBM_WBM_EMITTER_REST_SERVICES, V6.2, name changed in V7: IBM_WBM_EMITTER_SERVICES, reinstall.
	# ABX applications: AlphabloxPlatform, AlphabloxStudio, by ABX admin task
	updateMonAppList = ['IBM_WBM_DATA_SERVICES', 'IBM_WBM_MOBILE_DASHBOARD', 'IBM_WBM_ACTIONSERVICES']
	earFileNames = Properties()
	earFileNames.put('IBM_WBM_DATA_SERVICES', 'MonitorDataServices.ear')
	earFileNames.put('IBM_WBM_MOBILE_DASHBOARD', 'MobileDashboard.ear')
	earFileNames.put('IBM_WBM_ACTIONSERVICES', 'monactionmgr.ear')
	earFileNames.put('IBM_WBM_EMITTER_SERVICES', 'EmitterServices.ear')	
	
	# to store app name + module name, and the old server mappings
	#  format: ( 'appName',  [   [modulename1, uri, servermapping] , [modulename2, uri, servermapping]   ]  )
	appModuleMapping = []
	appModuleMapping.append([ 'IBM_WBM_DATA_SERVICES', [['DataServicesSchedulerEJB', 'DataServicesSchedulerEJB.jar,META-INF/ejb-jar.xml', '']] ])
	appModuleMapping.append(['IBM_WBM_MOBILE_DASHBOARD', [['MobileDashboard', 'MobileDashboard.war,WEB-INF/web.xml', '']]])
	appModuleMapping.append(['IBM_WBM_ACTIONSERVICES', [ \
		['Action Manager BootStrap', 'monAAMBootstrap.jar,META-INF/ejb-jar.xml', ''], \
		['Action Manager Dispatcher', 'monAAMMDB.jar,META-INF/ejb-jar.xml', ''] ]])
	appModuleMapping.append(['IBM_WBM_EMITTER_REST_SERVICES', [ ['eventEmitter.war', 'eventEmitter.war,WEB-INF/web.xml', ''] ]])
	if (ifRemoveABX != 'YES'):		
		appModuleMapping.append(['AlphabloxPlatform', [ \
			['IBM Alphablox Administration Application', 'AlphabloxAdmin.war,WEB-INF/web.xml', ''], \
			['IBM Alphablox', 'AlphabloxServer.war,WEB-INF/web.xml', ''], \
			['IBM Alphablox Blox Builder Web Service', 'AlphabloxTooling.war,WEB-INF/web.xml', ''], \
			['IBM Alphablox BloxBuilder Application Viewer', 'BloxBuilder.war,WEB-INF/web.xml', ''], \
			['Dashboard.ABX', 'WBMDashboardABX.war,WEB-INF/web.xml', ''] ]])
		appModuleMapping.append(['ApplicationStudio', [ \
			['Alphablox Reporting', 'Examples/AlphabloxReporting.war,WEB-INF/web.xml', ''], \
			['Blox Sampler', 'Examples/BloxSampler.war,WEB-INF/web.xml', ''], \
			['emailexample', 'Examples/EMail.war,WEB-INF/web.xml', ''], \
			['IBM Alphablox Query Builder', 'Workbench/DHTMLQueryBuilder.war,WEB-INF/web.xml', ''], \
			['IBM Alphablox FastForward', 'FastForward.war,WEB-INF/web.xml', ''] ]])
	
	# Initialization
	nodeType = getPropertyValue('bpm.upgrade.node.type')
	print 'The nodeType is: '+nodeType
	previousVersion = getPropertyValue('wbm.upgrade.previous.version')
	print 'Previous version is: '+previousVersion
	cell = getCell()
	print "\ncell is: " + cell
	node = getUpdatedNode();
	print "node is : " + node
	cluster = getUpdatedCluster()
	print "cluster is: " + cluster
	
	# update for dmgr and stand alone server
	if (nodeType == 'dmgr') and (cluster == ''):
		print "Deleting web server mappings..."
		deleteWebServerMapping(appModuleMapping)
		# update application REST Services Gateway
		migrateRESTGatewayServices()
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, None)
		updateJDBCDriverLocation(cell, node)
	elif (nodeType =='unmanaged'):
		# update appModuleMapping to V7 information, for stand alone server
		print "\nUpdate application module information for standalone server"
		updateAppModuleInfo(appModuleMapping)
		# delete web server mappings using updated app module mapping info
		print "\nDeleting web server mappings..."
		deleteWebServerMapping(appModuleMapping)
		# update for stand alone Monitor server only
		updateStandAloneProfile(cell, node)
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, None)
		updateJDBCDriverLocation(cell, node)
		# update the alphablox library
		#if (ifRemoveABX != 'YES'):
		#	updateAbxLibrary(cell, node)
		#else:
		#	removeAbxLibrary(cell, node)
		# remove the cognos service
		serverNameList = getServerList(cell,node)
		for server in serverNameList:			
			removeCognosService(cell, node, server, None)
			if (cognosProperties != None):
				print "calling deployCognosService in standalone migration..."
				deployCognosService(cell, node, server, None)	
			# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0
			if(showEventServiceStaus(cell, node, server, None) == 1 and IfNeedUpdateEventService == 1):				
				updateApplication(EventServiceAppName,EventServiceAppLocation)				
		# regenerate the rest service gateway configuration to use the correct port number
		removeRESTGatewayConfig()
		updateRESTGatewayApplication(cell, node, None, None)		
	elif (nodeType == 'managed'):
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, node)
		# update the alphablox library
		if (ifRemoveABX != 'YES'):
			updateAbxLibrary(cell, node)
		else:
			removeAbxLibrary(cell, node)
	
	# update datasource for V6.1.x to V7 migration
	if ( previousVersion.find('6.1') >= 0 ):
		if ( (nodeType == 'dmgr') and (cluster == '') ) or (nodeType == 'unmanaged'):
			updateDataSource(cell)
	
	# 1. check whether the app update procedure should be invoked
	whetherUpdateRequired = checkWhetherUpdateRequired()
	
	# 2. If update required, perform update
	if (whetherUpdateRequired == 0):
		# update appModuleMapping to V7 module/mappings
		updateModuleMappings(appModuleMapping)
		print "\nUpdated module mapping information: "
		print appModuleMapping

		installableAppsLocation = getInstallableAppsLocation(cell,node)
		print "installable apps location : " + installableAppsLocation
		
		#2.2 If federated node update is performed
		if(cluster == ''):
			# Migrate rest services registration
			restServicesRegistrationMigration(cluster);
			serverList = getServerList(cell,node)
			print "Upgrade server list:"
			print serverList
			for server in serverList:
				# reinstall 3 normal was app
				for appName in updateMonAppList:
					if (checkApplicationExist(appName)  == 0):
						print "\nOld app " + appName + " not exist"
					elif(checkOldAppMapping(appName,cell,node,server,None) == 0):
						print "Old mapping for " + appName + " on server " + server + " not exist"
					else:
						print "Old app " + appName + " exist on server " + server
						uninstallApplication(appName)
						deployOptions = getDeployOptions(appModuleMapping, appName)
						print 'Deploy options for application ' + appName
						print deployOptions
						installApplication(appName, installableAppsLocation + earFileNames.get(appName), cell, node, server, deployOptions)
						#updateApplication(appName, installableAppsLocation + earFileNames.get(appName))
				
				# uninstall and reinstall IBM_WBM_EMITTER_REST_SERVICES, to IBM_WBM_EMITTER_SERVICES
				oldEmitterAppName = 'IBM_WBM_EMITTER_REST_SERVICES'
				newEmitterAppName = 'IBM_WBM_EMITTER_SERVICES'
				if ( checkApplicationExist(oldEmitterAppName) == 0 ):
					print "\nOld app " + oldEmitterAppName + " not exist"
				elif(checkOldAppMapping(oldEmitterAppName,cell,node,server,None) == 0):
					print "Old mapping for " + oldEmitterAppName + " on server " + server + " not exist"
				else:
					print "Old app " + oldEmitterAppName + " exist on server " + server
					uninstallApplication(oldEmitterAppName)
					deployOptions = getDeployOptions(appModuleMapping, newEmitterAppName)
					print 'Deploy options for application ' + newEmitterAppName
					print deployOptions
					installApplication(newEmitterAppName, installableAppsLocation + earFileNames.get('IBM_WBM_EMITTER_SERVICES'), cell, node, server, deployOptions)
					# set the AllAuthenticated Security roll for the application
					roll="eventemitters"
					rc3 = setAllAuthenticated(newEmitterAppName, roll)
					if(rc3 == -1):
						rc = -1
					#try:
						#AdminTask.wbmDeployEventEmitterServices('[-node ' + node + ' -server ' + server + ']')
					#	print ""
					#except:
					#	rc = -1
					#	type, value, traceback = sys.exc_info()
					#	print type
					#	print value
					#	print traceback
					#	print "Error creating resources for Emitter rest services"				
					#else:
					#	AdminConfig.save()
					# create resources needed by Emitter rest service
					createEmitterRestResources(cell, cluster, node, server)
				
				# updata ABX related application and configuration
				if (ifRemoveABX != 'YES'):
					updateAbxApp(cell, node, server, None)
				else:
					removeAbxAppAndDeployCognosService(cell, node, server, None)
					removeAbxLibrary(cell, node)		
				# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0					
				if(showEventServiceStaus(cell, node, server, None) == 1 and IfNeedUpdateEventService == 1):
					updateApplication(EventServiceAppName,EventServiceAppLocation)		
		#2.3 If cluster update is performed
		else:			
			clusterID = AdminConfig.getid('/ServerCluster:'+cluster)
			memberListStr = AdminConfig.list('ClusterMember',clusterID)
			memberList = wsadminToList(memberListStr)
			
			# Migrate rest services registration
			restServicesRegistrationMigration(cluster);
			
			# reinstall 3 normal was app
			for appName in updateMonAppList:
				if (checkApplicationExist(appName)  == 0):
					print "\nOld app " + appName + " not exist"
				elif(checkOldAppMapping(appName,cell,None,None,cluster) == 0):
					print "Old mapping for " + appName + " on cluster " + cluster + " not exist"
				else:
					print "Old app " + appName + " exist on cluster " + cluster
					uninstallApplication(appName)
					deployOptions = getDeployOptions(appModuleMapping, appName)
					print 'Deploy options for application ' + appName
					print deployOptions
					installApplicationCluster(appName, installableAppsLocation + earFileNames.get(appName), cell, cluster, deployOptions)
					#updateApplication(appName, installableAppsLocation + earFileNames.get(appName))
			
			#uninstall and reinstall IBM_WBM_EMITTER_REST_SERVICES, to IBM_WBM_EMITTER_SERVICES
			oldEmitterAppName = 'IBM_WBM_EMITTER_REST_SERVICES'
			newEmitterAppName = 'IBM_WBM_EMITTER_SERVICES'
			if ( checkApplicationExist(oldEmitterAppName) == 0 ):
				print "\nOld app " + oldEmitterAppName + " not exist"
			elif(checkOldAppMapping(oldEmitterAppName,cell,None,None,cluster) == 0):
				print "Old mapping for " + oldEmitterAppName + " on cluster " + cluster + " not exist"
			else:
				print "Old app " + oldEmitterAppName + " exist on cluster " + cluster
				uninstallApplication(oldEmitterAppName)
				deployOptions = getDeployOptions(appModuleMapping, newEmitterAppName)
				print 'Deploy options for application ' + newEmitterAppName
				print deployOptions
				installApplicationCluster(newEmitterAppName, installableAppsLocation + earFileNames.get('IBM_WBM_EMITTER_SERVICES'), cell, cluster, deployOptions)
				# set the AllAuthenticated Security roll for the application
				roll="eventemitters"
				rc4 = setAllAuthenticated(newEmitterAppName, roll)
				if(rc4 == -1):
					rc = -1
				#try:
					#AdminTask.wbmDeployEventEmitterServices(['-cluster', cluster])
				#	print ""
				#except:
				#	rc = -1
				#	type, value, traceback = sys.exc_info()
				#	print type
				#	print value
				#	print traceback
				#	print "Error creating resources for Emitter rest services"
				#else:
				#	AdminConfig.save()
				# create resources needed by Emitter rest service
				createEmitterRestResources(cell, cluster, None, None)
			
			# updata ABX related application and configuration
			if (ifRemoveABX != 'YES'):
				updateAbxApp(cell, None, None, cluster)
			else:
				removeAbxAppAndDeployCognosService(cell, None, None, cluster)
				removeAbxLibrary(cell, node)	
			# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0
			if(showEventServiceStaus(cell, None, None, cluster) == 1 and IfNeedUpdateEventService == 1):
					updateApplication(EventServiceAppName,EventServiceAppLocation)		
	# 3. Otherwise update not required, do nothing
	else:
		print "Nothing need to be done for WBM application upgrade."
	print "**************** updateWBMProfile() end ****************"

#.............................................................................	
# Update applications for Monitor profile 7.0/7.5.x to 8.0 migration
#..............................................................................	
def updateWBMProfileFor7x():
	print "\n**************** updateWBMProfileFor7x() start ****************"
	# update application list	
	updateMonAppList = ['IBM_WBM_DATA_SERVICES', 'IBM_WBM_MOBILE_DASHBOARD', 'IBM_WBM_ACTIONSERVICES', 'IBM_WBM_EMITTER_SERVICES']	
	earFileNames = Properties()
	earFileNames.put('IBM_WBM_DATA_SERVICES', 'MonitorDataServices.ear')
	earFileNames.put('IBM_WBM_MOBILE_DASHBOARD', 'MobileDashboard.ear')
	earFileNames.put('IBM_WBM_ACTIONSERVICES', 'monactionmgr.ear')
	earFileNames.put('IBM_WBM_EMITTER_SERVICES', 'EmitterServices.ear')	

	if (previousVersionList.index(previousVersion[0:3]) >= previousVersionList.index('7.5')):
		updateMonAppList.append('IBM Cognos')
		
	# Initialization
	nodeType = getPropertyValue('bpm.upgrade.node.type')
	print 'The nodeType is: '+nodeType
	cell = getCell()
	print "\ncell is: " + cell
	node = getUpdatedNode();
	print "node is : " + node
	cluster = getUpdatedCluster()
	print "cluster is: " + cluster
	
	installableAppsLocation = getInstallableAppsLocation(cell,node)	
	print "\nInstallable applications location: " + installableAppsLocation
	
	CognosServiceDeployed = 'NO'
	
	if (nodeType == 'dmgr') and (cluster == ''):
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, None)
		updateJDBCDriverLocation(cell, node)
	if (nodeType == 'unmanaged'):
		print "\nUpdate application for standalone server"
		for appName in updateMonAppList:
			if (checkApplicationExist(appName)  == 0):
				print "Old app " + appName + " not exist"
			else:
				print "Update app " + appName
				if (appName != 'IBM Cognos'):
					updateApplication(appName, installableAppsLocation + earFileNames.get(appName))
		# update for stand alone Monitor server only
		updateStandAloneProfile(cell, node)
		# create the resources needed by Lombardi Emitter services
		createLombardiEmitterResources(cell, node)
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, None)
		updateJDBCDriverLocation(cell, node)
		# remove the cognos service
		serverNameList = getServerList(cell,node)
		for server in serverNameList:			
			removeCognosService(cell, node, server, None)
			if (cognosProperties != None):
				print "calling deployCognosService in standalone migration..."
				deployCognosService(cell, node, server, None)	
			# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0
			if(showEventServiceStaus(cell, node, server, None) == 1 and IfNeedUpdateEventService == 1):
				updateApplication(EventServiceAppName,EventServiceAppLocation)	
		# Update rest gateway application
		updateRESTGatewayApplication(cell, node, None, None)		
	elif (nodeType == 'managed'):
		print "\nUpdate application for managed node " + node
		serverList = getServerList(cell,node)
		print "Upgrade server list:"
		print serverList
		for server in serverList:
			for appName in updateMonAppList:
				if (checkApplicationExist(appName)  == 0):
					print "Old app " + appName + " not exist"
				elif(checkOldAppMapping(appName,cell,node,server,None) == 0):
					print "Old mapping for " + appName + " on server " + server + " not exist"						
				else:
					print "Update app " + appName
					if (appName == 'IBM_WBM_EMITTER_SERVICES'):
						createEmitterFactoryForJMS(cell)
					if (appName == 'IBM Cognos'):
						#If cognos service was deployed on current server, we need to check if user set location
						#where to deploy cognos explicitly in pre-defined cognos config.
						#If user didn't set location, uninstall old cognos app and redeploy cognos service
						#If user set location and the location is current server, uninstall old cognos app and redeploy cognos service
						#If user set location but the location is not current server, only uninstall old cognos app
						print "Cognos service has been deployed here in source env, will check if need to redeploy it here."
						if(cognosProperties == None):
							print "No pre-defined cognos config found, only remove cognos service deployed in old env."
							#uninstallApplication(appName)
							removeCognosService(cell, node, server, None)
						elif(ifSetCognosDeploymentLocation() == 0 or (ifSetCognosDeploymentLocation() == 1 and checkNeedCognosDeploymentByLocation(None,node,server) == 1)):
							print "Uninstall current Cognos App and redeploy Cognos Service via admin command."
							#uninstallApplication(appName)
							removeCognosService(cell, node, server, None)
							deployCognosService(cell, node, server, None)
							CognosServiceDeployed = 'YES'
						else:
							print "Will not deploy Cognos service here as pre-defined cognos config define another location, only remove cognos service deployed in old env."
							#uninstallApplication(appName)
							removeCognosService(cell, node, server, None)
							
					else:
						updateApplication(appName, installableAppsLocation + earFileNames.get(appName))	
					
			
			# updata ABX related application and configuration
			if (ifRemoveABX != 'YES'):
				updateAbxApp(cell, node, server, None)
			elif (CognosServiceDeployed != 'YES'):
				#Try to remove abx and redeploy cognos. For V7.0-V8.0 path
				removeAbxAppAndDeployCognosService(cell, node, server, None)
				removeAbxLibrary(cell, node)
			else:
				removeAbxApp(cell, node, server, None)
				removeAbxLibrary(cell, node)
			# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0
			if(showEventServiceStaus(cell, node, server, None) == 1 and IfNeedUpdateEventService == 1):
				updateApplication(EventServiceAppName,EventServiceAppLocation)
			# Update rest gateway application
			updateRESTGatewayApplication(cell, node, server, None)
			
		# update the db2 jdbc driver location
		updateJDBCDriverLocation(cell, node)
	elif (cluster != ''):
		print "\nUpdate application for cluster " + cluster 
		for appName in updateMonAppList:
			if (checkApplicationExist(appName)  == 0):
				print "Old app " + appName + " not exist"
			elif(checkOldAppMapping(appName,cell,None,None,cluster) == 0):
				print "Old mapping for " + appName + " on cluster " + cluster + " not exist"
			else:
				print "Update app " + appName
				if (appName == 'IBM_WBM_EMITTER_SERVICES'):
					createEmitterFactoryForJMS(cell)
				if (appName == 'IBM Cognos'):
					#If cognos service was deployed on current cluster, we need to check if user set location
					#where to deploy cognos explicitly in pre-defined cognos config.
					#If user didn't set location, uninstall old cognos app and redeploy cognos service
					#If user set location and the location is current cluster, uninstall old cognos app and redeploy cognos service
					#If user set location but the location is not current cluster, only uninstall old cognos app
					print "Cognos service has been deployed here in source env, will check if need to redeploy it here."
					if(cognosProperties == None):
						print "No pre-defined cognos config found, only remove cognos service deployed in old env."
						#uninstallApplication(appName)
						removeCognosService(cell, None, None, cluster)
					elif(ifSetCognosDeploymentLocation() == 0 or (ifSetCognosDeploymentLocation() == 1 and checkNeedCognosDeploymentByLocation(cluster,None,None) == 1)):
						print "Uninstall current Cognos App and redeploy Cognos Service via admin command."
						#uninstallApplication(appName)
						removeCognosService(cell, None, None, cluster)
						deployCognosService(cell, None, None, cluster)
						CognosServiceDeployed = 'YES'
					else:
						print "Will not deploy Cognos service here as pre-defined cognos config define another location, only remove cognos service deployed in old env."
						#uninstallApplication(appName)
						removeCognosService(cell, None, None, cluster)
				else:
					updateApplication(appName, installableAppsLocation + earFileNames.get(appName))
				
		# updata ABX related application and configuration
		if (ifRemoveABX != 'YES'):
			updateAbxApp(cell, None, None, cluster)
		elif (CognosServiceDeployed != 'YES'):
			#Try to remove abx and redeploy cognos. For V7.0-V8.0 path
			removeAbxAppAndDeployCognosService(cell, None, None, cluster)
			removeAbxLibrary(cell, node)
		else:
			removeAbxApp(cell, None, None, cluster)
			removeAbxLibrary(cell, node)
		# Update Event Service App as event service migrated from 6.2 and 7.0 couldn't work on v8.0
		if(showEventServiceStaus(cell, None, None, cluster) == 1 and IfNeedUpdateEventService == 1):
				updateApplication(EventServiceAppName,EventServiceAppLocation)
		# Update rest gateway application
		updateRESTGatewayApplication(cell, None, None, cluster)
		
	else:
		print "Nothing need to be done for WBM application upgrade."
	print "**************** updateWBMProfileFor7x() end ****************"
	

#main 

previousVersionList = ['6.0','6.1','6.2','7.0','7.5']
previousVersion = getPropertyValue('wbm.upgrade.previous.version')
print 'Previous version is: '+ previousVersion
installDir = getWASHome()
print "\nThe install dir is : " + installDir

profileType = getPropertyValue('bpm.upgrade.profile.type')
print "Profile Type is : " + profileType

profileName = getPropertyValue('bpm.upgrade.previous.profileName')
print "Profile Name is : " + profileName

ifRemoveABX = 'YES'

cognosProperties = getCognosPropertiesFromFile()
printCognosProperties()
	
EventServiceAppName = 'EventService'
EventServiceAppLocation = installDir + '/systemApps/EventService.ear'

IfNeedUpdateEventService = 0

if (previousVersionList.index(previousVersion[0:3]) <= previousVersionList.index('7.0')):
	IfNeedUpdateEventService = 1

if (profileType.find('wbmonitor') >= 0):
	print "It is a Monitor profile migration"
	#previousVersion = getPropertyValue('wbm.upgrade.previous.version')	
	#print 'Previous version is: '+ previousVersion
	if (previousVersionList.index(previousVersion[0:3]) > previousVersionList.index('6.2')):
		updateWBMProfileFor7x()
	else:
		updateWBMProfile()	
else:
	print "It is not a Monitor profile migration"